#ifndef lint
static	char sccsid[] = "@(#)dev_gp1_create.c 2.1 87/10/28 SMI";
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc.
 */



#include "dev_gp1_internal.h"



typedef	struct {
	    Pixwin	*pw ;
	    int		clipid ;
	} Clipinfo ;

static	void	pw_clipfunc() ;
static	void	pw_maxmask() ;
static	void	pw_putcmap() ;
static	void	pw_wmask() ;
static	void	pw_cmsname() ;
static	void	pw_dolock() ;
static	void	pw_dounlock() ;


/*
 * Allocate a gp1_handle and get a static block.
 * Initialize everything...
 */

GP1_handle
Dev_gp1_create(pw, n_buffers, error_num)
	Pixwin	*pw;		/* pixwin  */
	int	n_buffers;	/* number of buffers to allocate at once. */
	int	*error_num;	/* if an error, return the error number */
{
register GP1	*gp1 ;
register Clipinfo	*clipinfo ;

	gp1 = dev_gp1_int_open(pw->pw_clipdata->pwcd_prmulti,
		n_buffers, error_num) ;

	if(gp1)
	{
	    gp1->pw = pw ;	/* for backwards compatibility */
	    gp1->clipfunc = pw_clipfunc ;
	    gp1->nclip = 0 ;
	    gp1->cliplist = (Rect *) malloc(GP_MAXCLIP * sizeof(Rect)) ;
	    gp1->flags |= GP_RESIZE ;
	    clipinfo = (Clipinfo *) malloc(sizeof(Clipinfo)) ;
		gp1->clippriv = (caddr_t) clipinfo ;
		clipinfo->pw = pw ;
		clipinfo->clipid = -1 ;
	    gp1->maskfunc = pw_maxmask ;
	    gp1->putcmap = pw_putcmap ;
	    gp1->putmask = pw_wmask ;
	    gp1->setcms = pw_cmsname ;
	    gp1->lockfunc = pw_dolock ;
	    gp1->unlockfunc = pw_dounlock ;
	    gp1->dbfunc = NULL ;
	    Dev_gp1_reset_to_defaults(gp1) ;
	}

	return(gp1) ;
}



/*
 * This function manages the pixwin clip lists.
 *
 *   0 - conditionally update cliplist, return 1 if any change.
 *   1 - unconditionally update cliplist
 *   2 - update viewscale
 *   3 - destroy
 */

static	void
pw_clipfunc(gp1, command)
register GP1	*gp1 ;
	int	command ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	switch(command)
	{
	    case 0:		/* conditional cliplist update */
		if( clipinfo->clipid ==
		    clipinfo->pw->pw_clipdata->pwcd_clipid)
		    break ;

	    case 1:		/* mandatory cliplist update */
	    {
		register Rect			*ptr ;
		register struct	gp1pr		*gp1dat;
		register struct pixwin_prlist	*prl;
		register short			*n_ptr;
		register int			nrect = 0 ;

		gp1->flags |= GP_RESIZE ;
		gp1->pr = clipinfo->pw->pw_clipdata->pwcd_prmulti ;
						/* backwards compat. */
		gp1->win_size.r_left = 0 ;
		gp1->win_size.r_top = 0 ;

		ptr = gp1->cliplist ;
		gp1->nclip = 0 ;

		prl = clipinfo->pw->pw_clipdata->pwcd_prl ;
		while(prl && (nrect < GP_MAXCLIP))
		{
		    gp1dat = gp1_d(prl->prl_pixrect) ;
		    ptr->r_left = gp1dat->cgpr_offset.x ;
		    ptr->r_top = gp1dat->cgpr_offset.y ;
		    ptr->r_width = prl->prl_pixrect->pr_size.x ;
		    ptr->r_height = prl->prl_pixrect->pr_size.y ;
		    ++gp1->nclip ;
		    prl = prl->prl_next ;
		    ++ptr ;
		}
		gp1->win_size.r_width =
		    clipinfo->pw->pw_clipdata->pwcd_prmulti->pr_size.x ;
		gp1->win_size.r_height =
		    clipinfo->pw->pw_clipdata->pwcd_prmulti->pr_size.y ;
		gp1dat = gp1_d(clipinfo->pw->pw_clipdata->pwcd_prmulti) ;
		gp1->win_org_x = gp1dat->cgpr_offset.x ;
		gp1->win_org_y = gp1dat->cgpr_offset.y ;
		gp1->clipid = clipinfo->pw->pw_clipdata->pwcd_clipid ;
		clipinfo->clipid = clipinfo->pw->pw_clipdata->pwcd_clipid ;
		break ;
	    }

	    case 2:		/* update view scale */
		dev_gp1_int_viewscale(gp1, &gp1->out_buffer) ;
		printf("called clipfunc(gp,2)?\n") ;
		break ;

	    case 3:		/* destroy */
		free(gp1->cliplist) ;
		free(gp1->clippriv) ;
		gp1->cliplist = NULL ;
		gp1->clippriv = NULL ;
		break ;
	}
}



static	void
pw_maxmask(gp1)
register GP1	*gp1 ;
{
	struct colormapseg	cms ;
register int	i,j ;
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	pw_getcmsdata(clipinfo->pw, &cms, NULL) ;
	i = cms.cms_size - 1 ;
	j = 0 ;
	while(i > 0)
	{
	    ++j ;
	    i >>= 1 ;
	}
	gp1->planes = j ;
	gp1->maxmask = (1<<gp1->planes) - 1 ;
	if(gp1->r0)
	    gp1->planes /= 2 ;
	pw_getattributes(clipinfo->pw, &gp1->realmask) ;
}



static	void
pw_putcmap(gp1, dir, start, length, r,g,b)
register GP1		*gp1 ;
	int		dir ;		/* 0 = put, 1=get */
	int		start ;
	int		length ;
	unsigned char	*r, *g, *b ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	if( dir == 0 )
	{
	    pw_putcolormap(clipinfo->pw, start, length, r,g,b) ;
	    pw_maxmask(gp1) ;
	}
	else
	    pw_getcolormap(clipinfo->pw, start, length, r,g,b) ;
}



static	void
pw_wmask(gp1, mask)
register GP1	*gp1 ;
	int	mask;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;
	struct colormapseg	cms;
	int			oldmask;

	(void)pw_getcmsdata(clipinfo->pw, &cms, &oldmask);

#ifdef  planes_fully_implemented 
	pr_putattributes(clipinfo->pw->pw_pixrect, &mask);
#else
	(void)pw_full_putattributes(clipinfo->pw->pw_pixrect, &mask);
#endif  planes_fully_implemented

	if (oldmask != mask)
	    (void)pw_getclipping(clipinfo->pw);
}



int
Dev_gp1_set_pw(gp1, pw)
register GP1	*gp1 ;
	Pixwin	*pw ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;
	Pixrect		*pr ;

	if( pw == clipinfo->pw )
	    return 0 ;

	Dev_gp1_flush(gp1, TRUE) ;

	pr = pw->pw_clipdata->pwcd_prmulti ;

	gp1->pw = pw ;		/* backwards compatibility */
	clipinfo->pw = pw ;

	if( gp1_d(pr)->minordev != gp1->minor_devno  ||
	    gp1_d(pr)->ioctl_fd != gp1->fd)
	    return( DEV_GP1_ERR_WRONG_GP ) ;

	gp1->fbi = gp1_d(pr)->cg2_index ;

	(*gp1->maskfunc)(gp1) ;
	(*gp1->clipfunc)(gp1,1) ;

	MAKE_ROOM(gp1, 2);
	GP1_PUT_SHORT(gp1, GP1_SET_FB_NUM | gp1->fbi) ;
	GP1_PUT_SHORT(gp1, GP1_SET_FB_PLANES | gp1->realmask) ;

	return 0 ;
}



static	void
pw_cmsname(gp1, dir, name)
register GP1	*gp1 ;
	int	dir ;		/* 0=write, 1=read */
	char	*name ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	if( dir == 0 )
	{
	    strncpy(gp1->cmsname, name, CMS_NAMESIZE) ;
	    pw_setcmsname(clipinfo->pw,gp1->cmsname) ;
	    pw_maxmask(gp1) ;
	}
	else
	    bcopy(gp1->cmsname, name, CMS_NAMESIZE) ;
}



static	void
pw_dolock(gp1, rect)
register GP1	*gp1 ;
register Rect	*rect ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	if( rect == NULL )
	    rect = &gp1->win_size ;
	pw_lock(clipinfo->pw, rect) ;
}



static	void
pw_dounlock(gp1)
register GP1	*gp1 ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	pw_unlock(clipinfo->pw) ;
}




Pixwin	*
Dev_gp1_get_pw(gp1)
	GP1	*gp1 ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	return clipinfo->pw ;
}
