#ifndef lint
static	char sccsid[] = "@(#)dev_gp1_create_pr.c 2.1 87/10/28 SMI";
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc.
 */



#include "dev_gp1_internal.h"



typedef	struct {
	    Pixrect	*pr ;
	} Clipinfo ;

static	void	pr_clipfunc() ;
static	void	pr_maxmask() ;
static	void	pr_putcmap() ;
static	void	pr_wmask() ;
static	void	pr_cmsname() ;


/* 
 * Allocate a gp1_handle and get a static block.
 * Initialize everything...
 */
     

GP1_handle
Dev_gp1_create_pr(pr, n_buffers, error_num)
	struct pixrect	*pr;	/* pixrect  */
	int	n_buffers;	/* number of buffers to allocate at once. */
	int	*error_num;	/* if an error, return the error number */
{   
register GP1	*gp1 ;
register Clipinfo *clipinfo ;

	gp1 = dev_gp1_int_open(pr, n_buffers, error_num) ;

	if(gp1)
	{
	    gp1->pr = pr ;		/* backwards compatibility */
	    gp1->clipfunc = pr_clipfunc ;
	    gp1->cliplist = (Rect *) malloc(sizeof(Rect)) ;
	    gp1->flags |= GP_RESIZE ;
	    clipinfo = (Clipinfo *) malloc(sizeof(Clipinfo)) ;
	    gp1->clippriv = (caddr_t) clipinfo ;
	    clipinfo->pr = pr ;
	    gp1->maskfunc = pr_maxmask ;
	    gp1->putcmap = pr_putcmap ;
	    gp1->putmask = pr_wmask ;
	    gp1->setcms = pr_cmsname ;
	    gp1->lockfunc = NULL ;
	    gp1->unlockfunc = NULL ;
	    Dev_gp1_reset_to_defaults(gp1) ;
	}

	return(gp1) ;
}



/*
 * This function manages the pixrect clip lists.
 *
 *   0 - conditionally update cliplist, return 1 if changed (never happens)
 *   1 - always update cliplist
 *   2 - update viewscale
 *   3 - destroy
 */

static	void
pr_clipfunc(gp1, command)
register GP1	*gp1 ;
	int	command ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	switch(command)
	{
	    case 0:		/* conditional cliplist update */
		break ;

	    case 1:		/* mandatory cliplist update */
	    {
		register struct	gp1pr		*gp1dat;

		gp1->nclip = 1 ;
		gp1->flags |= GP_RESIZE ;
		gp1dat = gp1_d(clipinfo->pr) ;
		gp1->cliplist->r_left = gp1dat->cgpr_offset.x ;
		gp1->cliplist->r_top = gp1dat->cgpr_offset.y ;
		gp1->cliplist->r_width = clipinfo->pr->pr_size.x ;
		gp1->cliplist->r_height = clipinfo->pr->pr_size.y ;

		gp1->win_size.r_left = 0 ;
		gp1->win_size.r_top = 0 ;
		gp1->win_size.r_width = clipinfo->pr->pr_size.x ;
		gp1->win_size.r_height = clipinfo->pr->pr_size.y ;

		gp1->win_org_x = gp1dat->cgpr_offset.y ;
		gp1->win_org_y = gp1dat->cgpr_offset.y ;
		break ;
	    }

	    case 2:		/* update view scale */
		dev_gp1_int_viewscale(gp1, &gp1->out_buffer) ;
		printf("called clipfunc(gp,2)?\n") ;
		break ;

	    case 3:		/* destroy */
		free(gp1->cliplist) ;
		free(gp1->clippriv) ;
		gp1->cliplist = NULL ;
		gp1->clippriv = NULL ;
		break ;
	}
}



static	void
pr_maxmask(gp1)
register GP1	*gp1 ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	gp1->planes = clipinfo->pr->pr_depth ;
	gp1->maxmask = (1 << gp1->planes) -1 ;
	pr_getattributes(clipinfo->pr, &gp1->realmask) ;
}



static	void
pr_putcmap(gp1, dir, start, length, r,g,b)
register GP1		*gp1 ;
	int		dir ;		/* 0 = put, 1=get */
	int		start ;
	int		length ;
	unsigned char	*r, *g, *b ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	if( dir == 0 )
	    pr_putcolormap(clipinfo->pr, start, length, r,g,b) ;
	else
	    pr_getcolormap(clipinfo->pr, start, length, r,g,b) ;
}



static	void
pr_wmask(gp1,mask)
register GP1	*gp1 ;
	int	mask ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	pr_putattributes(clipinfo->pr, mask) ;
}



int
Dev_gp1_set_pr(gp1, pr)
register GP1	*gp1;
	Pixrect	*pr;		/* pixrect  */
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	if( pr == clipinfo->pr )
	    return 0 ;

	Dev_gp1_flush(gp1, TRUE) ;

	if (gp1_d(pr)->minordev != gp1->minor_devno ||
	    gp1_d(pr)->ioctl_fd != gp1->fd)
	    return( DEV_GP1_ERR_WRONG_GP ) ;

	gp1->pr = pr ;		/* for backwards compatibility */
	clipinfo->pr = pr ;

	gp1->fbi = gp1_d(pr)->cg2_index ;

	(*gp1->maskfunc)(gp1) ;
	(*gp1->clipfunc)(gp1,1) ;

	MAKE_ROOM(gp1, 2);
	GP1_PUT_SHORT(gp1, GP1_SET_FB_NUM | gp1->fbi) ;
	GP1_PUT_SHORT(gp1, GP1_SET_FB_PLANES | gp1->realmask) ;

	return (0);
}



static	void
pr_cmsname(gp1, dir, name)
register GP1	*gp1 ;
	int	dir ;		/* 0=write, 1=read */
	char	*name ;
{
	if( dir == 0 )
	    bcopy(name, gp1->cmsname, CMS_NAMESIZE) ;
	else
	    bcopy(gp1->cmsname, name, CMS_NAMESIZE) ;
}



struct pixrect	*
Dev_gp1_get_pr(gp1)
	GP1	*gp1 ;
{
register Clipinfo	*clipinfo = (Clipinfo *) gp1->clippriv ;

	return clipinfo->pr ;
}
