#ifndef lint
static	char sccsid[] = "@(#)dev_gp1_dbuf.c 2.1 87/10/28 SMI";
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc.
 */


#include "dev_gp1_internal.h"


/*
 *	These routines handle color-table double buffering
 *
 *
 *	Routines provided here:
 *
 *
 *	Dev_gp1_dbuf_on(gp1, len, cmsname)
 *		turn on double buffering, len is colormap length
 *
 *	Dev_gp1_set_dbuf(gp1, planes, cmsname)
 *		turn on double buffering, 'planes' bitplanes/buffer
 *		(same as Dev_gp1_dbuf_on(gp1, 1<<planes, cmsname) )
 *
 *	Dev_gp1_sbuf_on(gp1, len, cmsname)
 *		turn single buffering on.
 *
 *	Dev_gp1_set_sbuf(gp1, planes, cmsname)
 *		turn single buffering on.
 *		(same as Dev_gp1_sbuf_on(gp1, 1<<planes, cmsname) )
 *
 *	Dev_gp1_dbuf_putcolormap(gp1, start, length, r,g,b)
 *		build the colortable entries for double buffering and load
 *		them into the gp's pw/pr's color table.
 *
 *	Dev_gp1_dbuf_getcolormap(gp1, start, length, r,g,b)
 *		Retrieve the gp's colormap.
 *
 *	Dev_gp1_dbuf_control(gp1, mask)
 *		set write enables: 1=front buffer, 2=back buffer, 3=both
 *
 *	Dev_gp1_dbuf_write_mask(gp1,mask)
 *		set write mask for double buffering
 *
 *	Dev_gp1_dbuf_color(gp1, color)
 *		set color for double buffering
 *
 *	Dev_gp1_dbuf_swap(gp1)
 *		swap buffers
 *
 *	Dev_gp1_dbuf_get_ctrl(gp1)
 *		return control mask
 *
 *	Dev_gp1_dbuf_get_write_mask(gp1)
 *		return write mask
 *
 *	Dev_gp1_dbuf_get_color(gp1, color)
 *		return "real" color
 *
 *
 *
 *	Edward A. Falk
 *	Sun Microsystems
 *
 *	June, 1987
 *
 *
 *
 */

#include "dev_gp1_internal.h"

/****
 *
 * Constants,  typedefs, externals, globals, statics, macros, block data
 *
 ****/


static	int	counter = 0 ;

static	void	dev_gp1_build_cmap() ;
	void	Dev_gp1_dbuf_swap() ;
	void	Dev_gp1_dbuf_write_mask() ;





static	void
put_dbuf_wmask(gp1)
register GP1	*gp1 ;
{
register unsigned int	omask = 0 ;
register unsigned int	mask = gp1->wmask ;
register unsigned int	ctrl = gp1->dbctrl ;

	if(gp1->dbuffer == 0)
	{
	    if(ctrl & 1)
		omask |= mask ;
	    if(ctrl & 2)
		omask |= mask << gp1->planes ;
	}
	else
	{
	    if(ctrl & 1)
	      omask |= mask << gp1->planes ;
	    if(ctrl & 2)
	      omask |= mask ;
	}

	gp1->realmask = omask & gp1->maxmask ;

	(*gp1->putmask)(gp1,gp1->realmask) ;

	MAKE_ROOM(gp1,1) ;
	GP1_PUT_SHORT(gp1, GP1_SET_FB_PLANES | gp1->realmask ) ;
}




/*
 * this routine writes out either the '0' or '1' color table, depending
 * on which buffer is currently being displayed
 */
static	void
put_dbuf_ctab(gp1)
register GP1	*gp1 ;
{
	unsigned char	*r,*g,*b ;

	if(gp1->r0 != NULL)
	{
	    if(gp1->dbuffer == 0)
	    {
		r = gp1->r0; g = gp1->g0 ; b = gp1->b0 ;
	    }
	    else
	    {
		r = gp1->r1; g = gp1->g1 ; b = gp1->b1 ;
	    }

	    (*gp1->putcmap)(gp1, 0, 0, gp1->rcmaplen, r,g,b) ;

	    put_dbuf_wmask(gp1) ;	/* rebuild write mask */
	}
}




void
Dev_gp1_dbuf_on(gp1, len, name)
register GP1	*gp1 ;
	int	len ;
	char	*name ;
{
	int	olen ;
	unsigned char	*r,*g,*b ;

	if(gp1->r0 == NULL  ||  gp1->cmaplen != len)
	{
	    if(gp1->r0 != NULL)
	    {
		free(gp1->r0) ;
		free(gp1->g0) ;
		free(gp1->b0) ;
		free(gp1->r1) ;
		free(gp1->g1) ;
		free(gp1->b1) ;
	    }

	    olen = gp1->cmaplen ;
	    if( olen <= 0 ) olen = 2 ;
	    gp1->cmaplen = len ;
	    {				/* how many bitplanes is this? */
		register unsigned int i,j ;
		for( i=len-1, j=0; i>0; ++j, i >>= 1 ) ;
		gp1->planes = j ;
	    }
	    gp1->rcmaplen = len << gp1->planes ;
	    gp1->r0 = (unsigned char *) malloc(gp1->rcmaplen) ;
	    gp1->g0 = (unsigned char *) malloc(gp1->rcmaplen) ;
	    gp1->b0 = (unsigned char *) malloc(gp1->rcmaplen) ;
	    gp1->r1 = (unsigned char *) malloc(gp1->rcmaplen) ;
	    gp1->g1 = (unsigned char *) malloc(gp1->rcmaplen) ;
	    gp1->b1 = (unsigned char *) malloc(gp1->rcmaplen) ;

	    gp1->wmask = (1<<gp1->planes) - 1 ;
	    gp1->maxmask = (1<<gp1->planes*2) - 1 ;

	    if(name != NULL)
		strncpy(gp1->cmsname, name, CMS_NAMESIZE) ;
	    else
		sprintf(gp1->cmsname,"dbuf%d.%d.%d",
		    getpid(),gp1->planes,counter++);

	    (*gp1->putcmap)(gp1, 0, 0, olen, gp1->r0, gp1->g0, gp1->b0) ;
	    (*gp1->setcms)(gp1, 0, gp1->cmsname) ;

	    dev_gp1_build_cmap(gp1, 0, olen, gp1->r0, gp1->g0, gp1->b0) ;
	    put_dbuf_ctab(gp1) ;
	}
}  




void
Dev_gp1_set_dbuf(gp1, planes, name)
register GP1	*gp1 ;
	int	planes ;
	char	*name ;
{
	Dev_gp1_dbuf_on(gp1, 1<<planes, name) ;
}  




void
Dev_gp1_sbuf_on(gp1, len, name)
register GP1	*gp1 ;
	int	len ;
	char	*name ;
{
	int	olen ;
	unsigned char	r[256],g[256],b[256] ;

	if(gp1->r0 != NULL  ||  gp1->cmaplen != len)
	{
	    if(gp1->r0 != NULL)
	    {
		free(gp1->r0) ;
		free(gp1->g0) ;
		free(gp1->b0) ;
		free(gp1->r1) ;
		free(gp1->g1) ;
		free(gp1->b1) ;
	    }

	    olen = gp1->cmaplen ;
	    if( olen <= 0 ) olen = 2 ;
	    gp1->cmaplen = gp1->rcmaplen = len ;
	    {				/* how many bitplanes is this? */
		register unsigned int i,j ;
		for( i=len-1, j=0; i>0; ++j, i >>= 1 ) ;
		gp1->planes = j ;
	    }
	    gp1->maxmask = gp1->realmask = gp1->wmask = (1<<gp1->planes) - 1 ;
	    gp1->dbctrl = 1 ;

	    if(name != NULL)
		strncpy(gp1->cmsname, name, CMS_NAMESIZE) ;
	    else
		sprintf(gp1->cmsname,"sbuf%d.%d.%d",
		    getpid(),gp1->planes,counter++) ;

	    (*gp1->putcmap)(gp1, 1, 0, olen, r,g,b) ;
	    (*gp1->setcms)(gp1, 0, gp1->cmsname) ;
	    (*gp1->putcmap)(gp1, 0, 0, len, r,g,b) ;
	    put_dbuf_wmask(gp1) ;
	}
}  




void
Dev_gp1_set_sbuf(gp1, planes, name)
register GP1	*gp1 ;
	int	planes ;
	char	*name ;
{
	Dev_gp1_sbuf_on(gp1, 1<<planes, name) ;
}  




void
Dev_gp1_dbuf_putcolormap(gp1, start, length, r,g,b)
register GP1		*gp1 ;
	int		start ;
	int		length ;
	unsigned char	*r, *g, *b ;
{
	if(gp1->r0)
	{
	    dev_gp1_build_cmap(gp1, start, length, r,g,b) ;
	    put_dbuf_ctab(gp1) ;
	}
	else
	{
	    (*gp1->putcmap)(gp1, 0, start, length, r,g,b) ;
	}
}



void
Dev_gp1_dbuf_control(gp1, mask)
register GP1		*gp1 ;
	unsigned int	mask ;
{
	gp1->dbctrl = mask ;
	put_dbuf_wmask(gp1) ;
}



void
Dev_gp1_dbuf_write_mask(gp1, mask)
register GP1		*gp1 ;
register unsigned int	mask ;
{
	if( gp1->r0 != NULL )
	    mask &= gp1->maxmask >> gp1->planes ;
	gp1->wmask = mask ;
	put_dbuf_wmask(gp1) ;
}




void
Dev_gp1_dbuf_color(gp1, color)
register GP1		*gp1 ;
register unsigned int	color ;
{
	DEV_GP1_DBUF_COLOR(gp1,color) ;
}



void
Dev_gp1_dbuf_swap(gp1)
register GP1	*gp1 ;
{
	unsigned char	*r,*g,*b ;

	if(gp1->r0 != NULL)
	{
	    gp1->dbuffer = 1 - gp1->dbuffer ;
	    put_dbuf_ctab(gp1) ;
	}
}




static	void
dev_gp1_build_cmap(gp1, start, len, r, g, b)
register GP1	*gp1 ;
	int	start, len ;
	unsigned char	*r,*g,*b ;
{
	int	len2 ;
register int	i, j ;
register unsigned char	ir,ig,ib ;
register unsigned char	*or,*og,*ob ;

	len2 = 1 << gp1->planes ;

	if(start+len > gp1->cmaplen)
	  return ;

	or = gp1->r0 + start ;
	og = gp1->g0 + start ;
	ob = gp1->b0 + start ;
	for(i=0; i<gp1->cmaplen; ++i)
	{
	  bcopy(r,or, len) ;
	  bcopy(g,og, len) ;
	  bcopy(b,ob, len) ;
	  or += len2 ;
	  og += len2 ;
	  ob += len2 ;
	}

	or = gp1->r1 + start*len2 ;
	og = gp1->g1 + start*len2 ;
	ob = gp1->b1 + start*len2 ;
	for(i=0; i<len; ++i)
	{
	  ir = *r++ ;
	  ig = *g++ ;
	  ib = *b++ ;
	  for(j=0; j<len2; ++j)
	  {
	    *or++ = ir ;
	    *og++ = ig ;
	    *ob++ = ib ;
	  }
	}
}



int
Dev_gp1_dbuf_get_ctrl(gp1)
register GP1	*gp1 ;
{
	return(gp1->dbctrl) ;
}


int
Dev_gp1_dbuf_get_write_mask(gp1)
register GP1	*gp1 ;
{
	return(gp1->wmask) ;
}



int
Dev_gp1_dbuf_get_realmask(gp1)
register GP1	*gp1 ;
{
	return(gp1->realmask) ;
}



int
Dev_gp1_dbuf_get_color(gp1,color)
register GP1	*gp1 ;
	int	color ;
{
	return DEV_GP1_DBUF_GET_COLOR(gp1,color) ;
}
