/*	@(#)dev_gp1_internal.h 2.1 87/10/28 SMI	*/

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#ifndef	DEV_GP1_INTERNAL
#define	DEV_GP1_INTERNAL


#include <sys/types.h>
#include <sunwindow/window_hs.h>
#include <pixrect/gp1cmds.h>
#include "dev_gp1.h"


#ifndef NULL
#define	NULL	0
#endif


#ifndef	TRUE
#define	TRUE	1
#define	FALSE	0
#endif




typedef	enum {				/* what type of list is in progress */
	  GP1_NO_LIST,
	  GP1_VEC_LIST_2DF,
	  GP1_VEC_LIST_3DF,
	  GP1_VEC_LIST_2DI,
	  GP1_VEC_LIST_3DI,
	  GP1_POLY_LIST_2DF,
	  GP1_POLY_LIST_3DF,
	  GP1_POLY_LIST_2DI,
	  GP1_POLY_LIST_3DI,
	} coordlist ;

typedef	enum { GP_GP1, GP_GPPLUS, GP_GP2 } Gptype ;


typedef	struct {
	  unsigned int	bitvec ;	/* bit mask for freeblocks	*/
	  int		bufoffset ;	/* offset of current output buffer */
	  short		*gpbuffer ;	/* pointer to current output buffer */
	  short		*cur_ptr ;	/* pointer to next word to add	*/
	  int		words_left ;	/* how many words remaining in buf */
	  int		len ;		/* length of this buffer in words */
	} Gpbuf ;



typedef struct _GP1 {
	  Pixrect	*pr ;		/* pixrect part (obsolete)	*/
	  Pixwin	*pw ;		/* pixwin, part (obsolete)	*/
	  int		static_block ;	/* static block id #		*/
	  int		pattern_block ;	/* pattern block id #		*/
	  u_char	minor_devno ;	/* minor device #		*/
	  int		fd ;		/* ioctl_fd			*/
	  int		fbi ;		/* frame buffer index		*/
	  short		*shmem ;	/* shared memory pointer	*/
	  int		ucode_ver ;	/* ucode serial # MMmmss	*/
	  Gptype	gptype ;	/* gp type			*/
	  int		fbtype ;	/* frame buffer type		*/
	  int		fbdepth ;	/* frame buffer depth		*/
	  int		n_buffers ;	/* number of buffers to alloc	*/
	  Gpbuf		out_buffer ;	/* main output buffer		*/
	  Gpbuf		tmp_buffer ;	/* scratch buffer		*/
	  short		*bound_ptr ;	/* pointer to next bound to add	*/
	  Rect		win_size ;	/* dimensions of screen rect.	*/
	  int		win_org_x, win_org_y;	/* origin of window	*/
	  float		xmin,xmax ;	/* view boundaries for x	*/
	  float		ymin,ymax ;	/* view boundaries for y	*/
	  float		zmin,zmax ;	/* view boundaries for z	*/
	  unsigned int	wmask ;		/* current write mask		*/
	  unsigned int	realmask ;	/* actual write mask		*/
	  unsigned int	maxmask ;	/* maximum permissible wmask	*/
	  int		planes ;	/* # of bit planes/buffer	*/
	  int		gp_3d ;		/* true iff 3-d mode		*/
	  coordlist	list_type ;	/* type of list being created	*/
	  int		gp_pickid ;	/* current pick id		*/
	  short		*gp_pickinfo ;	/* pointer to pick block if any	*/
	  int		clipid ;	/* most recent clip id		*/
	  unsigned int	flags ;		/* described below		*/
	  int		cur_matrix ;	/* matrix currently in use	*/
	  char		cmsname[CMS_NAMESIZE] ; /* used in double buffering */
	  unsigned int	cmaplen ;	/* color map length		*/
	  unsigned int	rcmaplen ;	/* real color map length	*/
	  unsigned char	*r0,*g0,*b0 ;	/* buffer 0 colormap		*/
	  unsigned char	*r1,*g1,*b1 ;	/* buffer 1 colormap		*/
	  int		dbuffer ;	/* currently displayed buffer	*/
	  int		dbctrl ;	/* write control		*/

	  /* window-system specific things go here */

	  int		nclip ;		/* # rects in clip list		*/
	  Rect		*cliplist ;	/* clip list			*/
	  caddr_t	clippriv ;	/* private data to clipfunc	*/
	  void		(*clipfunc)() ;	/* function to load new clip list */
	  void		(*maskfunc)() ;	/* finds maximum write mask	*/
	  void		(*putcmap)() ;	/* reads/loads colormap		*/
	  void		(*putmask)() ;	/* set write mask		*/
	  void		(*setcms)() ;	/* colormap name		*/
	  void		(*lockfunc)() ;	/* obtain lock			*/
	  void		(*unlockfunc)() ; /* release lock		*/
	  void		(*dbfunc)() ;	/* double-buffer control	*/
	} GP1 ;


/* flags:
 */
#define	GP_CONTAINS_MAT	0xff		/* there's a matrix update in buf */
#define	GP_CONTAINS_VWP	0x100		/* there's a viewport update in buf */
#define GP_CONTAINS_GFX 0x200		/* there are graphics in buf	*/
#define	GP_RESIZE	0x400		/* clip list has changed, need resize */
#define	GP_PICK		0x800		/* picking enabled		*/
#define	GP_NOFREE	0x1000		/* don't free this buffer	*/
#define	GP_FLOAT	0x2000		/* most recent point was float	*/

#define	GP1_MOVE	1
#define	GP1_DRAW	0
#define	GP1_TERMINATE	0x8000

#define GP_MAXCLIP	60

#define	GP1_DEFAULT_NUMBER_OF_BUFFERS	2
#define	GP1_WORDS_PER_BLOCK		512
#define	GP1_RESERVE			4



	/* compatibility macros */

#define	bytes_left	(outbuffer.words_left*2)


	/* size of a vector in a polyline */

#define	SIZE_V2DI	(1+2*sizeof(long)/sizeof(short))
#define	SIZE_V2DF	(1+2*sizeof(float)/sizeof(short))
#define	SIZE_V3DI	(1+3*sizeof(long)/sizeof(short))
#define	SIZE_V3DF	(1+3*sizeof(float)/sizeof(short))

	/* size of a vertex in a polygon */

#define	SIZE_P2DI	(2*sizeof(long)/sizeof(short))
#define	SIZE_P2DF	(2*sizeof(float)/sizeof(short))
#define	SIZE_P3DI	(3*sizeof(long)/sizeof(short))
#define	SIZE_P3DF	(3*sizeof(float)/sizeof(short))
#define	SIZE_P3DIRGB	((3*sizeof(int)+3*sizeof(float))/sizeof(short))
#define	SIZE_P3DFRGB	(6*sizeof(float)/sizeof(short))

	/* size of a point in a GP1_MUL_POINT */

#define	SIZE_T2DI	(1+2*sizeof(long)/sizeof(short))
#define	SIZE_T2DF	(1+2*sizeof(float)/sizeof(short))
#define	SIZE_T3DI	(1+4*sizeof(float)/sizeof(short))
#define	SIZE_T3DF	(1+4*sizeof(float)/sizeof(short))






#ifndef	GP1_PUT_F

/*	MACROS DEFINED IN 4.x
 */

#ifdef	mc68000

#define	GP1_PUT_I(p,a)	( *((long *)(p))++ = (long)(a) )
#define	GP1_PUT_F(p,a)	( *((float *)(p))++ = (float)(a) )
#define	GP1_GET_I(p,a)	( (a) = *((long *)(p))++ )
#define	GP1_GET_F(p,a)	( (a) = *((float *)(p))++ )

#else	mc68000

#define	GP1_PUT_I(p,a)	GP1_PUT_F(p,a)
#define	GP1_PUT_F(p,a)	( ((short *)(p))++ = ((short *)&(a))[0],	\
			  ((short *)(p))++ = ((short *)&(a))[1] )
#define	GP1_GET_I(p,a)	GP1_GET_F(p,a)
#define	GP1_GET_F(p,a)	( ((short *)&(a))[0] = ((short *)(p))++,	\
			  ((short *)&(a))[1] = ((short *)(p))++ )

#endif	mc68000

#endif	GP1_PUT_F



/*	PRIVATE MACRO DEFINITIONS
 *
 * gpo_d(gp1)				pointer to gp's pr data structure
 * DEV_GP1_LOCK(gp1)			lock gp's window
 * DEV_GP1_UNLOCK(gp1)			unlock gp's window
 *
 * GP1_PUT_SHORT_B(gpbuf,val)		add a short to a gp buffer
 * GP1_PUT_INT_B(gpbuf,val)		add a long int to a gp buffer
 * GP1_PUT_FLOAT_B(gpbuf,val)		add a float to a gp buffer
 * GP1_PUT_INT2_B(gpbuf,x,y)		add two long ints to a gp buffer
 * GP1_PUT_FLOAT2_B(gpbuf,x,y)		add two floats to a gp buffer
 * GP1_PUT_INT3_B(gpbuf,x,y,z)		add three long ints to a gp buffer
 * GP1_PUT_FLOAT3_B(gpbuf,x,y,z)	add three floats to a gp buffer
 * GP1_ISROOM_B(gpbuf,len)		see if there's room for len words
 * GP1_USEUP_B(gpbuf,len)		useup len words from a gp buffer
 * GP1_INT_BUFCLEAR(gp1,gpbuf)		reset gp buffer
 *
 * GP1_PUT_SHORT(gp1,val)		add a short to gp's main buffer
 * GP1_PUT_INT(gp1,val)			add a long int to gp's main buffer
 * GP1_PUT_FLOAT(gp1,val)		add a float to gp's main buffer
 * GP1_PUT_INT2(gp1,x,y)		add two long ints to gp's main buffer
 * GP1_PUT_FLOAT2(gp1,x,y)		add two floats to gp's main buffer
 * GP1_PUT_INT3(gp1,x,y,z)		add three long ints to gp's main buffer
 * GP1_PUT_FLOAT3(gp1,x,y,z)		add three floats to gp's main buffer
 * GP1_ISROOM(gp1,len)			see if there's room for len words
 * GP1_USEUP(gp1,len)			useup len words from gp's main buffer
 * GP1_CLEAR_BUF(gp1)			reset gp's main buffer
 * MAKE_ROOM(gp1,len)			see if there's room, flush if needed
 *
 */



#define	DEV_GP1_LOCK(gp1)	\
	{ if( (gp1)->lockfunc )	\
	    (*(gp1)->lockfunc)(gp1,&(gp1)->win_size) ; }

#define	DEV_GP1_UNLOCK(gp1)	\
	{ if( (gp1)->unlockfunc )	\
	    (*(gp1)->unlockfunc)(gp1) ; }


#define GP1_PUT_SHORT_B(gpbuf,val)		\
	((*(gpbuf).cur_ptr++ = (short)(val)),	\
	 --(gpbuf).words_left )

#define	GP1_PUT_INT_B(gpbuf,val)		\
	(GP1_PUT_I((gpbuf).cur_ptr, (val)),	\
	 (gpbuf).words_left-=2 )

#define	GP1_PUT_FLOAT_B(gpbuf,val)		\
	(GP1_PUT_F((gpbuf).cur_ptr, (val)),	\
	 (gpbuf).words_left-=2 )

#define	GP1_PUT_INT2_B(gpbuf,x,y)		\
	(GP1_PUT_I((gpbuf).cur_ptr, (x)),	\
	 GP1_PUT_I((gpbuf).cur_ptr, (y)),	\
	 (gpbuf).words_left-=4 )

#define	GP1_PUT_FLOAT2_B(gpbuf,x,y)		\
	(GP1_PUT_F((gpbuf).cur_ptr, (x)),	\
	 GP1_PUT_F((gpbuf).cur_ptr, (y)),	\
	 (gpbuf).words_left-=4 )

#define	GP1_PUT_INT3_B(gpbuf,x,y,z)		\
	(GP1_PUT_I((gpbuf).cur_ptr, (x)),	\
	 GP1_PUT_I((gpbuf).cur_ptr, (y)),	\
	 GP1_PUT_I((gpbuf).cur_ptr, (z)),	\
	 (gpbuf).words_left-=6 )

#define	GP1_PUT_FLOAT3_B(gpbuf,x,y,z)		\
	(GP1_PUT_F((gpbuf).cur_ptr, (x)),	\
	 GP1_PUT_F((gpbuf).cur_ptr, (y)),	\
	 GP1_PUT_F((gpbuf).cur_ptr, (z)),	\
	 (gpbuf).words_left-=6 )

#define	GP1_USEUP_B(gpbuf,len)		\
	((gpbuf).cur_ptr += (len), (gpbuf).words_left -= (len))

#define	GP1_ISROOM_B(gpbuf,len)				\
	((gpbuf).words_left >= (len))

#define	GP1_INT_BUFCLEAR(gp1,gpbuf)			\
	( (gpbuf).cur_ptr = (gpbuf).gpbuffer ,	\
	  (gpbuf).words_left = (gpbuf).len-4 ,	\
	  GP1_PUT_SHORT_B((gpbuf), GP1_USE_CONTEXT | (gp1)->static_block) )


#define	GP1_PUT_SHORT(gp1,val)	GP1_PUT_SHORT_B((gp1)->out_buffer,(val))
#define	GP1_PUT_INT(gp1,val)	GP1_PUT_INT_B((gp1)->out_buffer,(val))
#define	GP1_PUT_FLOAT(gp1,val)	GP1_PUT_FLOAT_B((gp1)->out_buffer,(val))
#define	GP1_PUT_INT2(gp1,x,y)	GP1_PUT_INT2_B((gp1)->out_buffer,(x),(y))
#define	GP1_PUT_FLOAT2(gp1,x,y)	GP1_PUT_FLOAT2_B((gp1)->out_buffer,(x),(y))
#define	GP1_PUT_INT3(gp1,x,y,z)	GP1_PUT_INT3_B((gp1)->out_buffer,(x),(y),(z))
#define	GP1_PUT_FLOAT3(gp1,x,y,z)			\
				GP1_PUT_FLOAT3_B((gp1)->out_buffer,(x),(y),(z))
#define	GP1_USEUP(gp1,len)	GP1_USEUP_B((gp1)->out_buffer,(len))
#define	GP1_ISROOM(gp1,len)	GP1_ISROOM_B((gp1)->out_buffer,(len))

#define	GP1_CLEAR_BUF(gp1)	\
	( GP1_INT_BUFCLEAR((gp1), (gp1)->out_buffer),	\
	  (gp1)->flags = 0 )

#define	MAKE_ROOM(gp1,len)		\
	{ if(!GP1_ISROOM(gp1,len))	\
	    Dev_gp1_flush(gp1,1) ; }



/*
 * private externs
 */

extern	void		_gp1_alloc_buffers() ;
extern	GP1_handle	dev_gp1_int_open() ;



/*
 * MACRO DEFINITIONS
 *
 * DEV_GP1_SET_COLOR(gp1, color)		set color
 * DEV_GP1_CLIP(gp1, planes)			set clip planes
 * DEV_GP1_ROP_MODE(gp1, rop)			set rop
 * DEV_GP1_HSR(gp1, control)			set hidden surface removal
 * DEV_GP1_LINSTY(gp1, count, pattrn, ofs, ops)	set line style
 * DEV_GP1_WIDTH(gp1, width)			set line width
 * DEV_GP1_SELECT_MATRIX(gp1, num)		select matrix id
 * DEV_GP1_PRS(gp1,npts)			submit line process block
 * DEV_GP1_XFS(gp1,npts)			submit point transform block
 * DEV_GP1_DBUF_COLOR(gp1, color)		set dbuf color
 */














/*
 * Macros for setting attributes.
 */

#define DEV_GP1_SET_COLOR(gp1, color)				\
{   								\
    MAKE_ROOM((gp1), 1);					\
    (gp1)->list_type = GP1_NO_LIST;				\
    GP1_PUT_SHORT((gp1), GP1_SET_COLOR | ((color) & 0xff) ) ;	\
}



#define	DEV_GP1_RGBCOLOR(gp1,r,g,b)				\
{								\
    MAKE_ROOM((gp1), 1+3*sizeof(float)/sizeof(short));		\
    (gp1)->list_type = GP1_NO_LIST;    				\
    GP1_PUT_SHORT(gp1, GP2_SET_RGB_COLOR | 1) ;			\
    GP1_PUT_FLOAT(gp1, (r)) ;					\
    GP1_PUT_FLOAT(gp1, (g)) ;					\
    GP1_PUT_FLOAT(gp1, (b)) ;					\
}



#define	DEV_GP1_RGBPACKED(gp1,val)				\
{								\
    MAKE_ROOM((gp1), 1+sizeof(int)/sizeof(short));		\
    (gp1)->list_type = GP1_NO_LIST;    				\
    GP1_PUT_SHORT(gp1, GP2_SET_RGB_COLOR | 2) ;			\
    GP1_PUT_INT(gp1, (val)) ;					\
}



#define DEV_GP1_SET_CLIP_PLANES(gp1, mask)			\
{   								\
    MAKE_ROOM((gp1), 1);					\
    (gp1)->list_type = GP1_NO_LIST;    				\
    GP1_PUT_SHORT((gp1), GP1_SET_CLIP_PLANES | ((mask) & 0xff) ) ; \
}


#define	DEV_GP1_SET_HIDDEN_SURF(gp1, type)			\
{   								\
    MAKE_ROOM((gp1), 1);					\
    (gp1)->list_type = GP1_NO_LIST;    				\
    GP1_PUT_SHORT((gp1), GP1_SET_HIDDEN_SURF | ((type) & 0xff) ) ; \
}


#define	DEV_GP1_SET_ROP_MODE(gp1, type)		\
{   						\
    MAKE_ROOM((gp1), 1);			\
    (gp1)->list_type = GP1_NO_LIST;    		\
    GP1_PUT_SHORT((gp1), GP1_SET_ROP ) ;	\
    GP1_PUT_SHORT((gp1), type) ;		\
}




#define	DEV_GP1_SET_LINE_WIDTH(gp1, width)	\
{						\
    MAKE_ROOM((gp1), 3);			\
    GP1_PUT_SHORT(gp1, GP1_SET_LINE_WIDTH) ;	\
    GP1_PUT_SHORT(gp1, (width)) ;		\
    GP1_PUT_SHORT(gp1, 0) ;			\
    (gp1)->list_type = GP1_NO_LIST;    		\
}


#define	DEV_GP1_DBUF_GET_COLOR(gp1, color)	\
		((color) << (gp1)->planes  |  (color))


#define	DEV_GP1_DBUF_COLOR(gp1, color)	\
    DEV_GP1_SET_COLOR((gp1), (DEV_GP1_DBUF_GET_COLOR(gp1,color) & 0xff ))


#define DEV_GP1_SET_TEXTURE_ORG_SCREEN(gp1, x, y)	\
{							\
    MAKE_ROOM((gp1), 3);				\
    GP1_PUT_SHORT(gp1, GP1_SET_PGON_TEX_ORG_SCR) ;	\
    GP1_PUT_SHORT(gp1, (x)) ;				\
    GP1_PUT_SHORT(gp1, (y)) ;				\
    (gp1)->list_type = GP1_NO_LIST;    			\
}


#define DEV_GP1_SET_TEXTURE_ORG_2DF(gp1, x, y)		\
{							\
    MAKE_ROOM((gp1), 1+2*sizeof(float)/sizeof(short));	\
    GP1_PUT_SHORT(gp1, GP1_SET_PGON_TEX_ORG_XF_2D) ;	\
    GP1_PUT_FLOAT(gp1, (x)) ;				\
    GP1_PUT_FLOAT(gp1, (y)) ;				\
    (gp1)->list_type = GP1_NO_LIST;    			\
}


#define DEV_GP1_SET_TEXTURE_ORG_3DF(gp1, x, y, z)	\
{							\
    MAKE_ROOM((gp1), 1+3*sizeof(float)/sizeof(short));	\
    GP1_PUT_SHORT(gp1, GP1_SET_PGON_TEX_ORG_XF_3D) ;	\
    GP1_PUT_FLOAT(gp1, (x)) ;				\
    GP1_PUT_FLOAT(gp1, (y)) ;				\
    GP1_PUT_FLOAT(gp1, (z)) ;				\
    (gp1)->list_type = GP1_NO_LIST;    			\
}


#define	DEV_GP1_SELECT_TEXTURE_BLOCK(gp1, num)	\
{						\
    MAKE_ROOM((gp1), 1);			\
    GP1_PUT_SHORT((gp1), GP1_SET_PGON_TEX_BLK | ((num) & 0x7) ) ; \
    (gp1)->list_type = GP1_NO_LIST;    		\
    (gp1)->pattern_block = (num) ;		\
}






/*
 * Macros for matrix handling
 */


#define	DEV_GP1_SELECT_MATRIX(gp1, num)		\
{						\
    MAKE_ROOM((gp1), 1);			\
    GP1_PUT_SHORT((gp1), GP1_SET_MAT_NUM | ((num) & 0xff) ) ; \
    (gp1)->list_type = GP1_NO_LIST;    		\
    (gp1)->cur_matrix = num ;			\
}


#define	DEV_GP1_MATRIX_MUL_2D(gp1, result, a, b)		\
{								\
    register short *_gp1_tmp ;					\
    MAKE_ROOM((gp1), 4) ;					\
    _gp1_tmp = (gp1)->out_buffer.cur_ptr ;			\
    *_gp1_tmp++ = GP1_MUL_MAT_2D ;				\
    *_gp1_tmp++ = a;						\
    *_gp1_tmp++ = b;						\
    *_gp1_tmp++ = result;					\
    GP1_USEUP((gp1), 4) ;					\
    (gp1)->list_type = GP1_NO_LIST;				\
    (gp1)->flags |= 1 << result ;				\
}


#define	DEV_GP1_MATRIX_COPY(gp1, src,dst)	\
{						\
    register short *_gp1_tmp ;			\
    MAKE_ROOM((gp1), 3) ;			\
    _gp1_tmp = (gp1)->out_buffer.cur_ptr ;	\
    *_gp1_tmp++ = GP1_COPY_MAT ;		\
    *_gp1_tmp++ = src;				\
    *_gp1_tmp++ = dst;				\
    GP1_USEUP((gp1), 3) ;			\
    (gp1)->list_type = GP1_NO_LIST;		\
    (gp1)->flags |= 1 << dst ;			\
}


#define	DEV_GP1_MATRIX_MUL_3D(gp1, result, a, b)		\
{								\
    register short *_gp1_tmp ;					\
    MAKE_ROOM((gp1), 4) ;					\
    _gp1_tmp = (gp1)->out_buffer.cur_ptr ;			\
    *_gp1_tmp++ = GP1_MUL_MAT_3D ;				\
    *_gp1_tmp++ = a;						\
    *_gp1_tmp++ = b;						\
    *_gp1_tmp++ = result;					\
    GP1_USEUP((gp1), 4) ;					\
    (gp1)->list_type = GP1_NO_LIST;				\
    (gp1)->flags |= 1 << result ;				\
}








/*
 * Macros for stuffing polygons into the gp1.
 */



#define	DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit, shade, cmd, size) \
{   									\
    register	int	total;						\
    total = 2 + (n_bounds) + (n_vertices) * (size) ;			\
    MAKE_ROOM((gp1), total) ;						\
    if(!GP1_ISROOM((gp1), total))					\
    {									\
	(fit) = ((gp1)->out_buffer.words_left - 1 - (n_bounds) )/(size) ; \
    }									\
    else								\
    {									\
	(fit) = (n_vertices) ;						\
	(gp1)->list_type = GP1_NO_LIST;					\
	GP1_PUT_SHORT((gp1), (cmd) | ((shade) & 0x7)) ;			\
	(gp1)->bound_ptr = (gp1)->out_buffer.cur_ptr;			\
	GP1_PUT_SHORT((gp1), (n_bounds)) ;				\
	GP1_USEUP((gp1), (n_bounds)) ;					\
    }									\
}





#define	DEV_GP1_POLY_2DI_INIT(gp1, n_bounds, n_vertices, fit, shade)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			shade, GP1_XF_PGON_INT_2D, SIZE_P2DI)

#define DEV_GP1_POLY_2DI_ADD_BOUND(gp1)					\
    *++((gp1)->bound_ptr) = 0

#define DEV_GP1_POLY_2DI_ADD_VERTEX(gp1, pt)				\
{									\
	GP1_PUT_INT2(gp1,(pt)->x,(pt)->y) ;				\
	(*((gp1)->bound_ptr))++;					\
}





#define	DEV_GP1_POLY_2DF_INIT(gp1, n_bounds, n_vertices, fit, shade)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			shade, GP1_XF_PGON_FLT_2D, SIZE_P2DF)

#define DEV_GP1_POLY_2DF_ADD_BOUND(gp1)					\
    *++((gp1)->bound_ptr) = 0

#define DEV_GP1_POLY_2DF_ADD_VERTEX(gp1, pt)				\
{									\
	GP1_PUT_FLOAT2(gp1,(pt)->x,(pt)->y) ;				\
	(*((gp1)->bound_ptr))++;					\
}




#define	DEV_GP1_POLY_3DI_INIT(gp1, n_bounds, n_vertices, fit, shade)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			shade, GP1_XF_PGON_FLT_3D,			\
			SIZE_P3DF + (shade == GP1_SHADE_GOURAUD ? 2:0))

#define DEV_GP1_POLY_3DI_ADD_BOUND(gp1)					\
    *++((gp1)->bound_ptr) = 0

#define DEV_GP1_POLY_3DI_ADD_VERTEX(gp1, pt)				\
{									\
	GP1_PUT_INT3(gp1,(pt)->x,(pt)->y,(pt)->z) ;			\
	(*((gp1)->bound_ptr))++;					\
}

#define DEV_GP1_POLY_3DI_ADD_VERTEX_GOURAUD(gp1, pt)			\
{									\
	GP1_PUT_INT3(gp1,(pt)->p.x,(pt)->p.y,(pt)->p.z) ;		\
	GP1_PUT_INT(gp1,(pt)->color) ;					\
	(*((gp1)->bound_ptr))++;					\
}




#define	DEV_GP1_POLY_3DF_INIT(gp1, n_bounds, n_vertices, fit, shade)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			shade, GP1_XF_PGON_FLT_3D,			\
			SIZE_P3DF + (shade == GP1_SHADE_GOURAUD ? 2:0))


#define DEV_GP1_POLY_3DF_ADD_BOUND(gp1)					\
    *++((gp1)->bound_ptr) = 0

#define DEV_GP1_POLY_3DF_ADD_VERTEX(gp1, pt)				\
{									\
	GP1_PUT_FLOAT3(gp1,(pt)->x,(pt)->y,(pt)->z) ;			\
	(*((gp1)->bound_ptr))++;					\
}

#define DEV_GP1_POLY_3DF_ADD_VERTEX_GOURAUD(gp1, pt)			\
{									\
	GP1_PUT_FLOAT3(gp1,(pt)->p.x,(pt)->p.y,(pt)->p.z) ;		\
	GP1_PUT_INT(gp1,(pt)->color) ;					\
	(*((gp1)->bound_ptr))++;					\
}




#define	DEV_GP2_POLY_3DI_INIT(gp1, n_bounds, n_vertices, fit, shade)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			shade, GP2_XF_PGON_INT_3D_RGB, SIZE_P3DF )

#define	DEV_GP2_POLY_3DI_INIT_RGB(gp1, n_bounds, n_vertices, fit)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			GP2_RGB_COLOR_TRIPLE, GP2_XF_PGON_INT_3D_RGB,	\
			SIZE_P3DFRGB )

#define	DEV_GP2_POLY_3DI_INIT_PACKED(gp1, n_bounds, n_vertices, fit)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			GP2_RGB_COLOR_PACK, GP2_XF_PGON_INT_3D_RGB,	\
			SIZE_P3DF + 2 )

#define DEV_GP2_POLY_3DI_ADD_BOUND(gp1)					\
    *++((gp1)->bound_ptr) = 0

#define DEV_GP2_POLY_3DI_ADD_VERTEX(gp1, pt)				\
{									\
	GP1_PUT_INT3(gp1,(pt)->x,(pt)->y,(pt)->z) ;			\
	(*((gp1)->bound_ptr))++;					\
}

#define DEV_GP2_POLY_3DI_ADD_VERTEX_RGB(gp1, pt)			\
{									\
	GP1_PUT_INT3(gp1,(pt)->p.x,(pt)->p.y,(pt)->p.z) ;		\
	GP1_PUT_FLOAT3(gp1,(pt)->c.r,(pt)->c.g,(pt)->c.b) ;		\
	(*((gp1)->bound_ptr))++;					\
}

#define DEV_GP2_POLY_3DI_ADD_VERTEX_PACKED	\
		DEV_GP1_POLY_3DI_ADD_VERTEX_GOURAUD





#define	DEV_GP2_POLY_3DF_INIT(gp1, n_bounds, n_vertices, fit, shade)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			shade, GP2_XF_PGON_FLT_3D_RGB, SIZE_P3DF )

#define	DEV_GP2_POLY_3DF_INIT_RGB(gp1, n_bounds, n_vertices, fit)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			GP2_RGB_COLOR_TRIPLE, GP2_XF_PGON_FLT_3D_RGB,	\
			SIZE_P3DFRGB )

#define	DEV_GP2_POLY_3DF_INIT_PACKED(gp1, n_bounds, n_vertices, fit)	\
		DEV_GP1_POLY_INIT(gp1, n_bounds, n_vertices, fit,	\
			GP2_RGB_COLOR_PACK, GP2_XF_PGON_FLT_3D_RGB,	\
			SIZE_P3DF + 2 )

#define DEV_GP2_POLY_3DF_ADD_BOUND(gp1)					\
    *++((gp1)->bound_ptr) = 0

#define DEV_GP2_POLY_3DF_ADD_VERTEX(gp1, pt)				\
{									\
	GP1_PUT_FLOAT3(gp1,(pt)->x,(pt)->y,(pt)->z) ;			\
	(*((gp1)->bound_ptr))++;					\
}

#define DEV_GP2_POLY_3DF_ADD_VERTEX_RGB(gp1, pt)			\
{									\
	GP1_PUT_FLOAT3(gp1,(pt)->p.x,(pt)->p.y,(pt)->p.z) ;		\
	GP1_PUT_FLOAT3(gp1,(pt)->c.r,(pt)->c.g,(pt)->c.b) ;		\
	(*((gp1)->bound_ptr))++;					\
}

#define DEV_GP2_POLY_3DF_ADD_VERTEX_PACKED	\
		DEV_GP1_POLY_3DF_ADD_VERTEX_GOURAUD










/*
 * VECTORS
 *
 */

#define	DEV_GP1_VEC_INIT(gp1, nvert, fit, type, cmd, size)	\
{   							\
    register	int	total;				\
    total = 1 + nvert * (size) ;			\
    MAKE_ROOM((gp1), total) ;				\
    if(!GP1_ISROOM((gp1), total))			\
    {							\
	(fit) = (gp1)->out_buffer.words_left ;		\
	(fit) = ((fit) - 1)/(size) ;			\
    }							\
    else						\
    {							\
	(fit) = (nvert) ;				\
    }							\
    (gp1)->list_type = type ;				\
    GP1_PUT_SHORT((gp1), cmd) ;				\
}




/*
 * 2-D Integer vectors
 */



#define	DEV_GP1_VEC_INIT_2DI(gp1, nvert, fit)		\
	    DEV_GP1_VEC_INIT((gp1), (nvert), (fit),	\
		GP1_VEC_LIST_2DI, GP1_XF_LINE_INT_2D, SIZE_V2DI)


#define	DEV_GP1_VEC_MOVE_2DI(gp1,pt)	DEV_GP1_VEC_2DI(gp1,GP1_MOVE,(pt))
#define	DEV_GP1_VEC_DRAW_2DI(gp1,pt)	DEV_GP1_VEC_2DI(gp1,GP1_DRAW,(pt))


#define	DEV_GP1_VEC_TERM_2DI(gp1)	\
{								\
    (*((gp1)->out_buffer.cur_ptr - SIZE_V2DI) |= GP1_TERMINATE); \
}


#define	DEV_GP1_VEC_2DI(gp1, flag, pt)					\
{									\
    register short *_gp1_tmp = (gp1)->out_buffer.cur_ptr ;		\
    *_gp1_tmp++ = (flag) ;						\
    GP1_PUT_I(_gp1_tmp,((pt)->x)) ;					\
    GP1_PUT_I(_gp1_tmp,((pt)->y)) ;					\
    GP1_USEUP(gp1, SIZE_V2DI) ;						\
}

#define DEV_GP1_MOVE_2DI(gp1, pt)					\
{									\
    register short	*_gp1_tmp ;					\
    if ((gp1)->list_type != GP1_VEC_LIST_2DI) {				\
	MAKE_ROOM((gp1), 1+2*SIZE_V2DI) ;				\
	(gp1)->list_type = GP1_VEC_LIST_2DI ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_INT_2D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else if (!GP1_ISROOM((gp1), SIZE_V2DI)) {				\
	Dev_gp1_flush((gp1), TRUE) ;					\
	(gp1)->list_type = GP1_VEC_LIST_2DI ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_INT_2D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else {								\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*(_gp1_tmp-SIZE_V2DI) &= ~GP1_TERMINATE;			\
    }									\
    *_gp1_tmp++ = GP1_TERMINATE | GP1_MOVE ;				\
    GP1_PUT_I(_gp1_tmp,((pt)->x)) ;					\
    GP1_PUT_I(_gp1_tmp,((pt)->y)) ;					\
    (gp1)->out_buffer.words_left -= SIZE_V2DI;				\
    (gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
}

#define DEV_GP1_DRAW_2DI(gp1, pt)					\
{									\
    register short	*_gp1_tmp = (gp1)->out_buffer.cur_ptr ;		\
    if ((gp1)->list_type == GP1_VEC_LIST_2DI) {				\
	if (!GP1_ISROOM((gp1), SIZE_V2DI)) {				\
	    long	_tmp_x, _tmp_y;					\
	    								\
	    _gp1_tmp -= (SIZE_V2DI-1) ;					\
	    GP1_GET_I(_gp1_tmp, _tmp_x) ;				\
	    GP1_GET_I(_gp1_tmp, _tmp_y) ;				\
	    Dev_gp1_flush((gp1), TRUE);					\
	    (gp1)->list_type = GP1_VEC_LIST_2DI ;			\
	    _gp1_tmp = (gp1)->out_buffer.cur_ptr ;			\
	    *_gp1_tmp++ = GP1_XF_LINE_INT_2D ;				\
	    *_gp1_tmp++ = GP1_MOVE ;					\
	    GP1_PUT_I(_gp1_tmp, _tmp_x) ;				\
	    GP1_PUT_I(_gp1_tmp, _tmp_y) ;				\
	    (gp1)->out_buffer.words_left -= 1+SIZE_V2DI ;		\
	}								\
	else								\
	{								\
	    *(_gp1_tmp-SIZE_V2DI) &= ~GP1_TERMINATE ;			\
	}								\
	*_gp1_tmp++ = GP1_TERMINATE | GP1_DRAW ;			\
	GP1_PUT_I(_gp1_tmp, (pt)->x) ;					\
	GP1_PUT_I(_gp1_tmp, (pt)->y) ;					\
	(gp1)->out_buffer.words_left -= SIZE_V2DI ;			\
	(gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
    }									\
}







/*
 * 2-D Floating vectors
 */

#define	DEV_GP1_VEC_INIT_2DF(gp1, nvert, fit)		\
	    DEV_GP1_VEC_INIT((gp1), (nvert), (fit),	\
		GP1_VEC_LIST_2DF, GP1_XF_LINE_FLT_2D, SIZE_V2DF)



#define	DEV_GP1_VEC_MOVE_2DF(gp1,pt)	DEV_GP1_VEC_2DF(gp1,GP1_MOVE,(pt))
#define	DEV_GP1_VEC_DRAW_2DF(gp1,pt)	DEV_GP1_VEC_2DF(gp1,GP1_DRAW,(pt))


#define	DEV_GP1_VEC_TERM_2DF(gp1)	\
{								\
    (gp1)->list_type = GP1_NO_LIST;				\
    (*((gp1)->out_buffer.cur_ptr - SIZE_V2DF) |= GP1_TERMINATE); \
}


#define	DEV_GP1_VEC_2DF(gp1, flag, pt)				\
{								\
    register short *_gp1_tmp = (gp1)->out_buffer.cur_ptr ;	\
    *_gp1_tmp++ = (flag) ;					\
    GP1_PUT_F(_gp1_tmp,((pt)->x)) ;				\
    GP1_PUT_F(_gp1_tmp,((pt)->y)) ;				\
    GP1_USEUP(gp1, SIZE_V2DF) ;					\
}



#define DEV_GP1_MOVE_2DF(gp1, pt)					\
{									\
    register short	*_gp1_tmp ;					\
    if ((gp1)->list_type != GP1_VEC_LIST_2DF) {				\
	MAKE_ROOM((gp1), 1+2*SIZE_V2DF) ;				\
	(gp1)->list_type = GP1_VEC_LIST_2DF ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_FLT_2D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else if (!GP1_ISROOM((gp1), SIZE_V2DF)) {				\
	Dev_gp1_flush((gp1), TRUE) ;					\
	(gp1)->list_type = GP1_VEC_LIST_2DF ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_FLT_2D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else {								\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*(_gp1_tmp-SIZE_V2DF) &= ~GP1_TERMINATE;			\
    }									\
    *_gp1_tmp++ = GP1_TERMINATE | GP1_MOVE ;				\
    GP1_PUT_F(_gp1_tmp,((pt)->x)) ;					\
    GP1_PUT_F(_gp1_tmp,((pt)->y)) ;					\
    (gp1)->out_buffer.words_left -= SIZE_V2DF;				\
    (gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
}


#define DEV_GP1_DRAW_2DF(gp1, pt)					\
{									\
    register short	*_gp1_tmp = (gp1)->out_buffer.cur_ptr ;		\
    if ((gp1)->list_type == GP1_VEC_LIST_2DF) {				\
	if (!GP1_ISROOM((gp1), SIZE_V2DF)) {				\
	    float	_tmp_x, _tmp_y;					\
	    								\
	    _gp1_tmp -= (SIZE_V2DF-1) ;					\
	    GP1_GET_F(_gp1_tmp, _tmp_x) ;				\
	    GP1_GET_F(_gp1_tmp, _tmp_y) ;				\
	    Dev_gp1_flush((gp1), TRUE);					\
	    (gp1)->list_type = GP1_VEC_LIST_2DF ;			\
	    _gp1_tmp = (gp1)->out_buffer.cur_ptr ;			\
	    *_gp1_tmp++ = GP1_XF_LINE_FLT_2D ;				\
	    *_gp1_tmp++ = GP1_MOVE ;					\
	    GP1_PUT_F(_gp1_tmp, _tmp_x) ;				\
	    GP1_PUT_F(_gp1_tmp, _tmp_y) ;				\
	    (gp1)->out_buffer.words_left -= 1+SIZE_V2DF ;		\
	}								\
	else								\
	{								\
	    *(_gp1_tmp-SIZE_V2DF) &= ~GP1_TERMINATE ;			\
	}								\
	*_gp1_tmp++ = GP1_TERMINATE | GP1_DRAW ;			\
	GP1_PUT_F(_gp1_tmp, (pt)->x) ;					\
	GP1_PUT_F(_gp1_tmp, (pt)->y) ;					\
	(gp1)->out_buffer.words_left -= SIZE_V2DF ;			\
	(gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
    }									\
}





/*
 * 3-D Integer vectors
 */

#define	DEV_GP1_VEC_INIT_3DI(gp1, nvert, fit)		\
	    DEV_GP1_VEC_INIT((gp1), (nvert), (fit),	\
		GP1_VEC_LIST_3DI, GP1_XF_LINE_INT_3D, SIZE_V3DI)



#define	DEV_GP1_VEC_MOVE_3DI(gp1,pt)	DEV_GP1_VEC_3DI(gp1,GP1_MOVE,(pt))
#define	DEV_GP1_VEC_DRAW_3DI(gp1,pt)	DEV_GP1_VEC_3DI(gp1,GP1_DRAW,(pt))


#define	DEV_GP1_VEC_TERM_3DI(gp1)	\
{								\
    (gp1)->list_type = GP1_NO_LIST;				\
    (*((gp1)->out_buffer.cur_ptr - SIZE_V3DI) |= GP1_TERMINATE); \
}



#define	DEV_GP1_VEC_3DI(gp1, flag, pt)					\
{									\
    register short *_gp1_tmp = (gp1)->out_buffer.cur_ptr ;		\
    *_gp1_tmp++ = (flag) ;						\
    GP1_PUT_I(_gp1_tmp,((pt)->x)) ;					\
    GP1_PUT_I(_gp1_tmp,((pt)->y)) ;					\
    GP1_PUT_I(_gp1_tmp,((pt)->z)) ;					\
    GP1_USEUP(gp1, SIZE_V3DI) ;						\
}


#define DEV_GP1_MOVE_3DI(gp1, pt)					\
{									\
    register short	*_gp1_tmp ;					\
    if ((gp1)->list_type != GP1_VEC_LIST_3DI) {				\
	MAKE_ROOM((gp1), 1+2*SIZE_V3DI) ;				\
	(gp1)->list_type = GP1_VEC_LIST_3DI ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_INT_3D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else if (!GP1_ISROOM((gp1), SIZE_V3DI)) {				\
	Dev_gp1_flush((gp1), TRUE) ;					\
	(gp1)->list_type = GP1_VEC_LIST_3DI ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_INT_3D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else {								\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*(_gp1_tmp-SIZE_V3DI) &= ~GP1_TERMINATE;			\
    }									\
    *_gp1_tmp++ = GP1_TERMINATE | GP1_MOVE ;				\
    GP1_PUT_I(_gp1_tmp,((pt)->x)) ;					\
    GP1_PUT_I(_gp1_tmp,((pt)->y)) ;					\
    GP1_PUT_I(_gp1_tmp,((pt)->z)) ;					\
    (gp1)->out_buffer.words_left -= SIZE_V3DI;				\
    (gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
}

#define DEV_GP1_DRAW_3DI(gp1, pt)					\
{									\
    register short	*_gp1_tmp = (gp1)->out_buffer.cur_ptr ;		\
    if ((gp1)->list_type == GP1_VEC_LIST_3DI) {				\
	if (!GP1_ISROOM((gp1), SIZE_V3DI)) {				\
	    long	_tmp_x, _tmp_y, _tmp_z;				\
	    								\
	    _gp1_tmp -= (SIZE_V3DI-1) ;					\
	    GP1_GET_I(_gp1_tmp, _tmp_x) ;				\
	    GP1_GET_I(_gp1_tmp, _tmp_y) ;				\
	    GP1_GET_I(_gp1_tmp, _tmp_z) ;				\
	    Dev_gp1_flush((gp1), TRUE);					\
	    (gp1)->list_type = GP1_VEC_LIST_3DI ;			\
	    _gp1_tmp = (gp1)->out_buffer.cur_ptr ;			\
	    *_gp1_tmp++ = GP1_XF_LINE_INT_3D ;				\
	    *_gp1_tmp++ = GP1_MOVE ;					\
	    GP1_PUT_I(_gp1_tmp, _tmp_x) ;				\
	    GP1_PUT_I(_gp1_tmp, _tmp_y) ;				\
	    GP1_PUT_I(_gp1_tmp, _tmp_z) ;				\
	    (gp1)->out_buffer.words_left -= 1+SIZE_V3DI ;		\
	}								\
	else								\
	{								\
	    *(_gp1_tmp-SIZE_V3DI) &= ~GP1_TERMINATE ;			\
	}								\
	*_gp1_tmp++ = GP1_TERMINATE | GP1_DRAW ;			\
	GP1_PUT_I(_gp1_tmp, (pt)->x) ;					\
	GP1_PUT_I(_gp1_tmp, (pt)->y) ;					\
	GP1_PUT_I(_gp1_tmp, (pt)->z) ;					\
	(gp1)->out_buffer.words_left -= SIZE_V3DI ;			\
	(gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
    }									\
}





/*
 * 3-D Floating vectors
 */


#define	DEV_GP1_VEC_INIT_3DF(gp1, nvert, fit)		\
	    DEV_GP1_VEC_INIT((gp1), (nvert), (fit),	\
		GP1_VEC_LIST_3DF, GP1_XF_LINE_FLT_3D, SIZE_V3DF)



#define	DEV_GP1_VEC_MOVE_3DF(gp1,pt)	DEV_GP1_VEC_3DF(gp1,GP1_MOVE,(pt))
#define	DEV_GP1_VEC_DRAW_3DF(gp1,pt)	DEV_GP1_VEC_3DF(gp1,GP1_DRAW,(pt))


#define	DEV_GP1_VEC_TERM_3DF(gp1)	\
{								\
    (gp1)->list_type = GP1_NO_LIST;				\
    (*((gp1)->out_buffer.cur_ptr - SIZE_V3DF) |= GP1_TERMINATE); \
}




#define	DEV_GP1_START_VEC_3DF(gp1)				\
{								\
    MAKE_ROOM((gp1), 1+2*SIZE_V3DF) ;				\
    (gp1)->list_type = GP1_VEC_LIST_3DF ;			\
    GP1_PUT_SHORT((gp1), GP1_XF_LINE_FLT_3D) ;			\
}

#define	DEV_GP1_VEC_3DF(gp1, flag, pt)					\
{									\
    register short *_gp1_tmp = (gp1)->out_buffer.cur_ptr ;		\
    *_gp1_tmp++ = (flag) ;						\
    GP1_PUT_F(_gp1_tmp,((pt)->x)) ;					\
    GP1_PUT_F(_gp1_tmp,((pt)->y)) ;					\
    GP1_PUT_F(_gp1_tmp,((pt)->z)) ;					\
    GP1_USEUP(gp1, SIZE_V3DF) ;						\
}


#define DEV_GP1_MOVE_3DF(gp1, pt)					\
{									\
    register short	*_gp1_tmp ;					\
    if ((gp1)->list_type != GP1_VEC_LIST_3DF) {				\
	MAKE_ROOM((gp1), 1+2*SIZE_V3DF) ;				\
	(gp1)->list_type = GP1_VEC_LIST_3DF ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_FLT_3D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else if (!GP1_ISROOM((gp1), SIZE_V3DF)) {				\
	Dev_gp1_flush((gp1), TRUE) ;					\
	(gp1)->list_type = GP1_VEC_LIST_3DF ;				\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*_gp1_tmp++ = GP1_XF_LINE_FLT_3D ;				\
	--(gp1)->out_buffer.words_left ;				\
    }									\
    else {								\
	_gp1_tmp = (gp1)->out_buffer.cur_ptr ;				\
	*(_gp1_tmp-SIZE_V3DF) &= ~GP1_TERMINATE;			\
    }									\
    *_gp1_tmp++ = GP1_TERMINATE | GP1_MOVE ;				\
    GP1_PUT_F(_gp1_tmp,((pt)->x)) ;					\
    GP1_PUT_F(_gp1_tmp,((pt)->y)) ;					\
    GP1_PUT_F(_gp1_tmp,((pt)->z)) ;					\
    (gp1)->out_buffer.words_left -= SIZE_V3DF;				\
    (gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
}


#define DEV_GP1_DRAW_3DF(gp1, pt)					\
{									\
    register short	*_gp1_tmp = (gp1)->out_buffer.cur_ptr ;		\
    if ((gp1)->list_type == GP1_VEC_LIST_3DF) {				\
	if (!GP1_ISROOM((gp1), SIZE_V3DF)) {				\
	    float	_tmp_x, _tmp_y, _tmp_z;				\
	    								\
	    _gp1_tmp -= (SIZE_V3DF-1) ;					\
	    GP1_GET_F(_gp1_tmp, _tmp_x) ;				\
	    GP1_GET_F(_gp1_tmp, _tmp_y) ;				\
	    GP1_GET_F(_gp1_tmp, _tmp_z) ;				\
	    Dev_gp1_flush((gp1), TRUE);					\
	    (gp1)->list_type = GP1_VEC_LIST_3DF ;			\
	    _gp1_tmp = (gp1)->out_buffer.cur_ptr ;			\
	    *_gp1_tmp++ = GP1_XF_LINE_FLT_3D ;				\
	    *_gp1_tmp++ = GP1_MOVE ;					\
	    GP1_PUT_F(_gp1_tmp, _tmp_x) ;				\
	    GP1_PUT_F(_gp1_tmp, _tmp_y) ;				\
	    GP1_PUT_F(_gp1_tmp, _tmp_z) ;				\
	    (gp1)->out_buffer.words_left -= 1+SIZE_V3DF ;		\
	}								\
	else								\
	{								\
	    *(_gp1_tmp-SIZE_V3DF) &= ~GP1_TERMINATE ;			\
	}								\
	*_gp1_tmp++ = GP1_TERMINATE | GP1_DRAW ;			\
	GP1_PUT_F(_gp1_tmp, (pt)->x) ;					\
	GP1_PUT_F(_gp1_tmp, (pt)->y) ;					\
	GP1_PUT_F(_gp1_tmp, (pt)->z) ;					\
	(gp1)->out_buffer.words_left -= SIZE_V3DF ;			\
	(gp1)->out_buffer.cur_ptr = _gp1_tmp ;				\
    }									\
}







/*
 * TRANSFORM POINTS
 */


#define	DEV_GP1_MUL_POINTS(gp1)					\
{								\
	GP1_PUT_SHORT_B((gp1)->tmp_buffer, GP1_EOCL) ;		\
	gp1_post((gp1)->shmem, (gp1)->tmp_buffer.bufoffset, (gp1)->fd) ; \
	(gp1)->tmp_buffer.cur_ptr = (gp1)->bound_ptr + 1 ;	\
}



/*
 * 2-D Integer points
 */


#define	DEV_GP1_MUL_ADD_2DI(gp1, x, y)				\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	++(*(gp1)->bound_ptr) ;					\
	GP1_PUT_I(_ptr,(x)) ;					\
	GP1_PUT_I(_ptr,(y)) ;					\
	*_ptr++ = -1 ;						\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T2DI ;		\
}


#define	DEV_GP1_MUL_GET_2DI(gp1, x, y)				\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	if(*(_ptr+SIZE_T2DI-1) != 0)				\
	    gp1_wait0(_ptr+SIZE_T2DI-1, (gp1)->fd) ;		\
	GP1_GET_I(_ptr,(x)) ;					\
	GP1_GET_I(_ptr,(y)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T2DI ;		\
}




/*
 * 2-D floating points
 */


#define	DEV_GP1_MUL_ADD_2DF(gp1, x, y)				\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	++(*(gp1)->bound_ptr) ;					\
	GP1_PUT_F(_ptr,(x)) ;					\
	GP1_PUT_F(_ptr,(y)) ;					\
	*_ptr++ = -1 ;						\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T2DF ;		\
}


#define	DEV_GP1_MUL_GET_2DF(gp1, x, y)				\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	if(*(_ptr+SIZE_T2DF-1) != 0)				\
	    gp1_wait0(_ptr+SIZE_T2DF-1, (gp1)->fd) ;		\
	GP1_GET_F(_ptr,(x)) ;					\
	GP1_GET_F(_ptr,(y)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T2DF ;		\
}




/*
 * 3-D Integer points
 */


#define	DEV_GP1_MUL_ADD_3DI(gp1, x, y, z, w)			\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	++(*(gp1)->bound_ptr) ;					\
	GP1_PUT_I(_ptr,(x)) ;					\
	GP1_PUT_I(_ptr,(y)) ;					\
	GP1_PUT_I(_ptr,(z)) ;					\
	GP1_PUT_I(_ptr,(w)) ;					\
	*_ptr++ = -1 ;						\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T3DI ;		\
}


#define	DEV_GP1_MUL_GET_3DI(gp1, x, y, z, w)			\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	if(*(_ptr+SIZE_T3DI-1) != 0)				\
	    gp1_wait0(_ptr+SIZE_T3DI-1, (gp1)->fd) ;		\
	GP1_GET_I(_ptr,(x)) ;					\
	GP1_GET_I(_ptr,(y)) ;					\
	GP1_GET_I(_ptr,(z)) ;					\
	GP1_GET_I(_ptr,(w)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T3DI ;		\
}




/*
 * 3-D floating points
 */


#define	DEV_GP1_MUL_ADD_3DF(gp1, x, y, z, w)			\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	++(*(gp1)->bound_ptr) ;					\
	GP1_PUT_F(_ptr,(x)) ;					\
	GP1_PUT_F(_ptr,(y)) ;					\
	GP1_PUT_F(_ptr,(z)) ;					\
	GP1_PUT_F(_ptr,(w)) ;					\
	*_ptr++ = -1 ;						\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T3DF ;		\
}


#define	DEV_GP1_MUL_GET_3DF(gp1, x, y, z, w)			\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	if(*(_ptr+SIZE_T3DF-1) != 0)				\
	    gp1_wait0(_ptr+SIZE_T3DF-1, (gp1)->fd) ;		\
	GP1_GET_F(_ptr,(x)) ;					\
	GP1_GET_F(_ptr,(y)) ;					\
	GP1_GET_F(_ptr,(z)) ;					\
	GP1_GET_F(_ptr,(w)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_T3DF ;		\
}













/*
 * Transform a polygon.
 */
#define	DEV_GP1_XPOLY_BEGIN(gp1, n_bounds, n_vertices, fit, size, cmd)	\
{   									\
    register	int	GP1_total_words;				\
    register	short	*GP1_ptr;					\
									\
    /* if current matrix is in the pipe, flush it through */		\
    if((gp1)->flags & 1 << (gp1)->cur_matrix)				\
	Dev_gp1_flush((gp1), TRUE) ;					\
									\
    if((gp1)->tmp_buffer.gpbuffer == NULL)				\
	_gp1_alloc_buffers((gp1), &(gp1)->tmp_buffer) ;			\
    else								\
    {									\
	GP1_INT_BUFCLEAR((gp1), (gp1)->tmp_buffer) ;			\
    }									\
    									\
    GP1_total_words = n_vertices * (size) * sizeof(float)/sizeof(short); \
    GP1_total_words += 3 + n_bounds ;					\
    if (GP1_total_words > (GP1_WORDS_PER_BLOCK * (gp1)->n_buffers - 8))	\
	fit = 0;	/* cannot fit polygon in cmd buffers */		\
    else {								\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr ;				\
	*GP1_ptr++ = cmd ;						\
	(gp1)->bound_ptr = GP1_ptr ;					\
	*GP1_ptr = (n_bounds) ;						\
	GP1_USEUP_B((gp1)->tmp_buffer,2+(n_bounds)) ;			\
	fit = 1;							\
    }									\
}


#define DEV_GP1_XPOLY_ADD_BOUND(gp1)				\
    *++((gp1)->bound_ptr) = 0;



#define DEV_GP1_XPOLY_XFORM(gp1, n_bounds)				\
{									\
	register	short	*GP1_ptr;				\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr ;				\
	GP1_PUT_SHORT_B((gp1)->tmp_buffer,(gp1)->tmp_buffer.words_left) ; \
	GP1_PUT_SHORT_B((gp1)->tmp_buffer,GP1_EOCL) ;			\
	gp1_post((gp1)->shmem, (gp1)->tmp_buffer.bufoffset, (gp1)->fd) ; \
	gp1_wait0(GP1_ptr, (gp1)->fd);					\
	n_bounds = *(gp1)->tmp_buffer.cur_ptr++ ;			\
	(gp1)->bound_ptr = (gp1)->tmp_buffer.cur_ptr ;			\
	GP1_USEUP_B((gp1)->tmp_buffer, n_bounds) ;			\
}


#define DEV_GP1_XPOLY_GET_BOUND(gp1, bound)				\
    bound = *(gp1)->bound_ptr++;






#define	DEV_GP1_XPOLY_3DF_BEGIN(gp1, n_bounds, n_vertices, fit)		\
		DEV_GP1_XPOLY_BEGIN(gp1, n_bounds, n_vertices, fit,	\
			3, GP1_PROC_PGON_FLT_3D)


#define DEV_GP1_XPOLY_3DF_ADD_BOUND	DEV_GP1_XPOLY_ADD_BOUND

#define DEV_GP1_XPOLY_3DF_ADD_VERTEX(gp1, x, y, z)		\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	GP1_PUT_F(_ptr,(x)) ;					\
	GP1_PUT_F(_ptr,(y)) ;					\
	GP1_PUT_F(_ptr,(z)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_P3DF ;		\
	++(*(gp1)->bound_ptr) ;					\
}


#define DEV_GP1_XPOLY_3DF_XFORM		DEV_GP1_XPOLY_XFORM

#define DEV_GP1_XPOLY_3DF_GET_BOUND	DEV_GP1_XPOLY_GET_BOUND

#define DEV_GP1_XPOLY_3DF_GET_VERTEX	DEV_GP1_XPOLY_3DI_GET_VERTEX

#define DEV_GP1_XPOLY_3DF_END





#define	DEV_GP1_XPOLY_2DF_BEGIN(gp1, n_bounds, n_vertices, fit)		\
		DEV_GP1_XPOLY_BEGIN(gp1, n_bounds, n_vertices, fit,	\
			2, GP1_PROC_PGON_FLT_2D)


#define DEV_GP1_XPOLY_2DF_ADD_BOUND	DEV_GP1_XPOLY_ADD_BOUND

#define DEV_GP1_XPOLY_2DF_ADD_VERTEX(gp1, x, y)			\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	GP1_PUT_F(_ptr,(x)) ;					\
	GP1_PUT_F(_ptr,(y)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_P2DF ;		\
	++(*(gp1)->bound_ptr) ;					\
}


#define DEV_GP1_XPOLY_2DF_XFORM		DEV_GP1_XPOLY_XFORM

#define DEV_GP1_XPOLY_2DF_GET_BOUND	DEV_GP1_XPOLY_GET_BOUND

#define DEV_GP1_XPOLY_2DF_GET_VERTEX	DEV_GP1_XPOLY_2DI_GET_VERTEX

#define DEV_GP1_XPOLY_2DF_END





#define	DEV_GP1_XPOLY_3DI_BEGIN(gp1, n_bounds, n_vertices, fit)		\
		DEV_GP1_XPOLY_BEGIN(gp1, n_bounds, n_vertices, fit,	\
			3, GP1_PROC_PGON_INT_3D)


#define DEV_GP1_XPOLY_3DI_ADD_BOUND	DEV_GP1_XPOLY_ADD_BOUND

#define DEV_GP1_XPOLY_3DI_ADD_VERTEX(gp1, x, y, z)		\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	GP1_PUT_I(_ptr,(x)) ;					\
	GP1_PUT_I(_ptr,(y)) ;					\
	GP1_PUT_I(_ptr,(z)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_P3DI ;		\
	++(*(gp1)->bound_ptr) ;					\
}


#define DEV_GP1_XPOLY_3DI_XFORM		DEV_GP1_XPOLY_XFORM

#define DEV_GP1_XPOLY_3DI_GET_BOUND	DEV_GP1_XPOLY_GET_BOUND

#define DEV_GP1_XPOLY_3DI_GET_VERTEX(gp1, x, y, z)			\
{									\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;		\
	GP1_GET_I(_ptr, x) ;						\
	GP1_GET_I(_ptr, y) ;						\
	GP1_GET_I(_ptr, z) ;						\
	GP1_USEUP_B((gp1)->tmp_buffer, 3*sizeof(long)/sizeof(short)) ;	\
}

#define DEV_GP1_XPOLY_3DI_END





#define	DEV_GP1_XPOLY_2DI_BEGIN(gp1, n_bounds, n_vertices, fit)		\
		DEV_GP1_XPOLY_BEGIN(gp1, n_bounds, n_vertices, fit,	\
			2, GP1_PROC_PGON_INT_2D)


#define DEV_GP1_XPOLY_2DI_ADD_BOUND	DEV_GP1_XPOLY_ADD_BOUND

#define DEV_GP1_XPOLY_2DI_ADD_VERTEX(gp1, x, y)			\
{								\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;	\
	GP1_PUT_I(_ptr,(x)) ;					\
	GP1_PUT_I(_ptr,(y)) ;					\
	(gp1)->tmp_buffer.cur_ptr += SIZE_P2DI ;		\
	++(*(gp1)->bound_ptr) ;					\
}


#define DEV_GP1_XPOLY_2DI_XFORM		DEV_GP1_XPOLY_XFORM

#define DEV_GP1_XPOLY_2DI_GET_BOUND	DEV_GP1_XPOLY_GET_BOUND

#define DEV_GP1_XPOLY_2DI_GET_VERTEX(gp1, x, y)				\
{									\
	register short *_ptr = (gp1)->tmp_buffer.cur_ptr ;		\
	GP1_GET_I(_ptr, x) ;						\
	GP1_GET_I(_ptr, y) ;						\
	GP1_USEUP_B((gp1)->tmp_buffer, 2*sizeof(long)/sizeof(short)) ;	\
}

#define DEV_GP1_XPOLY_2DI_END







/*
 * transform a vector list
 */
#define	DEV_GP1_XVECTOR_BEGIN(gp1, fit, size, cmd)			\
{   									\
	register	short	*GP1_ptr;				\
									\
	/* if current matrix is in the pipe, flush it through */	\
	if((gp1)->flags & 1 << (gp1)->cur_matrix)			\
	    Dev_gp1_flush((gp1), TRUE) ;				\
									\
	if((gp1)->tmp_buffer.gpbuffer == NULL)				\
	    _gp1_alloc_buffers((gp1), &(gp1)->tmp_buffer) ;		\
	else								\
	{								\
	    GP1_INT_BUFCLEAR(gp1, (gp1)->tmp_buffer) ;			\
	}								\
									\
	/* find # of vectors that will with in the buffer */		\
	fit = ((gp1)->tmp_buffer.words_left - 2) / ((size) + 2);	\
									\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr ;				\
	*GP1_ptr++ = cmd ;						\
	(gp1)->bound_ptr = GP1_ptr ;					\
	*GP1_ptr++ = 0 ;						\
	(gp1)->tmp_buffer.cur_ptr = GP1_ptr ;				\
}


#define DEV_GP1_XVECTOR_XFORM(gp1)					\
{									\
	*(gp1)->tmp_buffer.cur_ptr++ = GP1_EOCL ;			\
	gp1_post((gp1)->shmem, (gp1)->tmp_buffer.bufoffset, (gp1)->fd) ; \
	(gp1)->tmp_buffer.cur_ptr = (gp1)->bound_ptr + 1 ;		\
}





/*
 * transform 2-d integer vectors
 */


#define DEV_GP1_XVECTOR_2DI_BEGIN(gp1, fit)				\
		DEV_GP1_XVECTOR_BEGIN((gp1), (fit),			\
			4*sizeof(long)/sizeof(short), GP1_PROC_LINE_INT_2D)


#define DEV_GP1_XVECTOR_2DI_ADD_VECTOR( gp1, x1, y1, x2, y2)		\
{									\
	register	short	*GP1_ptr;				\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr + 1 ; /* skip clip flag */	\
	GP1_PUT_F(GP1_ptr, (x1));					\
	GP1_PUT_F(GP1_ptr, (y1));					\
	GP1_PUT_F(GP1_ptr, (x2));					\
	GP1_PUT_F(GP1_ptr, (y2));					\
	*GP1_ptr++ = 0xff;	/* set the ready flag to non-zero */	\
	(gp1)->tmp_buffer.cur_ptr = GP1_ptr ;				\
	(*(gp1)->bound_ptr)++;						\
}


#define DEV_GP1_XVECTOR_2DI_XFORM	DEV_GP1_XVECTOR_XFORM

#define DEV_GP1_XVECTOR_2DI_GET_VECTOR(gp1, clip, x1, y1, x2, y2)	\
{									\
	register short *GP1_ptr ;					\
	register short *GP1_ready ;					\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr ;				\
	GP1_ready = GP1_ptr + 1+4*sizeof(long)/sizeof(short) ;		\
	if(*GP1_ready != 0)						\
	    gp1_wait0(GP1_ready, (gp1)->fd);				\
	clip = *GP1_ptr++;						\
	GP1_GET_I(GP1_ptr, (x1));					\
	GP1_GET_I(GP1_ptr, (y1));					\
	GP1_GET_I(GP1_ptr, (x2));					\
	GP1_GET_I(GP1_ptr, (y2));					\
	(gp1)->tmp_buffer.cur_ptr = GP1_ptr + 1 ;			\
}

#define DEV_GP1_XVECTOR_2DI_END





/*
 * transform 2-d floating vectors
 */


#define DEV_GP1_XVECTOR_2DF_BEGIN(gp1, fit)				\
		DEV_GP1_XVECTOR_BEGIN((gp1), (fit),			\
			4*sizeof(float)/sizeof(short), GP1_PROC_LINE_FLT_2D)


#define DEV_GP1_XVECTOR_2DF_ADD_VECTOR( gp1, x1, y1, x2, y2)		\
{									\
	register	short	*GP1_ptr;				\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr + 1 ; /* skip clip flag */	\
	GP1_PUT_F(GP1_ptr, (x1));					\
	GP1_PUT_F(GP1_ptr, (y1));					\
	GP1_PUT_F(GP1_ptr, (x2));					\
	GP1_PUT_F(GP1_ptr, (y2));					\
	*GP1_ptr++ = 0xff;	/* set the ready flag to non-zero */	\
	(gp1)->tmp_buffer.cur_ptr = GP1_ptr ;				\
	(*(gp1)->bound_ptr)++;						\
}


#define DEV_GP1_XVECTOR_2DF_XFORM	DEV_GP1_XVECTOR_XFORM

#define DEV_GP1_XVECTOR_2DF_GET_VECTOR	DEV_GP1_XVECTOR_2DI_GET_VECTOR

#define DEV_GP1_XVECTOR_2DF_END







/*
 * transform 3-d integer vectors
 */


#define DEV_GP1_XVECTOR_3DI_BEGIN(gp1, fit)				\
		DEV_GP1_XVECTOR_BEGIN((gp1), (fit),			\
			6*sizeof(long)/sizeof(short), GP1_PROC_LINE_INT_3D)


#define DEV_GP1_XVECTOR_3DI_ADD_VECTOR( gp1, x1, y1, z1, x2, y2, z2)	\
{									\
	register	short	*GP1_ptr;				\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr + 1 ; /* skip clip flag */	\
	GP1_PUT_F(GP1_ptr, (x1));					\
	GP1_PUT_F(GP1_ptr, (y1));					\
	GP1_PUT_F(GP1_ptr, (z1));					\
	GP1_PUT_F(GP1_ptr, (x2));					\
	GP1_PUT_F(GP1_ptr, (y2));					\
	GP1_PUT_F(GP1_ptr, (z2));					\
	*GP1_ptr++ = 0xff;	/* set the ready flag to non-zero */	\
	(gp1)->tmp_buffer.cur_ptr = GP1_ptr ;				\
	(*(gp1)->bound_ptr)++;						\
}


#define DEV_GP1_XVECTOR_3DI_XFORM	DEV_GP1_XVECTOR_XFORM

#define DEV_GP1_XVECTOR_3DI_GET_VECTOR(gp1, clip, x1, y1, z1, x2, y2, z2) \
{									\
	register short *GP1_ptr ;					\
	register short *GP1_ready ;					\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr ;				\
	GP1_ready = GP1_ptr + 1+6*sizeof(long)/sizeof(short) ;		\
	if(*GP1_ready != 0)						\
	    gp1_wait0(GP1_ready, (gp1)->fd);				\
	clip = *GP1_ptr++;						\
	GP1_GET_I(GP1_ptr, (x1));					\
	GP1_GET_I(GP1_ptr, (y1));					\
	GP1_GET_I(GP1_ptr, (z1));					\
	GP1_GET_I(GP1_ptr, (x2));					\
	GP1_GET_I(GP1_ptr, (y2));					\
	GP1_GET_I(GP1_ptr, (z2));					\
	(gp1)->tmp_buffer.cur_ptr = GP1_ptr + 1 ;			\
}

#define DEV_GP1_XVECTOR_3DI_END





/*
 * transform 3-d floating vectors
 */


#define DEV_GP1_XVECTOR_3DF_BEGIN(gp1, fit)				\
		DEV_GP1_XVECTOR_BEGIN((gp1), (fit),			\
			6*sizeof(float)/sizeof(short), GP1_PROC_LINE_FLT_3D)


#define DEV_GP1_XVECTOR_3DF_ADD_VECTOR( gp1, x1, y1, z1, x2, y2, z2)	\
{									\
	register	short	*GP1_ptr;				\
	GP1_ptr = (gp1)->tmp_buffer.cur_ptr + 1 ; /* skip clip flag */	\
	GP1_PUT_F(GP1_ptr, (x1));					\
	GP1_PUT_F(GP1_ptr, (y1));					\
	GP1_PUT_F(GP1_ptr, (z1));					\
	GP1_PUT_F(GP1_ptr, (x2));					\
	GP1_PUT_F(GP1_ptr, (y2));					\
	GP1_PUT_F(GP1_ptr, (z2));					\
	*GP1_ptr++ = 0xff;	/* set the ready flag to non-zero */	\
	(gp1)->tmp_buffer.cur_ptr = GP1_ptr ;				\
	(*(gp1)->bound_ptr)++;						\
}


#define DEV_GP1_XVECTOR_3DF_XFORM	DEV_GP1_XVECTOR_XFORM

#define DEV_GP1_XVECTOR_3DF_GET_VECTOR	DEV_GP1_XVECTOR_3DI_GET_VECTOR

#define DEV_GP1_XVECTOR_3DF_END




	int	vpdbg ;


#endif	DEV_GP1_INTERNAL
