#ifndef lint
static	char sccsid[] = "@(#)dev_gp1_mul_3df.c 2.1 87/10/28 SMI";
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc.
 */


#include "dev_gp1_internal.h"


/*
 *		TRANSFORM ROUTINES
 *
 * Dev_gp1_mul_init_3df(gp1)			open transform block
 *						return # points that will fit
 *
 * Dev_gp1_mul_add_3df(gp1, x, y, z, w)		add a point to transform block
 *	float	x,y,z ;
 *
 * Dev_gp1_mul_points(gp)			submit transform block
 *
 * Dev_gp1_mul_get_3df(gp1, &x, &y, &z, &w)	get a transformed point
 *
 * Dev_gp1_mul_term(gp)				close transform block
 *
 * Dev_gp1_mul_3df(gp1, npts, pts, retain)	transform an array of points
 *
 */



int
Dev_gp1_mul_init_3df(gp)
register GP1	*gp ;
{
	/* if current matrix is in the pipe, flush it through */

	if(gp->flags & 1 << gp->cur_matrix)
	  Dev_gp1_flush(gp,1) ;

	if(gp->tmp_buffer.gpbuffer == NULL)
	    _gp1_alloc_buffers(gp, &gp->tmp_buffer) ;
	else
	{
	    GP1_INT_BUFCLEAR(gp, gp->tmp_buffer) ;
	}

	GP1_PUT_SHORT_B(gp->tmp_buffer, GP1_MUL_POINT_FLT_3D) ;
	gp->bound_ptr = gp->tmp_buffer.cur_ptr ;
	GP1_PUT_SHORT_B(gp->tmp_buffer, 0) ;


	return (gp->tmp_buffer.words_left / SIZE_T3DF);
}



void
Dev_gp1_mul_add_3df(gp1, x, y, z, w)
register GP1	*gp1 ;
	float	x,y,z,w ;
{
	DEV_GP1_MUL_ADD_3DF(gp1, x, y, z, w) ;
}





void
Dev_gp1_mul_points(gp)
register GP1	*gp ;
{
	DEV_GP1_MUL_POINTS(gp) ;
}



void
Dev_gp1_mul_get_3df(gp1, x, y, z, w)
register GP1	*gp1 ;
	float	*x, *y, *z, *w ;
{
	DEV_GP1_MUL_GET_3DF(gp1, *x, *y, *z, *w) ;
}



void
Dev_gp1_mul_term(gp)
register GP1	*gp ;
{
	GP1_INT_BUFCLEAR(gp, gp->tmp_buffer) ;
	GP1_PUT_SHORT_B(gp->tmp_buffer, GP1_EOCL | GP1_FREEBLKS) ;
	GP1_PUT_INT_B(gp->tmp_buffer, gp->tmp_buffer.bitvec) ;
	gp1_post(gp->shmem, gp->tmp_buffer.bufoffset, gp->fd ) ;
	gp->tmp_buffer.gpbuffer = NULL ;
}




void
Dev_gp1_mul_3df(gp1, npts, pts, retain)
register GP1		*gp1 ;
	int		npts ;
register Point3df	*pts ;
	int		retain ;
{
register Point3df	*result ;
register int		fit, cnt ;
	float		dummy ;
static	float		c1 = 1.0 ;


	while(npts > 0)
	{
	    fit = Dev_gp1_mul_init_3df(gp1) ;
	    if(fit > npts)
		fit = npts ;
	    cnt = fit ;
	    result = pts ;
	    while(cnt-- > 0)
	    {
		DEV_GP1_MUL_ADD_3DF(gp1, pts->x, pts->y, pts->z, c1) ;
		++pts ;
	    }
	    DEV_GP1_MUL_POINTS(gp1) ;
	    cnt = fit ;
	    while(cnt-- > 0)
	    {
		DEV_GP1_MUL_GET_3DF(gp1,result->x,result->y,result->z,dummy);
		++result ;
	    }
	    npts -= fit ;
	}

	if(!retain)
	    Dev_gp1_mul_term(gp1) ;
}
