#ifndef lint
static	char sccsid[] = "@(#)dev_gp1_util.c	2.1 10/28/87";
#endif


/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

/*
 * Dev_gp1_ident_mat2(matrix)
 * Dev_gp1_ident_mat(matrix)
 *	build an identity matrix
 *
 * Dev_gp1_xlate2(x,y,matrix)
 * Dev_gp1_xlate(x,y,z,matrix)
 *	build translation transformation matrix.
 *
 * Dev_gp1_rot(theta,matrix2df)
 * Dev_gp1_rotx(theta,matrix3df)
 * Dev_gp1_roty(theta,matrix3df)
 * Dev_gp1_rotz(theta,matrix3df)
 *	build rotation matrix
 *
 * Dev_gp1_scale2(xs,ys,matrix)
 * Dev_gp1_scale(xs,ys,zs,matrix)
 *	build scale matrix
 * 
 * Dev_gp1_persp(ze,zp,matrix)
 *	build perspective matrix.  ze is viewpoint distance, zp is viewplane
 *	distance (typically zero).
 *
 * Dev_gp1_window_mat2(x0,x1,y0,y1,matrix)
 * Dev_gp1_window_mat(x0,x1,y0,y1,z0,z1,matrix)
 *	build windowing matrix.  The resulting matrix maps world coordinate
 *	space as defined by [x0,y0,z0 x1,y1,z1] to the viewport (defined
 *	using Dev_gp1_viewport())
 *
 * Dev_gp1_persp_window_mat(ze,zp,x0,x1,y0,y1,z0,z1,matrix)
 *	full perspective matrix.  Zp is projection plane, x0,x1,y0,y1
 *	are window bounds on projection plane, ze is view position
 *	(on line normal to center of window), z0,z1 are near and far
 *	clipping planes.  All coordinates are in WC.
 *
 * Dev_gp1_point_transform3df(matrix, p)
 *	transform point p by matrix.  GP is not used.
 *
 * Dev_gp1_mul_matrix(m3,m1,m2)
 *	multiply 3 matrices, m3=m1*m2.  GP is not used.
 *
 * Dev_gp1_invert_mat(matrix)
 *	take a 4x4 matrix, invert it, and replace the original
 *	return nonzero if can't invert.
 *
 * Dev_gp1_transpose_mat(matrix)
 *	take a 4x4 matrix, transpose it, and replace the original.
 *
 * Dev_gp1_adjoint_mat(matrix)
 *	take a 4x4 matrix and replace it with it's adjoint
 */


#include <stdio.h>
#include <math.h>
#include "dev_gp1.h"


#ifndef	abs
#define	abs(x)	((x)<0 ? -(x) : (x))
#endif

#define	register


static	Matrix3df Identity = {{1.0,0.0,0.0,0.0},
			      {0.0,1.0,0.0,0.0},
			      {0.0,0.0,1.0,0.0},
			      {0.0,0.0,0.0,1.0}} ;

static	Matrix2df Identity2 =  {{1.0,0.0},
				{0.0,1.0},
				{0.0,0.0}} ;



/*
 * return an identity matrix
 */

void
Dev_gp1_ident_mat2(matrix)
	Matrix2df	matrix ;
{
	bcopy(Identity2, matrix, sizeof(Matrix2df)) ;
}

void
Dev_gp1_ident_mat(matrix)
	Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;
}




/*
 * generate a matrix that will translate coordinates by the vector
 * x,y,z
 */


void
Dev_gp1_xlate2(x,y,matrix)
	float	x,y ;
	Matrix2df	matrix ;
{
	bcopy(Identity2, matrix, sizeof(Matrix2df)) ;

	matrix[2][0] = x ;
	matrix[2][1] = y ;
}

void
Dev_gp1_xlate(x,y,z,matrix)
	float	x,y,z ;
	Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	matrix[3][0] = x ;
	matrix[3][1] = y ;
	matrix[3][2] = z ;
}


/*
 * generate a matrix that will rotate coordinates in 2-d
 * by the angle "theta"
 */

void
Dev_gp1_rot(theta,matrix)
	float	theta ;
	Matrix2df	matrix ;
{
	bcopy(Identity2, matrix, sizeof(Matrix2df)) ;

	matrix[0][0] = matrix[1][1] = cos(theta) ;
	matrix[1][0] = - (matrix[0][1] = sin(theta)) ;
}


/*
 * generate a matrix that will rotate coordinates around the x axis
 * by the angle "theta"
 */

void
Dev_gp1_rotx(theta,matrix)
	float	theta ;
	Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	matrix[1][1] = matrix[2][2] = cos(theta) ;
	matrix[2][1] = - (matrix[1][2] = sin(theta)) ;
}



/*
 * generate a matrix that will rotate coordinates around the y axis
 * by the angle "theta"
 */

void
Dev_gp1_roty(theta,matrix)
	float	theta ;
	Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	matrix[0][0] = matrix[2][2] = cos(theta) ;
	matrix[0][2] = - (matrix[2][0] = sin(theta)) ;
}



/*
 * generate a matrix that will rotate coordinates around the z axis
 * by the angle "theta"
 */

void
Dev_gp1_rotz(theta,matrix)
	float	theta ;
	Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	matrix[0][0] = matrix[1][1] = cos(theta) ;
	matrix[1][0] = - (matrix[0][1] = sin(theta)) ;
}


/*
 * generate a matrix that will scale coordinates by the amounts given
 * by x, y & z
 */

void
Dev_gp1_scale2(x,y,matrix)
	float	x,y ;
	Matrix2df	matrix ;
{
	bcopy(Identity2, matrix, sizeof(Matrix2df)) ;

	matrix[0][0] = x ;
	matrix[1][1] = y ;
}

void
Dev_gp1_scale(x,y,z,matrix)
	float	x,y,z ;
	Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	matrix[0][0] = x ;
	matrix[1][1] = y ;
	matrix[2][2] = z ;
}


/*
 *   viewplane model of perspective.  calling routine specifies the
 *   z-coordinate from which the model is being viewed (ze) and the
 *   z-coordinate of the viewing projection plane (zp, typically 0.0)
 */

void
Dev_gp1_persp(ze,zp,matrix)
	float	ze,zp ;
	Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	if(ze!=zp)
	{
	  matrix[2][3] = -1.0/(zp-ze) ;
	  matrix[3][3] = -ze * matrix[2][3] ;
	  matrix[2][2] = matrix[2][3] + matrix[3][3] ;
	}
}


/*
 * build a matrix that maps the region defined by [x0,y0,z0 x1,y1,z1]
 * to the gp's viewport
 */

void
Dev_gp1_window_mat2(x0,x1,y0,y1,matrix)
	float		x0,x1,y0,y1 ;
register Matrix2df	matrix ;
{
	bcopy(Identity2, matrix, sizeof(Matrix2df)) ;

	matrix[0][0] = 2.0/(x1-x0) ;
	matrix[1][1] = 2.0/(y1-y0) ;
	matrix[2][0] = -1.0 - x0 * matrix[0][0] ;
	matrix[2][1] = -1.0 - y0 * matrix[1][1] ;
}

void
Dev_gp1_window_mat(x0,x1,y0,y1,z0,z1,matrix)
	float		x0,x1,y0,y1,z0,z1 ;
register Matrix3df	matrix ;
{
	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	matrix[0][0] = 2.0/(x1-x0) ;
	matrix[1][1] = 2.0/(y1-y0) ;
	matrix[2][2] = 1.0/(z1-z0) ;
	matrix[3][0] = -1.0 - x0 * matrix[0][0] ;
	matrix[3][1] = -1.0 - y0 * matrix[1][1] ;
	matrix[3][2] = - z0 * matrix[2][2] ;
}




Dev_gp1_persp_window_mat(ze,zp,x0,x1,y0,y1,z0,z1,matrix)
	float		ze,zp,x0,x1,y0,y1,z0,z1 ;
register Matrix3df	matrix ;
{
register float		dist ;


	bcopy(Identity, matrix, sizeof(Matrix3df)) ;

	dist = x1-x0 ;
	matrix[0][0] = 2.0/dist ;
	matrix[3][0] = -(x1+x0)*.5*matrix[0][0] ;

	dist = y1-y0 ;
	matrix[1][1] = 2.0/dist ;
	matrix[3][1] = -(y1+y0)*.5*matrix[1][1] ;

	dist = zp-ze ;
	matrix[2][3] = 1.0/dist ;
	matrix[3][3] = -ze * matrix[2][3] ;

	dist = z1-z0 ;
	matrix[2][2] = (z1*matrix[2][3] + matrix[3][3]) / dist ;
	matrix[3][2] = -z0 * matrix[2][2] ;
}




Dev_gp1_mul_matrix(m3,m1,m2)
	Matrix3df	m1,m2,m3 ;
{
register int	i,j,k ;
register float	*p1,*p2,*p3 ;

	p3 = &m3[0][0] ;
	for(i=0; i<4; ++i)
	  for(j=0; j<4; ++j)
	  {
	    *p3 = 0.0 ;
	    p1 = &m1[i][0] ;
	    p2 = &m2[0][j] ;
	    for(k=0; k<4; ++k)
	    {
	      *p3 += *p1++ * *p2 ;
	      p2 += 4 ;
	    }
	    ++p3 ;
	  }
}





/*
 * invert the given 4x4 matrix
 */

int
Dev_gp1_invert_mat(matrix)
	Matrix3df	matrix ;
{
	float	workmat[4][8] ;
register int	i, j ;
	int	col, row ;
	float	maxval, k ;
	int	indices[4] ;
	int	used[4] ;
register float	*mptr, *wptr ;

	mptr = &matrix[0][0] ;
	wptr = &workmat[0][0] ;
	for(i=0; i<4; ++i)
	{
	  for(j=4; j-- > 0;)
	    *wptr++ = *mptr++ ;
	  for(j=4; j-- > 0;)
	    *wptr++ = 0.0 ;
	  workmat[i][i+4] = 1.0 ;
	  used[i] = 0 ;
	}

	for(col=0; col<4; ++col)
	{
	  /* find row with largest entry in this column */

	  maxval = 0.0 ;
	  for(i=0; i<4; ++i)
	    if( !used[i] && abs(workmat[i][col]) > maxval )
	    {
	      maxval = abs(workmat[i][col]) ;
	      row = i ;
	    }
	  if(maxval == 0.0) return(1) ;
	  indices[col] = row ;
	  used[row] = 1;


	  /* divide through */

	  k = workmat[row][col] ;
	  if(k != 1.0)
	  {
	    wptr = &workmat[row][col] ;
	    for(j=col; j<8; ++j)
	    {
	      if(*wptr != 0.0)
		*wptr /= k ;
	      ++wptr ;
	    }
	  }


	  /* subtract from neighboring rows */

	  for(i=0; i<4; ++i)
	    if(i != row)
	    {
	      k = workmat[i][col] ;
	      if(k != 0.0)
	      {
		mptr = &workmat[i][col] ;
		wptr = &workmat[row][col] ;
		for(j=col; j<8; ++j)
		{
		  if(*wptr != 0.0)
		    *mptr -= *wptr * k ;
		  ++mptr ;
		  ++wptr ;
		}
	      }
	    }
	}

	/* copy back and re-arrange */

	mptr = &matrix[0][0] ;
	for(i=0; i<4; ++i)
	{
	  wptr = &workmat[indices[i]][0] ;
	  wptr += 4 ;
	  for(j=0; j<4; ++j)
	    *mptr++ = *wptr++ ;
	}

	return(0) ;
}




/****
 *
 * Dev_gp1_point_transform3df(matrix, p)
 * Transform Point3df p by given matrix. All calculations are done in floating
 * coordinates. The current transformation matrix is temporarily usurped
 * and replaced by trans. The Point3df structure passed will contain the
 * transformed coordinates.
 *
 ****/
void
Dev_gp1_point_transform3df(matrix, p)
/*    --needs--			 */
	 Matrix3df	*matrix;
register Point3df	*p;	/* POINT to transform */
{
/*    --uses--			*/
static	float	t[4];		/* temporary Matrix structure */

register float	*m;		/* -> input matrix */
register float	*result = &t[0];/* -> temporary vector */
register float	r;		/* =  intermediate results */
register float	f1;		/* =  temporary term */
register float	f2;		/* =  temporary term */
register int	i;		/* =  matrix column */

	m = (float *)matrix;
/*
 *	   for (i=0; i<4; ++i)
 *	     result[i] = p->x * m[0][i] +
 *			 p->y * m[1][i] +
 *			 p->z * m[2][i] +
 *			   1  * m[3][i];
 */

	for (i=0; i<4; ++i)
	 {
	  r = 0.0;
	  if ((f1 = p->x)!=0 && (f2 = *m)!=0)
	     r += f1 * f2;			/* p[0] * m[0][i] + */
	  m += 4;
	  if ((f1 = p->y)!=0 && (f2 = *m)!=0)
	     r += f1 * f2;			/* p[1] * m[1][i] + */
	  m += 4;
	  if ((f1 = p->z)!=0 && (f2 = *m)!=0)
	     r += f1 * f2;			/* p[2] * m[2][i] + */
	  m += 4;
	  if ((f2 = *m)!=0)
	     r += f2;				/*  1   * m[3][i]   */
	  *result++ = r; m -= (12 - 1);		/* result[i] =      */
	 }
/*
 * copy the temporary array to POINT
 */
	p->x = t[0]; p->y = t[1]; p->z = t[2];
/*
 * divide x, y and z by w if w != 1.0
 */
	if (r != 1.0)
	 {
	  p->x /= r;  
	  p->y /= r;  
	  p->z /= r;  
	 }
}




/****
 *
 * Dev_gp1_transpose_mat(matrix)
 * Transpose the row and columns of Matrix3df trans.
 * The transposition is done in place with the minimum amount of
 * work.
 *
 ****/
void
Dev_gp1_transpose_mat(matrix)
/*    --needs--			 */
	 Matrix3df	*matrix;
{
/*    --uses--			*/
register float	*row;		/* -> input matrix accessed along rows */
register float	*col;		/* -> input matrix accessed along columns */
register float	t;		/* =  intermediate results */

	row = col = (float *) matrix;
	row += 1; col += 4;
	t = *row; *row = *col; *col = t;	/* M[0][1] <=> M[1][0] */
	row += 1; col += 4;
	t = *row; *row = *col; *col = t;	/* M[0][2] <=> M[2][0] */
	row += 1; col += 4;
	t = *row; *row = *col; *col = t;	/* M[0][3] <=> M[3][0] */
	row += (1 + 2); col += (-12 + 1 + 8);
	t = *row; *row = *col; *col = t;	/* M[1][2] <=> M[2][1] */
	row += 1; col += 4;
	t = *row; *row = *col; *col = t;	/* M[1][3] <=> M[3][1] */
	row += (1 + 3); col += (-12 + 1 + 12);
	t = *row; *row = *col; *col = t;	/* M[2][3] <=> M[3][2] */
}




/****
 *
 * Dev_gp1_adjoint_mat(matrix)
 * Find the adjoint of this matrix.
 *
 ****/
void
Dev_gp1_adjoint_mat(matrix)
/*    --needs--			 */
	 Matrix3df	matrix;
{
/*    --uses--			*/
	Matrix3df	tm ;
register float	*iptr, *optr ;
	int	i,j ;

	iptr = &matrix[0][0] ;
	optr = &tm[0][0] ;
	for(i=16; --i >= 0;)
	    *optr++ = *iptr++ ;

	Dev_gp1_transpose_mat(tm) ;

	for(i=0; i<4; ++i)
	{
	    register int i0,i1,i2 ;
	    i0 = (i+1)%4 ;
	    i1 = (i+2)%4 ;
	    i2 = (i+3)%4 ;
	    for(j=0; j<4; ++j)
	    {
		register int j0,j1,j2 ;

		j0 = (j+1)%4 ;
		j1 = (j+2)%4 ;
		j2 = (j+3)%4 ;

		matrix[i][j] = 
		      tm[i0][j0]*tm[i1][j1]*tm[i2][j2]
		    + tm[i0][j1]*tm[i1][j2]*tm[i2][j0]
		    + tm[i0][j2]*tm[i1][j0]*tm[i2][j1]
		    - tm[i2][j0]*tm[i1][j1]*tm[i0][j2]
		    - tm[i2][j1]*tm[i1][j2]*tm[i0][j0]
		    - tm[i2][j2]*tm[i1][j0]*tm[i0][j1] ;
		if( (i^j)&1 )
		    matrix[i][j] = -matrix[i][j] ;
	    }
	}
}




/****
 *
 * Dev_gp1_determinant(matrix)
 * Find the determinant of this matrix.
 *
 ****/
float
Dev_gp1_determinant(mat)
/*    --needs--			 */
	 Matrix3df	mat;
{
/*    --uses--			*/
register float	result,t ;
	int	i,j ;

	result = 0 ;
	for(i=0; i<4; ++i)
	{
	    register int i0,i1,i2 ;
	    i0 = (i+1)%4 ;
	    i1 = (i+2)%4 ;
	    i2 = (i+3)%4 ;
	    for(j=0; j<4; ++j)
	    {
		register int j0,j1,j2 ;

		j0 = (j+1)%4 ;
		j1 = (j+2)%4 ;
		j2 = (j+3)%4 ;

		t =   mat[i0][j0]*mat[i1][j1]*mat[i2][j2]
		    + mat[i0][j1]*mat[i1][j2]*mat[i2][j0]
		    + mat[i0][j2]*mat[i1][j0]*mat[i2][j1]
		    - mat[i2][j0]*mat[i1][j1]*mat[i0][j2]
		    - mat[i2][j1]*mat[i1][j2]*mat[i0][j0]
		    - mat[i2][j2]*mat[i1][j0]*mat[i0][j1] ;
		t *= mat[i][j] ;
		if( (i^j)&1 )
		    result -= t ;
		else
		    result += t ;
	    }
	}
	return result ;
}
