#ifndef lint
static	char	sccsid[] = "@(#)dev_pw.c 1.1 88/11/23 SMI" ;
#endif

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

/*
 * This file contains the control and viewport functions for the
 * DEV_PW package.  This code is not supported by Sun Microsystems
 *
 */


/*
 * dev_pw_int_open(pr, n_buffers, error_num)
 *	Allocate a pw_handle, Initialize everything...
 *
 * Dev_pw_select_matrix(pwh, index)
 *	set current matrix, change xform, vector & clipcheck pointers
 *	accordingly
 *
 * Dev_pw_reset_to_defaults(pwh)
 *	reset to defaults
 *
 * Dev_pw_flush(pwh, free_blocks)
 *	update viewscale if window size has changed.
 *
 * Dev_pw_wait(pwh)
 *	flush
 *
 * _dev_pw_maxmask(pwh)
 *	compute maximum permissible write mask
 *
 * _dev_pw_mat_anal(mat)
 *	determine what category a matrix falls into.
 *
 * _dev_pw_viewscale(pwh)
 *	reload window dimensions and recompute mapping scale & offset
 *
 * xform_ident2(pwh,pin,pout)
 * xform_ident3(pwh,pin,pout)
 * xform_xl_2d2(pwh,pin,pout)
 * xform_xl_2d3(pwh,pin,pout)
 * xform_xl_3d2(pwh,pin,pout)
 * xform_xl_3d3(pwh,pin,pout)
 * xform_scale_2d2(pwh,pin,pout)
 * xform_scale_2d3(pwh,pin,pout)
 * xform_scale_3d2(pwh,pin,pout)
 * xform_scale_3d3(pwh,pin,pout)
 * xform_xf_2d2(pwh,pin,pout)
 * xform_xf_2d3(pwh,pin,pout)
 * xform_xf_3d2(pwh,pin,pout)
 * xform_xf_3d3(pwh,pin,pout)
 * xform_persp2(pwh,pin,pout)
 * xform_persp3(pwh,pin,pout)
 *	perform different kinds of matrix multiplies on different kind of
 *	data.  Output is always xyzw
 *
 * _dev_pw_clip_2d(pwh,pout)
 *	2-d, non-perspective clip check.  Does not test Z
 *
 * _dev_pw_clip(pwh,pout)
 *	3-d, non-perspective clip check.  Clips against [-1 1],[-1 1],[0 1]
 *
 * _dev_pw_clip_persp(pwh,pout)
 *	full clip check.  Clips against [-w w],[-w w],[0 w]
 *
 * _dev_pw_wmask(pw, mask)
 *	replaces pw_putattributes(pw, &mask), handles funny-sized color
 *	tables.
 */




#include "dev_pw_internal.h"
#include <stdio.h>

static	float	identity[4][4] = {		/* 3D identity matrix */
    {   1.0,	0.0,	0.0,	0.0},
    {   0.0,	1.0,	0.0,	0.0},
    {   0.0,	0.0,	1.0,	0.0},
    {   0.0,	0.0,	0.0,	1.0}
};

extern	int	(*xforms_2d[])() ;
extern	int	(*xforms_3d[])() ;


extern	char	*malloc() ;

static	void	reset_to_defaults();




/*
 * Allocate a pw_handle and get a static block.
 * Initialize everything...
 */


PW_handle
dev_pw_int_open(pr, n_buffers, error_num)
	struct pixrect	*pr;		/* pixrect  */
	int	n_buffers;		/* dummy */
	int	*error_num;		/* error return value */
{
register PW	*pwh;


    pwh = (PW *) malloc(sizeof(PW));
    if (pwh == (PW *)NULL) {
	*error_num = DEV_PW_ERR_NO_MEMORY;
	return((PW *)NULL);
    }

    pwh->pixr = pr ;
    pwh->pixw = NULL ;
    pwh->pattern_block = NULL ;
    pwh->maxmask = pwh->realmask = pwh->wmask = 1 ;
    pwh->planes = 1 ;
    pwh->pw_3d = 0 ;
    pwh->pw_pickinfo = NULL ;
    pwh->clipid = -1 ;
    pwh->flags = 0 ;
    pwh->r0 = pwh->g0 = pwh->b0 = NULL ;
    pwh->r1 = pwh->g1 = pwh->b1 = NULL ;
    pwh->dbuffer = 0 ;
    pwh->dbctrl = 1 ;
    pwh->tmp_buffer = NULL ;
    pwh->tmp_ptr = NULL ;

    return (pwh);
}






void
Dev_pw_select_matrix(pwh, index)
register PW	*pwh;
register int	index ;
{
extern	int	_dev_pw_vector(), _dev_pw_clip_vector(),
		_dev_pw_persp_vector() ;
extern	int	_dev_pw_point(), _dev_pw_clip_point(),
		_dev_pw_persp_point() ;
	int	_dev_pw_clip_2d(), _dev_pw_clip(), _dev_pw_clip_persp() ;

	pwh->mat_index = index ;
	if( pwh->mtypes[index] != M_PERSP  &&  pwh->clip_planes == 0 )
	{
	  pwh->view_type = V_COMP ;
	  pwh->cur_mat = (Matrix3df *) pwh->comp_mat ;
	  _dev_pw_viewscale(pwh) ;
	  pwh->vec_func = _dev_pw_vector ;
	  pwh->pnt_func = _dev_pw_point ; /* Pell */
	  pwh->clip2 = _dev_pw_clip_2d ;
	  pwh->clip3 = _dev_pw_clip ;
	}
	else
	{
	  if( pwh->mtypes[index] != M_PERSP )
	  {
	    pwh->view_type = V_FULL ;
	    pwh->vec_func = _dev_pw_clip_vector ;
	    pwh->pnt_func = _dev_pw_clip_point ; /* Pell */
	    pwh->clip2 = _dev_pw_clip_2d ;
	    pwh->clip3 = _dev_pw_clip ;
	  }
	  else
	  {
	    pwh->view_type = V_PERSP ;
	    pwh->vec_func = _dev_pw_persp_vector ;
	    pwh->pnt_func = _dev_pw_persp_point ; /* Pell */
	    pwh->clip2 = pwh->clip3 = _dev_pw_clip_persp ;
	  }
	  pwh->cur_mat = (Matrix3df *) pwh->mats[index] ;
	  pwh->mat_type = pwh->mtypes[index] ;
	}
	pwh->xf_func2 = xforms_2d[(int) pwh->mat_type] ;
	pwh->xf_func3 = xforms_3d[(int) pwh->mat_type] ;
}




void
Dev_pw_reset_to_defaults(pwh)
register PW	*pwh;
{
	int	attr;		/* pw attributes */
	int	i,j ;

	pwh->xmin = pwh->ymin = pwh->zmin = 0.0 ;
	pwh->xmax = pwh->ymax = pwh->zmax = 1.0 ;

	if(pwh->pixw)
	{
	    _dev_pw_maxmask(pwh) ;
	    pw_getattributes(pwh->pixw, &pwh->wmask) ;
	}
	else
	{
	    pwh->planes = pwh->pixr->pr_depth ;
	    pwh->maxmask = (1 << pwh->planes) -1 ;
	    pr_getattributes(pwh->pixr, &pwh->wmask) ;
	}
	pwh->realmask = pwh->wmask ;

	pwh->clip_planes = 0 ;
	pwh->color = 1 ;
	pwh->rop = PIX_SRC ;
	pwh->hsr = 0 ;
	pwh->linewidth = 0 ;
	pwh->tex_org.x = 0 ;
	pwh->tex_org.y = 0 ;
	pwh->cur_mat = (Matrix3df *) pwh->comp_mat ;
	bcopy(identity, pwh->mats[0], sizeof(Matrix3df)) ;
	pwh->mtypes[0] = pwh->mat_type = M_IDENTITY ;
	Dev_pw_select_matrix(pwh, 0) ;

	if( pwh->pixw )
	  pwh->clipid = pwh->pixw->pw_clipdata->pwcd_clipid ;
}





void
Dev_pw_flush(pwh, free_blocks)
register PW	*pwh;
 	int	free_blocks;
{
	if( pwh->pixw  && pwh->clipid != pwh->pixw->pw_clipdata->pwcd_clipid )
	{
	  _dev_pw_viewscale(pwh) ;
	  pwh->clipid = pwh->pixw->pw_clipdata->pwcd_clipid ;
	}
}



void
Dev_pw_wait(pwh)
	PW	*pwh;
{
	Dev_pw_flush(pwh,1) ;
}






/*
 * determine the maximum permissible write mask from the colormap
 * size
 */

_dev_pw_maxmask(pwh)
register PW	*pwh ;
{
	struct colormapseg	cms ;
	int	i,j ;
static	short	pat[] = {0} ;

	pw_getcmsdata(pwh->pixw, &cms, NULL) ;
	i = cms.cms_size - 1 ;
	j = 0 ;
	while(i > 0)
	{
	    ++j ;
	    i >>= 1 ;
	}
	pwh->planes = j ;
	pwh->maxmask = (1<<pwh->planes) - 1 ;
	if(pwh->r0)
	    pwh->planes >>= 1 ;
}





Matrix_type
_dev_pw_mat_anal(mat)
register Matrix3df	mat ;
{
register unsigned int	mask = 0 ;
register int		i,j ;
register float		*p1, *p2 ;

	/* build deviation mask.  This is a 16-bit mask that contains a
	   '1' for every entry in the matrix that deviates from the
	   identity matrix.  We can then trivially examine the mask to
	   classify the matrix.  */

	p1 = (float *) mat ;
	p2 = (float *) identity ;
	for(i=0; i<16; ++i)
	{
	  mask <<= 1 ;
	  if( *p1++ != *p2++ )
	    mask |= 1 ;
	}

	if( mask == 0 )
	  return M_IDENTITY ;

	if( (mask & 0xfff3) == 0 )
	  return M_XL_2D ;

	if( (mask & 0xfff1) == 0 )
	  return M_XL_3D ;

	if( (mask & 0x7bf3) == 0 )
	  return M_SCALE_2D ;

	if( (mask & 0x7bd1) == 0 )
	  return M_SCALE_3D ;

	if( (mask & 0x33f3) == 0 )
	  return M_XF_2D ;

	if( (mask & 0x1111) == 0 )
	  return M_XF_3D ;

	else
	  return M_PERSP ;
}








_dev_pw_viewscale(pwh)
register PW	*pwh;
{
register float	x0,x1,y0,y1,z0,z1 ;

	/* convert [(-1,-1,0) (1,1,1)] space to view scale & offset */

	pwh->win_size.r_width =
	    pwh->pixw->pw_clipdata->pwcd_prmulti->pr_size.x ;
	pwh->win_size.r_height =
	    pwh->pixw->pw_clipdata->pwcd_prmulti->pr_size.y ;


	x0 = pwh->xmin * (pwh->win_size.r_width - 1) ;
	x1 = pwh->xmax * (pwh->win_size.r_width - 1) ;
	y0 = pwh->win_size.r_height - pwh->ymin * (pwh->win_size.r_height - 1) ;
	y1 = pwh->win_size.r_height - pwh->ymax * (pwh->win_size.r_height - 1) ;
	z0 = pwh->zmin * 65535.0 ;
	z1 = pwh->zmax * 65535.0 ;

	pwh->view_scl.x = (x1-x0)*.5 ;
	pwh->view_scl.y = (y1-y0)*.5 ;
	pwh->view_scl.z = (z1-z0) ;
	pwh->view_ofs.x = (x1+x0)*.5 ;
	pwh->view_ofs.y = (y1+y0)*.5 ;
	pwh->view_ofs.z = z0 ;

	/*
	 * optimization:
	 * if clipping & perspective are off, compose view matrix with
	 * mapping matrix to save a big step
	 */

	if( pwh->view_type == V_COMP )
	{
	  Matrix3df m0 ;
	  bcopy(identity,m0,sizeof(Matrix3df)) ;
	  m0[0][0] = pwh->view_scl.x ;
	  m0[1][1] = pwh->view_scl.y ;
	  m0[2][2] = pwh->view_scl.z ;
	  m0[3][0] = pwh->view_ofs.x ;
	  m0[3][1] = pwh->view_ofs.y ;
	  m0[3][2] = pwh->view_ofs.z ;
	  Dev_pw_mul_matrix( pwh->comp_mat, pwh->mats[pwh->mat_index], m0) ;
	  pwh->mat_type = _dev_pw_mat_anal(pwh->comp_mat) ;
	}
}



static	int
xform_ident2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = pin->x ;
	pout->y = pin->y ;
	pout->z = 0.0 ;
	pout->w = 1.0 ;
}

static	int
xform_ident3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = pin->x ;
	pout->y = pin->y ;
	pout->z = pin->z ;
	pout->w = 1.0 ;
}




static	int
xform_xl_2d2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = 0.0 ;
	pout->w = 1.0 ;
}

static	int
xform_xl_2d3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = pin->z ;
	pout->w = 1.0 ;
}




static	int
xform_xl_3d2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = (*pwh->cur_mat)[3][2] ;
	pout->w = 1.0 ;
}

static	int
xform_xl_3d3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = pin->z + (*pwh->cur_mat)[3][2] ;
	pout->w = 1.0 ;
}




static	int
xform_scale_2d2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[1][1] * pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = 0.0 ;
	pout->w = 1.0 ;
}

static	int
xform_scale_2d3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[1][1] * pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = 0.0 ;
	pout->w = 1.0 ;
}




static	int
xform_scale_3d2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[1][1] * pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = (*pwh->cur_mat)[3][2] ;
	pout->w = 1.0 ;
}

static	int
xform_scale_3d3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x + (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[1][1] * pin->y + (*pwh->cur_mat)[3][1] ;
	pout->z = (*pwh->cur_mat)[2][2] * pin->z + (*pwh->cur_mat)[3][2] ;
	pout->w = 1.0 ;
}




static	int
xform_xf_2d2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x +
		  (*pwh->cur_mat)[1][0] * pin->y +
		  (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[0][1] * pin->x +
		  (*pwh->cur_mat)[1][1] * pin->y +
		  (*pwh->cur_mat)[3][1] ;
	pout->z = 0.0 ;
	pout->w = 1.0 ;
}

static	int
xform_xf_2d3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x +
		  (*pwh->cur_mat)[1][0] * pin->y +
		  (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[0][1] * pin->x +
		  (*pwh->cur_mat)[1][1] * pin->y +
		  (*pwh->cur_mat)[3][1] ;
	pout->z = 0.0 ;
	pout->w = 1.0 ;
}




static	int
xform_xf_3d2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x +
		  (*pwh->cur_mat)[1][0] * pin->y +
		  (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[0][1] * pin->x +
		  (*pwh->cur_mat)[1][1] * pin->y +
		  (*pwh->cur_mat)[3][1] ;
	pout->z = (*pwh->cur_mat)[0][2] * pin->x +
		  (*pwh->cur_mat)[1][2] * pin->y +
		  (*pwh->cur_mat)[3][2] ;
	pout->w = 1.0 ;
}

static	int
xform_xf_3d3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x +
		  (*pwh->cur_mat)[1][0] * pin->y +
		  (*pwh->cur_mat)[2][0] * pin->z +
		  (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[0][1] * pin->x +
		  (*pwh->cur_mat)[1][1] * pin->y +
		  (*pwh->cur_mat)[2][1] * pin->z +
		  (*pwh->cur_mat)[3][1] ;
	pout->z = (*pwh->cur_mat)[0][2] * pin->x +
		  (*pwh->cur_mat)[1][2] * pin->y +
		  (*pwh->cur_mat)[2][2] * pin->z +
		  (*pwh->cur_mat)[3][2] ;
	pout->w = 1.0 ;
}




static	int
xform_persp2(pwh,pin,pout)
register PW		*pwh ;
register Point2df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x +
		  (*pwh->cur_mat)[1][0] * pin->y +
		  (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[0][1] * pin->x +
		  (*pwh->cur_mat)[1][1] * pin->y +
		  (*pwh->cur_mat)[3][1] ;
	pout->z = (*pwh->cur_mat)[0][2] * pin->x +
		  (*pwh->cur_mat)[1][2] * pin->y +
		  (*pwh->cur_mat)[3][2] ;
	pout->w = (*pwh->cur_mat)[0][3] * pin->x +
		  (*pwh->cur_mat)[1][3] * pin->y +
		  (*pwh->cur_mat)[3][3] ;
}

static	int
xform_persp3(pwh,pin,pout)
register PW		*pwh ;
register Point3df	*pin ;
register Point4df	*pout ;
{
	pout->x = (*pwh->cur_mat)[0][0] * pin->x +
		  (*pwh->cur_mat)[1][0] * pin->y +
		  (*pwh->cur_mat)[2][0] * pin->z +
		  (*pwh->cur_mat)[3][0] ;
	pout->y = (*pwh->cur_mat)[0][1] * pin->x +
		  (*pwh->cur_mat)[1][1] * pin->y +
		  (*pwh->cur_mat)[2][1] * pin->z +
		  (*pwh->cur_mat)[3][1] ;
	pout->z = (*pwh->cur_mat)[0][2] * pin->x +
		  (*pwh->cur_mat)[1][2] * pin->y +
		  (*pwh->cur_mat)[2][2] * pin->z +
		  (*pwh->cur_mat)[3][2] ;
	pout->w = (*pwh->cur_mat)[0][3] * pin->x +
		  (*pwh->cur_mat)[1][3] * pin->y +
		  (*pwh->cur_mat)[2][3] * pin->z +
		  (*pwh->cur_mat)[3][3] ;
}

static	int	(*xforms_2d[])() = {
	  xform_ident2, xform_xl_2d2, xform_xl_3d2, xform_scale_2d2,
	  xform_scale_3d2, xform_xf_2d2, xform_xf_3d2, xform_persp2} ;

static	int	(*xforms_3d[])() = {
	  xform_ident3, xform_xl_2d3, xform_xl_3d3, xform_scale_2d3,
	  xform_scale_3d3, xform_xf_2d3, xform_xf_3d3, xform_persp3} ;



int
_dev_pw_clip_2d(pwh,pout)
register PW		*pwh ;
register Point4df	*pout ;
{
	register int	clip_flag = pwh->clip_planes ;

	if( clip_flag & PW_CLIP_LEFT  &&  pout->x < -1.0 )
	  clip_flag &= ~PW_CLIP_RIGHT ;
	else if( clip_flag & PW_CLIP_RIGHT  &&  pout->x > 1.0 )
	  clip_flag &= ~PW_CLIP_LEFT ;
	else
	  clip_flag &= ~(PW_CLIP_LEFT|PW_CLIP_RIGHT) ;

	if( clip_flag & PW_CLIP_BOTTOM  &&  pout->y < -1.0 )
	  clip_flag &= ~PW_CLIP_TOP ;
	else if( clip_flag & PW_CLIP_TOP  &&  pout->y > 1.0 )
	  clip_flag &= ~PW_CLIP_BOTTOM ;
	else
	  clip_flag &= ~(PW_CLIP_TOP|PW_CLIP_BOTTOM) ;

	return( clip_flag ) ;
}



int
_dev_pw_clip(pwh,pout)
register PW		*pwh ;
register Point4df	*pout ;
{
	register int	clip_flag = pwh->clip_planes ;

	if( clip_flag & PW_CLIP_LEFT  &&  pout->x < -1.0 )
	  clip_flag &= ~PW_CLIP_RIGHT ;
	else if( clip_flag & PW_CLIP_RIGHT  &&  pout->x > 1.0 )
	  clip_flag &= ~PW_CLIP_LEFT ;
	else
	  clip_flag &= ~(PW_CLIP_LEFT|PW_CLIP_RIGHT) ;

	if( clip_flag & PW_CLIP_BOTTOM  &&  pout->y < -1.0 )
	  clip_flag &= ~PW_CLIP_TOP ;
	else if( clip_flag & PW_CLIP_TOP  &&  pout->y > 1.0 )
	  clip_flag &= ~PW_CLIP_BOTTOM ;
	else
	  clip_flag &= ~(PW_CLIP_TOP|PW_CLIP_BOTTOM) ;

	if( clip_flag & PW_CLIP_HITHER  &&  pout->z < 0.0 )
	  clip_flag &= ~PW_CLIP_YON ;
	else if( clip_flag & PW_CLIP_YON  &&  pout->z > 1.0 )
	  clip_flag &= ~PW_CLIP_HITHER ;
	else
	  clip_flag &= ~(PW_CLIP_HITHER|PW_CLIP_YON) ;

	return( clip_flag ) ;
}


int
_dev_pw_clip_persp(pwh,pout)
register PW		*pwh ;
register Point4df	*pout ;
{
	register int	clip_flag = pwh->clip_planes ;

	if( clip_flag & PW_CLIP_LEFT  &&  pout->x < -pout->w )
	  clip_flag &= ~PW_CLIP_RIGHT ;
	else if( clip_flag & PW_CLIP_RIGHT  &&  pout->x > pout->w )
	  clip_flag &= ~PW_CLIP_LEFT ;
	else
	  clip_flag &= ~(PW_CLIP_LEFT|PW_CLIP_RIGHT) ;

	if( clip_flag & PW_CLIP_BOTTOM  &&  pout->y < -pout->w )
	  clip_flag &= ~PW_CLIP_TOP ;
	else if( clip_flag & PW_CLIP_TOP  &&  pout->y > pout->w )
	  clip_flag &= ~PW_CLIP_BOTTOM ;
	else
	  clip_flag &= ~(PW_CLIP_TOP|PW_CLIP_BOTTOM) ;

	if( clip_flag & PW_CLIP_HITHER  &&  pout->z < 0.0 )
	  clip_flag &= ~PW_CLIP_YON ;
	else if( clip_flag & PW_CLIP_YON  &&  pout->z > pout->w )
	  clip_flag &= ~PW_CLIP_HITHER ;
	else
	  clip_flag &= ~(PW_CLIP_HITHER|PW_CLIP_YON) ;

	return( clip_flag ) ;
}







_dev_pw_wmask(pw, mask)
	struct	pixwin	*pw;
	int 		mask;
{
	struct colormapseg	cms;
	int			oldmask;

	(void)pw_getcmsdata(pw, &cms, &oldmask);

#ifdef  planes_fully_implemented 
	pr_putattributes(pw->pw_pixrect, &mask);
#else
	(void)pw_full_putattributes(pw->pw_pixrect, &mask);
#endif  planes_fully_implemented

	if (oldmask != mask)
		(void)pw_getclipping(pw);
}
