#ifndef lint
static	char	sccsid[] = "@(#)dev_pw_dbuf.c 1.1 88/11/23 SMI" ;
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc.
 */


#include "dev_pw_internal.h"


/*
 *	These routines handle color-table double buffering
 *
 *
 *	Routines provided here:
 *
 *
 *	Dev_pw_dbuf_on(pwh, planes, cmsname)
 *		turn on double buffering, len is colormap length
 *
 *	Dev_pw_set_dbuf(pwh, planes, cmsname)
 *		turn on double buffering, 'planes' bitplanes/buffer
 *		(same as Dev_pw_dbuf_on(pwh, 1<<planes, cmsname) )
 *
 *	Dev_pw_sbuf_on(pwh, len, cmsname)
 *		turn single buffering on.
 *
 *	Dev_pw_set_sbuf(pwh, planes, cmsname)
 *		turn single buffering on.
 *		(same as Dev_pw_sbuf_on(pwh, 1<<planes, cmsname) )
 *
 *	Dev_pw_dbuf_putcolormap(pwh, start, length, r,g,b)
 *		build the colortable entries for double buffering and load
 *		them into the pw/pr's color table.
 *
 *	Dev_pw_dbuf_getcolormap(pwh, start, length, r,g,b)
 *		Retrieve the colormap.
 *
 *	Dev_pw_dbuf_control(pwh, mask)
 *		set write enables: 1=front buffer, 2=back buffer, 3=both
 *
 *	Dev_pw_dbuf_write_mask(pwh,mask)
 *		set write mask for double buffering
 *
 *	Dev_pw_dbuf_color(pwh, color)
 *		set color for double buffering
 *
 *	Dev_pw_dbuf_swap(pwh)
 *		swap buffers
 *
 *	Dev_pw_dbuf_get_ctrl(pwh)
 *		return control mask
 *
 *	Dev_pw_dbuf_get_write_mask(pwh)
 *		return write mask
 *
 *	Dev_pw_dbuf_get_color(pwh, color)
 *		return "real" color
 *
 *
 *
 *	Edward A. Falk
 *	Sun Microsystems
 *
 *	June, 1987
 *
 *
 *
 */


/****
 *
 * Constants,  typedefs, externals, globals, statics, macros, block data
 *
 ****/


static	int	counter = 0 ;

static	void	dev_pw_build_cmap() ;
	void	Dev_pw_dbuf_swap() ;
	void	Dev_pw_dbuf_write_mask() ;





static	void
put_dbuf_wmask(pwh)
register PW	*pwh ;
{
register unsigned int	omask = 0 ;
register unsigned int	mask = pwh->wmask ;
register unsigned int	ctrl = pwh->dbctrl ;

	if(pwh->dbuffer == 0)
	{
	    if(ctrl & 1)
		omask |= mask ;
	    if(ctrl & 2)
		omask |= mask << pwh->planes ;
	}
	else
	{
	    if(ctrl & 1)
	      omask |= mask << pwh->planes ;
	    if(ctrl & 2)
	      omask |= mask ;
	}

	pwh->realmask = omask & pwh->maxmask ;

	if(pwh->pixw)
	    _dev_pw_wmask(pwh->pixw, pwh->realmask) ;
	else
	    pr_putattributes(pwh->pixr, &pwh->realmask) ;
}




/*
 * this routine writes out either the '0' or '1' color table, depending
 * on which buffer is currently being displayed
 */
static	void
put_dbuf_ctab(pwh)
register PW	*pwh ;
{
	unsigned char	*r,*g,*b ;

	if(pwh->r0 != NULL)
	{
	    if(pwh->dbuffer == 0)
	    {
		r = pwh->r0; g = pwh->g0 ; b = pwh->b0 ;
	    }
	    else
	    {
		r = pwh->r1; g = pwh->g1 ; b = pwh->b1 ;
	    }

	    if(pwh->pixw)
		pw_putcolormap(pwh->pixw, 0, pwh->rcmaplen, r,g,b) ;
	    else
		pr_putcolormap(pwh->pixr, 0, pwh->rcmaplen, r,g,b) ;

	    put_dbuf_wmask(pwh) ;	/* rebuild write mask */
	}
}




void
Dev_pw_dbuf_on(pwh, len, name)
register PW	*pwh ;
	int	len ;
	char	*name ;
{
	int	olen ;
	unsigned char	*r,*g,*b ;

	if(pwh->r0 == NULL  ||  pwh->cmaplen != len)
	{
	    if(pwh->r0 != NULL)
	    {
		free(pwh->r0) ;
		free(pwh->g0) ;
		free(pwh->b0) ;
		free(pwh->r1) ;
		free(pwh->g1) ;
		free(pwh->b1) ;
	    }

	    olen = pwh->cmaplen ;
	    if( olen <= 0 ) olen = 2 ;
	    pwh->cmaplen = len ;
	    {				/* how many bitplanes is this? */
		register unsigned int i,j ;
		for( i=len-1, j=0; i>0; ++j, i >>= 1 ) ;
		pwh->planes = j ;
	    }
	    pwh->rcmaplen = len << pwh->planes ;
	    pwh->r0 = (unsigned char *) malloc(pwh->rcmaplen) ;
	    pwh->g0 = (unsigned char *) malloc(pwh->rcmaplen) ;
	    pwh->b0 = (unsigned char *) malloc(pwh->rcmaplen) ;
	    pwh->r1 = (unsigned char *) malloc(pwh->rcmaplen) ;
	    pwh->g1 = (unsigned char *) malloc(pwh->rcmaplen) ;
	    pwh->b1 = (unsigned char *) malloc(pwh->rcmaplen) ;

	    pwh->wmask = (1<<pwh->planes) - 1 ;
	    pwh->maxmask = (1<<pwh->planes*2) - 1 ;

	    if(name != NULL)
		strncpy(pwh->cmsname, name, CMS_NAMESIZE) ;
	    else
		sprintf(pwh->cmsname,"dbuf%d.%d.%d",
		    getpid(),pwh->planes,counter++);

	    if(pwh->pixw)
	    {
		pw_getcolormap(pwh->pixw, 0, olen, pwh->r0, pwh->g0, pwh->b0) ;
		pw_setcmsname(pwh->pixw,pwh->cmsname) ;
	    }
	    else
	    {
		pr_getcolormap(pwh->pixr, 0, olen, pwh->r0, pwh->g0, pwh->b0) ;
	    }
	    dev_pw_build_cmap(pwh, 0, olen, pwh->r0, pwh->g0, pwh->b0) ;
	    put_dbuf_ctab(pwh) ;
	}
}  




void
Dev_pw_set_dbuf(pwh, planes, name)
register PW	*pwh ;
	int	planes ;
	char	*name ;
{
	Dev_pw_dbuf_on(pwh, 1<<planes, name) ;
}  




void
Dev_pw_sbuf_on(pwh, len, name)
register PW	*pwh ;
	int	len ;
	char	*name ;
{
	int	olen ;
	unsigned char	r[256],g[256],b[256] ;

	if(pwh->r0 != NULL  ||  pwh->cmaplen != len)
	{
	    if(pwh->r0 != NULL)
	    {
		free(pwh->r0) ;
		free(pwh->g0) ;
		free(pwh->b0) ;
		free(pwh->r1) ;
		free(pwh->g1) ;
		free(pwh->b1) ;
	    }

	    olen = pwh->cmaplen ;
	    if( olen <= 0 ) olen = 2 ;
	    pwh->cmaplen = pwh->rcmaplen = len ;
	    {				/* how many bitplanes is this? */
		register unsigned int i,j ;
		for( i=len-1, j=0; i>0; ++j, i >>= 1 ) ;
		pwh->planes = j ;
	    }
	    pwh->maxmask = pwh->realmask = pwh->wmask = (1<<pwh->planes) - 1 ;
	    pwh->dbctrl = 1 ;

	    if(name != NULL)
		strncpy(pwh->cmsname, name, CMS_NAMESIZE) ;
	    else
		sprintf(pwh->cmsname,"sbuf%d.%d.%d",
		    getpid(),pwh->planes,counter++) ;

	    if(pwh->pixw)
	    {
	      pw_getcolormap(pwh->pixw, 0, olen, r,g,b) ;
	      pw_setcmsname(pwh->pixw,pwh->cmsname) ;
	      pw_putcolormap(pwh->pixw, 0, len, r,g,b) ;
	    }
	    else
	    {
	      pr_getcolormap(pwh->pixr, 0, olen, r,g,b) ;
	      pr_putcolormap(pwh->pixr, 0, len, r,g,b) ;
	    }
	    put_dbuf_wmask(pwh) ;
	}
}  




void
Dev_pw_set_sbuf(pwh, planes, name)
register PW	*pwh ;
	int	planes ;
	char	*name ;
{
	Dev_pw_sbuf_on(pwh, 1<<planes, name) ;
}  




void
Dev_pw_dbuf_putcolormap(pwh, start, length, r,g,b)
register PW		*pwh ;
	int		start ;
	int		length ;
	unsigned char	*r, *g, *b ;
{
	int	len ;

	if(pwh->r0)
	{
	    len = 1 << pwh->planes*2 ;

	    dev_pw_build_cmap(pwh, start, length, r,g,b) ;
	    put_dbuf_ctab(pwh) ;
	}
	else
	{
	    if(pwh->pixw)
		pw_putcolormap(pwh->pixw, start, length, r,g,b) ;
	    else
		pr_putcolormap(pwh->pixr, start, length, r,g,b) ;
	}
}



void
Dev_pw_dbuf_control(pwh, mask)
register PW		*pwh ;
	unsigned int	mask ;
{
	pwh->dbctrl = mask ;
	put_dbuf_wmask(pwh) ;
}



void
Dev_pw_dbuf_write_mask(pwh, mask)
register PW		*pwh ;
register unsigned int	mask ;
{
	pwh->wmask = mask ;
	put_dbuf_wmask(pwh) ;
}




void
Dev_pw_dbuf_color(pwh, color)
register PW		*pwh ;
register unsigned int	color ;
{
	DEV_PW_DBUF_COLOR(pwh,color) ;
}



void
Dev_pw_dbuf_swap(pwh)
register PW	*pwh ;
{
	unsigned char	*r,*g,*b ;

	if(pwh->r0 != NULL)
	{
	    pwh->dbuffer = 1 - pwh->dbuffer ;
	    put_dbuf_ctab(pwh) ;
	}
}




static	void
dev_pw_build_cmap(pwh, start, len, r, g, b)
register PW	*pwh ;
	int	start, len ;
	unsigned char	*r,*g,*b ;
{
	int	len2 ;
register int	i, j ;
register unsigned char	ir,ig,ib ;
register unsigned char	*or,*og,*ob ;

	len2 = 1 << pwh->planes ;

	if(start+len > pwh->cmaplen)
	  return ;

	or = pwh->r0 + start ;
	og = pwh->g0 + start ;
	ob = pwh->b0 + start ;
	for(i=0; i<pwh->cmaplen; ++i)
	{
	  bcopy(r,or, len) ;
	  bcopy(g,og, len) ;
	  bcopy(b,ob, len) ;
	  or += len2 ;
	  og += len2 ;
	  ob += len2 ;
	}

	or = pwh->r1 + start*len2 ;
	og = pwh->g1 + start*len2 ;
	ob = pwh->b1 + start*len2 ;
	for(i=0; i<len; ++i)
	{
	  ir = *r++ ;
	  ig = *g++ ;
	  ib = *b++ ;
	  for(j=0; j<len2; ++j)
	  {
	    *or++ = ir ;
	    *og++ = ig ;
	    *ob++ = ib ;
	  }
	}
}



int
Dev_pw_dbuf_get_ctrl(pwh)
register PW	*pwh ;
{
	return(pwh->dbctrl) ;
}


int
Dev_pw_dbuf_get_write_mask(pwh)
register PW	*pwh ;
{
	return(pwh->wmask) ;
}



int
Dev_pw_dbuf_get_realmask(pwh)
register PW	*pwh ;
{
	return(pwh->realmask) ;
}



int
Dev_pw_dbuf_get_color(pwh,color)
register PW	*pwh ;
	int	color ;
{
	return DEV_PW_DBUF_GET_COLOR(pwh,color) ;
}
