/*	@(#)dev_pw_internal.h 1.1 88/11/23 SMI	*/

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#ifndef	DEV_PW_INTERNAL
#define	DEV_PW_INTERNAL


#include <sys/types.h>
#include <sunwindow/window_hs.h>
#include "dev_pw.h"


#ifndef NULL
#define	NULL	0
#endif


#ifndef	TRUE
#define	TRUE	1
#define	FALSE	0
#endif



typedef	enum {
	  M_IDENTITY = 0,
	  M_XL_2D = 1,
	  M_XL_3D = 2,
	  M_SCALE_2D = 3,
	  M_SCALE_3D = 4,
	  M_XF_2D = 5,
	  M_XF_3D = 6,
	  M_PERSP = 7,
	} Matrix_type ;

typedef enum {
	  V_PERSP,		/* xform, clip, divide, map */
	  V_FULL,		/* xform, clip, map */
	  V_COMP,		/* xform only */
	} View_type ;

typedef struct {
	  float	x,y,z,w ;
	} Point4df ;




typedef struct _PW {
	  Pixrect	*pixr ;		/* pixrect part			*/
	  Pixwin	*pixw ;		/* pixwin, if any		*/
	  Point3df	cdp ;		/* current draw position	*/
	  Point4df	cdp_xf ;	/* current draw position	*/
	  int		cdp_flag ;	/* clip assist flags for cdp	*/
	  int		color ;		/* current drawing color	*/
	  int		clip_planes ;	/* 3d clip planes mask		*/
	  int		rop ;		/* rop				*/
	  int		hsr ;		/* hidden surfaces (ignored)	*/
	  caddr_t	linestyle ;	/* ignored			*/
	  int		linewidth ;	/* ignored			*/
	  Pixrect	*pattern_block ; /* pattern block		*/
	  Point2df	tex_org ;	/* texture org			*/
	  short		*bound_ptr ;	/* pointer to next bound to add	*/
	  Rect		win_size ;	/* dimensions of screen rect.	*/
	  int		win_org_x, win_org_y;	/* origin of window	*/
	  float		xmin,xmax ;	/* view boundaries for x	*/
	  float		ymin,ymax ;	/* view boundaries for y	*/
	  float		zmin,zmax ;	/* view boundaries for z	*/
	  unsigned int	wmask ;		/* current write mask		*/
	  unsigned int	realmask ;	/* actual write mask		*/
	  unsigned int	maxmask ;	/* maximum permissible wmask	*/
	  int		planes ;	/* # of bit planes/buffer	*/
	  int		pw_3d ;		/* true iff 3-d mode		*/
	  int		pw_pickid ;	/* current pick id		*/
	  short		*pw_pickinfo ;	/* pointer to pick block if any	*/
	  unsigned int	flags ;		/* described below		*/
	  Matrix3df	mats[6] ;	/* matrices			*/
	  Matrix_type	mtypes[6] ;	/* optimization flags		*/
	  int		mat_index ;	/* current matrix #		*/
	  Matrix3df	*cur_mat ;	/* matrix currently in use	*/
	  Matrix_type	mat_type ;	/* flags for current matrix	*/
	  Point3df	view_scl,	/* view transformation		*/
			view_ofs ;
	  View_type	view_type ;	/* viewing/mapping info		*/
	  Matrix3df	comp_mat ;	/* opt. if no clipping or persp. */
	  int		(*xf_func2)() ;	/* function to transform a point */
	  int		(*xf_func3)() ;	/* function to transform a point */
	  int		(*vec_func)() ;	/* draw a vector		*/
	  int		(*pnt_func)() ;	/* draw a point /Pell		*/
	  int		(*clip2)() ;	/* 2-d clip check		*/
	  int		(*clip3)() ;	/* 3-d clip check		*/
	  char		cmsname[CMS_NAMESIZE] ; /* used in double buffering */
	  unsigned int	cmaplen ;	/* requested color map length	*/
	  unsigned int	rcmaplen ;	/* real color map length	*/
	  unsigned char	*r0,*g0,*b0 ;	/* buffer 0 colormap		*/
	  unsigned char	*r1,*g1,*b1 ;	/* buffer 1 colormap		*/
	  int		dbuffer ;	/* currently displayed buffer	*/
	  int		dbctrl ;	/* write control		*/
	  int		clipid ;	/* detecting window size changes */
	  caddr_t	tmp_buffer ;	/* scratch pad for xform routines */
	  caddr_t	tmp_ptr ;	/* pointer into tmp_buffer	*/
	} PW ;


/* flags:
 */
#define	PW_PICK		0x800		/* picking enabled		*/
#define	PW_FLOAT	0x2000		/* most recent point was float	*/








/*	PRIVATE MACRO DEFINITIONS
 *
 * DEV_PW_LOCK(pw)			lock pw's window
 * DEV_PW_UNLOCK(pw)			unlock pw's window
 */



#define DEV_PW_LOCK(pw)		\
	{ if((pw)->pixw)		\
	    pw_lock((pw)->pixw, &(pw)->win_size) }

#define	DEV_PW_UNLOCK(pw)		\
	{ if((pw)->pixw)		\
	    pw_unlock((pw)->pixw) }




/*
 * private externs
 */

extern	PW_handle	dev_pw_int_open() ;
extern	Matrix_type	_dev_pw_mat_anal() ;



/*
 * MACRO DEFINITIONS
 *
 * DEV_PW_SET_COLOR(pw, color)			set color
 * DEV_PW_CLIP(pw, planes)			set clip planes
 * DEV_PW_ROP_MODE(pw, rop)			set rop
 * DEV_PW_HSR(pw, control)			set hidden surface removal
 * DEV_PW_LINSTY(pw, count, pattrn, ofs, ops)	set line style
 * DEV_PW_WIDTH(pw, width)			set line width
 * DEV_PW_DBUF_COLOR(pw, color)			set dbuf color
 * DEV_PW_GET_PW(pw)				return pixwin
 * DEV_PW_GET_PR(pw)				return pixrect
 *
 */










#define	DEV_PW_GET_PW(pw)		((pw)->pixw)

#define	DEV_PW_GET_PR(pw)		((pw)->pixr)







/*
 * Macros for setting attributes.
 */

#define DEV_PW_SET_COLOR(pw, c)		((pw)->color = (c))
#define DEV_PW_SET_CLIP_PLANES(pw, mask)	((pw)->clip_planes = (mask))
#define	DEV_PW_SET_HIDDEN_SURF(pw, type)	((pw)->hsr = (type))
#define	DEV_PW_SET_ROP_MODE(pw, type)		((pw)->rop = (type))
#define	DEV_PW_SET_LINE_WIDTH(pw, width)	((pw)->linewidth = (width))


#define	DEV_PW_DBUF_GET_COLOR(pw, color)	\
		((color) << (pw)->planes  |  (color))


#define	DEV_PW_DBUF_COLOR(pw, color)	\
    DEV_PW_SET_COLOR((pw), (DEV_PW_DBUF_GET_COLOR(pw,color) & 0xff ))


#define DEV_PW_SET_TEXTURE_ORG_SCREEN(pw, px, py)	\
(							\
	(pw)->tex_org.x = (px),				\
	(pw)->tex_org.y = (py)				\
)


#define DEV_PW_SET_TEXTURE_ORG_2DF(pw, x, y)

#define DEV_PW_SET_TEXTURE_ORG_3DF(pw, x, y, z)


#define	DEV_PW_SELECT_TEXTURE_BLOCK(pw, num)







/*
 * Macros for stuffing polygons into the pw.
 */



/*
 * VECTORS
 *
 */

#define	Dev_pw_vec_init_2di(pwh,nvert,fit)	((fit) = (nvert))
#define	Dev_pw_vec_init_2df(pwh,nvert,fit)	((fit) = (nvert))
#define	Dev_pw_vec_init_3di(pwh,nvert,fit)	((fit) = (nvert))
#define	Dev_pw_vec_init_3df(pwh,nvert,fit)	((fit) = (nvert))
#define	Dev_pw_vec_move_2df	Dev_pw_move_2df
#define	Dev_pw_vec_move_2di	Dev_pw_move_2di
#define	Dev_pw_vec_move_3df	Dev_pw_move_3df
#define	Dev_pw_vec_move_3di	Dev_pw_move_3di
#define	Dev_pw_vec_draw_2df	Dev_pw_draw_2df
#define	Dev_pw_vec_draw_2di	Dev_pw_draw_2di
#define	Dev_pw_vec_draw_3df	Dev_pw_draw_3df
#define	Dev_pw_vec_draw_3di	Dev_pw_draw_3di
#define	Dev_pw_vec_term_2df(pwh)
#define	Dev_pw_vec_term_2di(pwh)
#define	Dev_pw_vec_term_3df(pwh)
#define	Dev_pw_vec_term_3di(pwh)





/*
 * TRANSFORM POINTS
 */



/*
 * Transform a polygon.
 */



	int	vpdbg ;


#endif	DEV_PW_INTERNAL
