#include <stdio.h>
#include <suntool/sunview.h>
#include <suntool/panel.h>
#include <suntool/canvas.h>
#include "dev_pw.h"

/* white black red x orange x yellow x green x cyan x blue x magenta x */
/* brown = (192,128,0) */

static	u_char	red1[] =   {255, 000, 255, 255, 255, 255, 255, 128, 000,
			    000, 000, 000, 000, 128, 255} ;
static	u_char	green1[] = {255, 000, 000,  64, 128, 192, 255, 255, 255,
			    255, 255, 128, 000, 000, 000} ;
static	u_char	blue1[] =  {255, 000, 000, 000, 000, 000, 000, 000, 000,
			    128, 255, 255, 255, 255, 255} ;

static	PW_handle	pwh ;

static	Frame	frame ;
static	Panel	panel ;
static	Canvas	canvas ;
static	Pixwin	*pw ;
static	Panel_item	clear_item, view_item, win1_item, win2_item,
			win_nc_item, win_fc_item, persp1_item, persp2_item,
			persp_nc_item, persp_fc_item, all_clip_item ;

static	void		event_proc() ;

static	void		clear_proc(), view_proc() ;


static	char	*help_info ="\
-h		this list\n" ;





main(argc,argv)
	int	argc ;
	char	**argv ;
{
	int	i,j ;
	int	retained = 0 ;


	frame = window_create(NULL, FRAME,
	    FRAME_LABEL, argv[0],
	    FRAME_ARGC_PTR_ARGV, &argc, argv,
	    0 ) ;
	if(frame == NULL)
	  exit(1) ;

	panel = window_create(frame, PANEL,
	    WIN_WIDTH, 500,
	    0) ;
	if(panel == NULL)
	  exit(1) ;

	clear_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE, panel_button_image(panel, "clear", 0, NULL),
	    PANEL_NOTIFY_PROC, clear_proc,
	    0 ) ;


	view_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "overview", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	win1_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "full parallel", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	win2_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "side clip par.", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	win_nc_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "near clip par.", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	win_fc_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "far clip par.", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	persp1_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "full perspective", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	persp2_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "side clip", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	persp_nc_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "near clip", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	persp_fc_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "far clip", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;

	all_clip_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE,
		panel_button_image(panel, "all clip", 0, NULL),
	    PANEL_NOTIFY_PROC, view_proc,
	    0 ) ;



	window_fit_height(panel) ;

	canvas = window_create(frame, CANVAS,
	    WIN_X, 0,
	    WIN_BELOW, panel,
	    WIN_WIDTH, 500,
	    WIN_HEIGHT, 500,
	    WIN_CONSUME_PICK_EVENT, WIN_NO_EVENTS,
	    WIN_CONSUME_KBD_EVENT, WIN_NO_EVENTS,
	    WIN_EVENT_PROC, event_proc,
	    0 ) ;
	if(canvas == NULL)
	  exit(1) ;

	window_fit(frame) ;

	pw = canvas_pixwin(canvas) ;
	if(pw == NULL)
	  exit(1) ;


	while(--argc > 0)
	{
	  ++argv ;

	  if(strcmp(*argv,"-h")==0)
	  {
	    printf(help_info) ;
	    exit(0) ;
	  }

	  else if(**argv == '-')
	    printf("unknown option %s, '-h' for more info\n",*argv) ;
	}


	if( (pwh = Dev_pw_create(pw, 2, &i)) == NULL)
	{
	  fprintf(stderr, "can't get pwh, err=%d\n",i) ;
	  exit(1) ;
	}

	Dev_pw_set_clip_planes(pwh, 0x3f) ;

	window_main_loop(frame) ;
}



static	void
event_proc(win,event,arg)
	Window		win ;
	Event		*event ;
	caddr_t		arg ;
{
}



static	void
clear_proc(item,event)
	Panel_item	item ;
	Event		*event ;
{
	pw_writebackground(pw, 0,0, 500,500, PIX_SRC) ;
}



static	void
draw()
{
	int		i ;
	Matrix3df	viewmat ;
static	Point3df	points[] = {
	  {-1.0,-1.0,-1.0}, { 1.0,-1.0,-1.0}, { 1.0, 1.0,-1.0}, /* cube */
	  {-1.0, 1.0,-1.0}, {-1.0,-1.0,-1.0}, {-1.0,-1.0, 1.0},
	  { 1.0,-1.0, 1.0}, { 1.0, 1.0, 1.0}, {-1.0, 1.0, 1.0},
	  {-1.0,-1.0, 1.0},
	  { 1.0,-1.0,-1.0}, { 1.0,-1.0, 1.0},
	  { 1.0, 1.0,-1.0}, { 1.0, 1.0, 1.0},
	  {-1.0, 1.0,-1.0}, {-1.0, 1.0, 1.0},
	  {-1.0,-1.0,-1.0}, {-1.0, 1.0, 1.0},	/* left side */
	  {-1.0,-1.0, 1.0}, {-1.0, 1.0,-1.0},
	  { 1.0,-1.0,-1.0}, { 1.0, 1.0, 1.0},	/* right side */
	  { 1.0,-1.0, 1.0}, { 1.0, 1.0,-1.0},
	  { 1.0, 1.0, 1.0}, { 0.8, 0.8, 1.0},	/* 'X' at 1,1,1 */
	  { 0.8, 1.0, 1.0}, { 1.0, 0.8, 1.0},
	  {-0.9,-1.0,-1.0}, {-0.9,-0.8,-1.0},	/* '+' at -1,-1,-1 */
	  {-1.0,-0.9,-1.0}, {-0.8,-0.9,-1.0},
	  {-1.0,-1.0,-1.0}, { 1.0, 1.0, 1.0},	/* diagonals through middle */
	  { 1.0,-1.0,-1.0}, {-1.0, 1.0, 1.0},
	  {-1.0, 1.0,-1.0}, { 1.0,-1.0, 1.0},
	  { 1.0, 1.0,-1.0}, {-1.0,-1.0, 1.0}} ;

	Dev_pw_line_3df(pwh, 10, points) ;
	Dev_pw_line_3df(pwh, 2, &points[10]) ;
	Dev_pw_line_3df(pwh, 2, &points[12]) ;
	Dev_pw_line_3df(pwh, 2, &points[14]) ;
	Dev_pw_line_3df(pwh, 2, &points[16]) ;
	Dev_pw_line_3df(pwh, 2, &points[18]) ;
	Dev_pw_line_3df(pwh, 2, &points[20]) ;
	Dev_pw_line_3df(pwh, 2, &points[22]) ;
	Dev_pw_line_3df(pwh, 2, &points[24]) ;
	Dev_pw_line_3df(pwh, 2, &points[26]) ;
	Dev_pw_line_3df(pwh, 2, &points[28]) ;
	Dev_pw_line_3df(pwh, 2, &points[30]) ;
#ifdef	COMMENT
	Dev_pw_line_3df(pwh, 2, &points[32]) ;
	Dev_pw_line_3df(pwh, 2, &points[34]) ;
	Dev_pw_line_3df(pwh, 2, &points[36]) ;
	Dev_pw_line_3df(pwh, 2, &points[38]) ;
#endif	COMMENT
	Dev_pw_flush(pwh, TRUE) ;
}



static	void
view_proc(item,event)
	Panel_item	item ;
	Event		*event ;
{
	Matrix3df	viewmat ;

	Dev_pw_select_matrix(pwh, 0) ;

	if( item == view_item )
	{
	  Dev_pw_persp_window_mat(-4.0,0.0, -1.5,1.5, -1.5,1.5, -1.5,1.5,
			viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, 0.,1., 0.,1., 0.0, 65535.) ;
	  draw() ;

	  printf("Cube is [(-1,-1,-1) (1,1,1)]\n") ;
	  printf("Clip planes are [(-1.5,-1.5,-1.5) (1.5,1.5,1.5)]\n") ;
	  printf("  except for plane being tested which is +/- 0.9\n") ;
	  printf("Projection plane is 0.0, eyepoint is -3.0\n") ;
	}

	if( item == win1_item )
	{
	  Dev_pw_window_mat(-1.5,1.5,-1.5,1.5,-1.5,1.5,viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .1,.4, .6,.9, 0.0, 65535.) ;
	  draw() ;
	}

	else if( item == win2_item )
	{
	  Dev_pw_window_mat(-1.5,.9,-1.5,.9,-1.5,1.5,viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .6,.9, .6,.9, 0.0, 65535.) ;
	  draw() ;
	}

	if( item == win_nc_item )
	{
	  Dev_pw_window_mat(-1.5,1.5,-1.5,1.5,-.9,1.5,viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .1,.4, .1,.4, 0.0, 65535.) ;
	  draw() ;
	}

	else if( item == win_fc_item )
	{
	  Dev_pw_window_mat(-1.5,1.5,-1.5,1.5,-1.5,0.9,viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .6,.9, .1,.4, 0.0, 65535.) ;
	  draw() ;
	}

	else if( item == persp1_item )
	{
	  Dev_pw_persp_window_mat(-3.1,0.0, -1.5,1.5, -1.5,1.5, -1.5,1.5,
	 			 viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .1,.4, .6,.9, 0.0, 65535.) ;
	  draw() ;
	}

	else if( item == persp2_item )
	{
	  Dev_pw_persp_window_mat(-3.1,0.0, -1.5,.9, -1.5,.9, -1.5,1.5,
	 			 viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .6,.9, .6,.9, 0.0, 65535.) ;
	  draw() ;
	}

	else if( item == persp_nc_item )
	{
	  Dev_pw_persp_window_mat(-3.1,0.0, -1.5,1.5, -1.5,1.5, -.9,1.5,
	 			 viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .1,.4, .1,.4, 0.0, 65535.) ;
	  draw() ;
	}

	else if( item == persp_fc_item )
	{
	  Dev_pw_persp_window_mat(-3.1,0.0, -1.5,1.5, -1.5,1.5, -1.5,.9,
	 			 viewmat) ;
	  print_mat(viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .6,.9, .1,.4, 0.0, 65535.) ;
	  draw() ;
	}

	else if( item == all_clip_item )
	{
	  Dev_pw_persp_window_mat(-4.0,0.0, -.9,.9, -.9,.9, -.9,.9,
	 			 viewmat) ;
	  Dev_pw_set_matrix_3d(pwh, 0, viewmat) ;
	  Dev_pw_viewport_3d(pwh, .25,.75, .25,.75, 0.0, 65535.) ;
	  draw() ;
	}
}



print_mat(matrix)
	Matrix3df	matrix ;
{
	int	i,j ;

	for(i=0; i<4; ++i)
	{
	  for(j=0; j<4; ++j)
	    printf("%8.5f ",matrix[i][j]) ;
	  putchar('\n') ;
	}
	putchar('\n') ;
}
