#include <stdio.h>
#include <suntool/sunview.h>
#include <suntool/panel.h>
#include <suntool/canvas.h>
#include "dev_pw.h"

/* white black red x orange x yellow x green x cyan x blue x magenta x */
/* brown = (192,128,0) */

static	u_char	red[16] =   {255, 000, 255, 255, 255, 255, 255, 128,
			     000, 000, 000, 000, 000, 128, 255, 255} ;
static	u_char	green[16] = {255, 000, 000,  64, 128, 192, 255, 255,
			     255, 255, 255, 128, 000, 000, 000, 000} ;
static	u_char	blue[16] =  {255, 000, 000, 000, 000, 000, 000, 000,
			     000, 128, 255, 255, 255, 255, 255, 128} ;

static	PW_handle	pwh ;

static	Frame	frame ;
static	Panel	panel ;
static	Canvas	canvas ;
static	Pixwin	*pw ;
static	Panel_item	draw_item, draw_clip_item, draw_persp_item ;

static	void		draw_proc() ;


static	char	*help_info ="\
-h		this list\n" ;




main(argc,argv)
	int	argc ;
	char	**argv ;
{
	int	i,j ;
	int	retained = 0 ;


	frame = window_create(NULL, FRAME,
	    FRAME_LABEL, argv[0],
	    FRAME_ARGC_PTR_ARGV, &argc, argv,
	    0 ) ;
	if(frame == NULL)
	  exit(1) ;

	panel = window_create(frame, PANEL,
	    WIN_WIDTH, 500,
	    WIN_HEIGHT, 60,
	    0) ;
	if(panel == NULL)
	  exit(1) ;

	draw_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE, panel_button_image(panel, "draw", 0, NULL),
	    PANEL_NOTIFY_PROC, draw_proc,
	    0 ) ;

	draw_clip_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE, panel_button_image(panel, "draw clip", 0, NULL),
	    PANEL_NOTIFY_PROC, draw_proc,
	    0 ) ;

	draw_persp_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE, panel_button_image(panel, "draw persp", 0, NULL),
	    PANEL_NOTIFY_PROC, draw_proc,
	    0 ) ;






	canvas = window_create(frame, CANVAS,
	    WIN_X, 0,
	    WIN_BELOW, panel,
	    WIN_WIDTH, 500,
	    WIN_HEIGHT, 500,
	    CANVAS_RETAINED, FALSE,
	    WIN_CONSUME_PICK_EVENT, WIN_NO_EVENTS,
	    WIN_CONSUME_KBD_EVENT, WIN_NO_EVENTS,
	    0 ) ;
	if(canvas == NULL)
	  exit(1) ;

	window_set(canvas,
	    CANVAS_AUTO_EXPAND, FALSE,
	    CANVAS_AUTO_SHRINK, FALSE,
	    0 ) ;

	window_fit(frame) ;

	pw = canvas_pixwin(canvas) ;
	if(pw == NULL)
	  exit(1) ;


	while(--argc > 0)
	{
	  ++argv ;

	  if(strcmp(*argv,"-h")==0)
	  {
	    printf(help_info) ;
	    exit(0) ;
	  }

	  else if(**argv == '-')
	    printf("unknown option %s, '-h' for more info\n",*argv) ;
	}


	if( (pwh = Dev_pw_create(pw, 2, &i)) == NULL)
	{
	  fprintf(stderr, "can't get pwh, err=%d\n",i) ;
	  exit(1) ;
	}

	Dev_pw_setcmsname(pwh, "polygon") ;
	Dev_pw_putcolormap(pwh, 0, 16, red,green,blue) ;
	Dev_pw_set_write_mask(pwh, 0xf) ;

	Dev_pw_set_clip_planes(pwh, 0x3f) ;

	window_main_loop(frame) ;
}





static	void
draw_proc(item,event)
	Panel_item	item ;
	Event		*event ;
{
	float		theta ;
	int		color ;
	Matrix3df	matrix ;
#ifdef	COMMENT
	float		time, timer_get() ;
#endif	COMMENT
	int		count ;
static	Point3df	points[] = {
	{-.75,0.,0.}, {-.5,.75,0.}, {-.25,.5,0.}, {.25,.5,0.}, {.5,.75,0.},
	{.75,0.,0.}, {.5,-.75,0.}, {.25,-.5,0.}, {-.25,-.5,0.}, {-.5,-.75,0.},
	{-.5,.5,0.}, {-.25,.25,0.}, {.25,.25,0.}, {.5,.5,0.},
	{.5,-.5,0.}, {.25,-.25,0.}, {-.25,-.25,0.}, {-.5,-.5,0.} } ;
static	int		bounds[] = {10,8} ;

#ifdef	COMMENT
static	Point3df	points[] = {
	{-.75,0.,0.}, {0.,.5,0.}, {.75,0.,0.}, {0.,-.5,0.}} ;
static	int		bounds[] = {4} ;
#endif	COMMENT


#ifdef	COMMENT
	timer_start() ;
#endif	COMMENT
	Dev_pw_clear(pwh, 0) ;
	Dev_pw_select_matrix(pwh, 0) ;
	if( item == draw_item )
	  Dev_pw_ident_mat(matrix) ;
	else if( item == draw_clip_item )
	  Dev_pw_window_mat(-.6,.6, -.6,.6, 0.,1., matrix) ;
	else
	  Dev_pw_persp_window_mat(-1.,0.,-.6,.6, -.6,.6, -.6,.6, matrix) ;
	Dev_pw_set_matrix_3d(pwh, 0, matrix) ;

	Dev_pw_viewport_2d(pwh, .25,.75, .25,.75) ;

	if( item != draw_persp_item )
	  Dev_pw_rotz(.0157080, matrix) ;
	else
	  Dev_pw_roty(.0157080, matrix) ;
	Dev_pw_set_matrix_3d(pwh, 1, matrix) ;
	color = 1 ;
	count = 0 ;
	for(count = 0; count < 400; ++count)
	{
	  Dev_pw_set_color(pwh, color) ;
	  Dev_pw_clear(pwh, 0) ;
	  Dev_pw_polygon_3df(pwh, 2, bounds, points, 0) ;
	  Dev_pw_matrix_mul_3d(pwh, 0, 1,0) ;
	  color = ((color-1) % 14) + 2 ;
	}
	Dev_pw_flush(pwh, TRUE) ;
	Dev_pw_wait(pwh) ;
#ifdef	COMMENT
	time = timer_get() ;
	printf("%d polygons in %f seconds = %f pgon/sec\n",
		count, time, (float)count/time ) ;
#endif	COMMENT
}






print_mat(matrix)
	Matrix3df	matrix ;
{
	int	i,j ;

	for(i=0; i<4; ++i)
	{
	  for(j=0; j<4; ++j)
	    printf("%8.5f ",matrix[i][j]) ;
	  putchar('\n') ;
	}
	putchar('\n') ;
}
