#include <stdio.h>
#include <suntool/sunview.h>
#include <suntool/panel.h>
#include <suntool/canvas.h>
#include "dev_pw.h"

/* white black red x orange x yellow x green x cyan x blue x magenta x */
/* brown = (192,128,0) */

static	u_char	red[16] =   {255, 000, 255, 255, 255, 255, 255, 128,
			     000, 000, 000, 000, 000, 128, 255, 255} ;
static	u_char	green[16] = {255, 000, 000,  64, 128, 192, 255, 255,
			     255, 255, 255, 128, 000, 000, 000, 000} ;
static	u_char	blue[16] =  {255, 000, 000, 000, 000, 000, 000, 000,
			     000, 128, 255, 255, 255, 255, 255, 128} ;

static	PW_handle	pwh ;

static	Frame	frame ;
static	Panel	panel ;
static	Canvas	canvas ;
static	Pixwin	*pw ;
static	Panel_item	draw_item, draw_clip_item ;

static	void		draw_proc() ;
static	void		repaint_proc() ;


static	char	*help_info ="\
-h		this list\n" ;





main(argc,argv)
	int	argc ;
	char	**argv ;
{
	int	i,j ;
	int	retained = 0 ;


	frame = window_create(NULL, FRAME,
	    FRAME_LABEL, argv[0],
	    FRAME_ARGC_PTR_ARGV, &argc, argv,
	    0 ) ;
	if(frame == NULL)
	  exit(1) ;

	panel = window_create(frame, PANEL,
	    WIN_WIDTH, 500,
	    WIN_HEIGHT, 60,
	    0) ;
	if(panel == NULL)
	  exit(1) ;

	draw_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE, panel_button_image(panel, "draw", 0, NULL),
	    PANEL_NOTIFY_PROC, draw_proc,
	    0 ) ;

	draw_clip_item = panel_create_item(panel, PANEL_BUTTON,
	    PANEL_LABEL_IMAGE, panel_button_image(panel, "draw clip", 0, NULL),
	    PANEL_NOTIFY_PROC, draw_proc,
	    0 ) ;






	canvas = window_create(frame, CANVAS,
	    WIN_X, 0,
	    WIN_BELOW, panel,
	    WIN_WIDTH, 500,
	    WIN_HEIGHT, 500,
	    CANVAS_RETAINED, FALSE,
	    WIN_CONSUME_PICK_EVENT, WIN_NO_EVENTS,
	    WIN_CONSUME_KBD_EVENT, WIN_NO_EVENTS,
	    CANVAS_REPAINT_PROC, repaint_proc,
	    0 ) ;
	if(canvas == NULL)
	  exit(1) ;

	window_set(canvas,
	    CANVAS_AUTO_EXPAND, FALSE,
	    CANVAS_AUTO_SHRINK, FALSE,
	    0 ) ;

	window_fit(frame) ;

	pw = canvas_pixwin(canvas) ;
	if(pw == NULL)
	  exit(1) ;



	if( (pwh = Dev_pw_create(pw, 2, &i)) == NULL)
	{
	  fprintf(stderr, "can't get pwh, err=%d\n",i) ;
	  exit(1) ;
	}

	Dev_pw_setcmsname(pwh, "polygon") ;
	Dev_pw_putcolormap(pwh, 0, 16, red,green,blue) ;
	Dev_pw_set_write_mask(pwh, 0xf) ;

	Dev_pw_set_clip_planes(pwh, 0x3f) ;

	window_main_loop(frame) ;
}





static	void
draw_proc(item,event)
	Panel_item	item ;
	Event		*event ;
{
	Matrix3df	matrix ;

	Dev_pw_viewport_2d(pwh, .25,.75, .25,.75) ;

	if( item == draw_item )
	  Dev_pw_ident_mat(matrix) ;
	else
	  Dev_pw_window_mat(-.5,.5, -.5,.5, 0.,1., matrix) ;
	Dev_pw_set_matrix_3d(pwh, 0, matrix) ;

	draw_picture() ;
}



static
draw_picture()
{
	float		theta ;
	int		color ;
	Matrix3df	matrix ;
	int		count ;
	int		i ;
	Rect		*rect ;
static	Point3df	points[] = {
	{-.75,0.,0.}, {-.5,.75,0.}, {-.25,.5,0.}, {.25,.5,0.}, {.5,.75,0.},
	{.75,0.,0.}, {.5,-.75,0.}, {.25,-.5,0.}, {-.25,-.5,0.}, {-.5,-.75,0.},
	{-.5,.5,0.}, {-.25,.25,0.}, {.25,.25,0.}, {.5,.5,0.},
	{.5,-.5,0.}, {.25,-.25,0.}, {-.25,-.25,0.}, {-.5,-.5,0.} } ;

static	Point2di	ip[] = {{-5,0}, {5,0}, {0,7}, {-6,0},
				{0,-6}, {8,0}, {0,11},{-11,0}} ;
#define	NPTS	(sizeof(points)/sizeof(Point3df))
#define	NIPTS	(sizeof(ip)/sizeof(Point2di))

	rect = Dev_pw_rect(pwh) ;
	Dev_pw_clear(pwh, 0) ;
	Dev_pw_select_matrix(pwh, 0) ;

	Dev_pw_rotz(.0157080, matrix) ;
	Dev_pw_set_matrix_3d(pwh, 1, matrix) ;
	color = 0 ;
	count = 0 ;
	for(count = 0; count < 4000; ++count)
	{
	  Dev_pw_set_color(pwh, color) ;
#ifdef	COMMENT
	  Dev_pw_line_3df(pwh, NPTS, points) ;
#endif	COMMENT
	  pw_lock(pw, rect) ;
	  Dev_pw_move_3df(pwh, &points[0]) ;
	  for(i=1; i<NPTS; ++i)
	    Dev_pw_draw_3df(pwh, &points[i]) ;
	  pw_unlock(pw) ;
	  color = ((color-1) % 14) + 2 ;
	  Dev_pw_matrix_mul_3d(pwh, 0, 1,0) ;
	}
}



static	void
repaint_proc(canvas,pixwin,repaint_area)
	Canvas		canvas ;
	Pixwin		*pixwin ;
	Rectlist	*repaint_area ;
{
#ifdef	COMMENT
	Matrix3df	matrix ;

	Dev_pw_ident_mat(matrix) ;
	Dev_pw_set_matrix_3d(pwh, 0, matrix) ;
	draw_picture() ;
#endif	COMMENT
}







print_mat(matrix)
	Matrix3df	matrix ;
{
	int	i,j ;

	for(i=0; i<4; ++i)
	{
	  for(j=0; j<4; ++j)
	    printf("%8.5f ",matrix[i][j]) ;
	  putchar('\n') ;
	}
	putchar('\n') ;
}
