#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	MOUSE_SCALE	(.1*RAD)	/* adjustment scale */
#define	MAX_ERROR	(.5*RAD)	/* when we need to adjust error */


/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


typedef	struct {
	  int	freq ;
	  char	freq_c[8] ;
	  int	type ;		/* type: 0=regular, 1=adjustable, 2=RMI */
	  Point2df station ;	/* location of station */
	  Point2df vec ;	/* vector to station in ship's coords */
	  float	ptr_err ;	/* pointer error */
	  Point2df err_vec ;	/* vector of pointer error */
	} Adf_private ;

static	enum	{
	  AD_FREQ,
	  AD_ROSE,
	} adjusting ;

static	int	update_adf(), draw_adf(), input_adf(), fail_adf() ;

static	FILE	*adf_file ;


/*
 * this file draws the ADF
 *
 * failure states:
 *   0 - normal, tracking
 *   1 - (RMI only) for 150 frames, slowly increase dg error_rate
 *   2 - (RMI only) for 150 frames, slowly decrease dg error_rate
 *   3 - (RMI only) calcualte composite matrix, etc. (same as draw_dg)
 *   4 - "test" mode, error increases 18 degrees/sec
 *   5 - "test" released, error decreases/increases 18 degrees/sec
 *   6 - no reception
 *   7 - broken
 *
 */


#define	PSIZE(pts)	(sizeof(pts)/sizeof(Point2df))


#ifdef	COMMENT
static	Point2df	pointer[] = {
	  {0.,-.95}, {0.,.95}, {-.1,.85},{.1,.85},{0.,.95}} ;
#endif	COMMENT

static	Point2df	pointer[] = {
	  {-.02,-.9}, {-.02,.8}, {-.1,.8}, {0.,.9},
	  {.1,.8}, {.02,.8}, {.02,-.9}} ;
static	int		pointer_bnds[] = {PSIZE(pointer)} ;


extern	int	debug_level ;

static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;



Instrument	*
create_adf(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for Adf */
{
	Instrument	*ins ;
	Adf_private	*priv ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_adf ;
	ins->update = update_adf ;
	ins->fail = fail_adf ;
	ins->input = input_adf ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->vec.x = .5 ;
	ins->vec.y = 0.8660254 ;
	ins->flags = INS_ELECTRIC ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	priv = (Adf_private *) malloc( sizeof(Adf_private) ) ;
	priv->freq = 370 ;
	strcpy(priv->freq_c, "370") ;
	priv->type = 0 ;
	priv->station.x = 0.0 ;
	priv->station.y = 0.0 ;
	priv->ptr_err = 0.0 ;
	ins->private = (caddr_t) priv ;

	adf_file = fopen("adfs.dat", "r") ;

	return ins ;
}



static	int	ddy ;

static
input_adf(ins,event,dx,dy)
register Instrument	*ins ;
	Event		*event ;
	int		dx,dy ;
{
register Adf_private	*priv = (Adf_private *) ins->private ;
	int		frac_y ;

	switch( event_id(event) )
	{
	  case MS_LEFT:
	    instrument_grabio(ins, event_is_down(event) ) ;
	    adjusting = AD_FREQ ;
	    if( event_is_up(event))
	      set_frequency(ins) ;
	    break ;

	  case MS_RIGHT:
	    fail_adf(ins) ;
	    break ;

	  case LOC_MOVE:
	  case LOC_DRAG:
	    ddy += dy ; frac_y = ddy/4 ; ddy -= frac_y*4 ;
	    switch( adjusting )
	    {
	      case AD_FREQ:
		priv->freq -= frac_y ;
		sprintf(priv->freq_c, "%d", priv->freq) ;
	    }
	    break ;
	}
}



static	int
fail_adf(ins)
register Instrument	*ins ;
{
register Adf_private	*priv = (Adf_private *) ins->private ;

	ins->fstate = 1 ;
}





static
update_adf(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Adf_private	*priv = (Adf_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;
	int	ix,iy,rop ;


	/* first, handle error accumulation */

	switch( ins->fstate )
	{
	  case 0:
	    /* xform station to ships coords to get vector */
	    priv->vec.x = Me_imatrix[0][0]*priv->station.x +
			  Me_imatrix[1][0]*priv->station.y +
			  Me_imatrix[3][0] ;
	    priv->vec.y = Me_imatrix[0][2]*priv->station.x +
			  Me_imatrix[1][2]*priv->station.y +
			  Me_imatrix[3][2] ;

	    /* normalize */
	    {
	      register float a ;
	      a = priv->vec.x*priv->vec.x + priv->vec.y*priv->vec.y ;
	      if( a != 0.0 && a != 1.0 )
	      {
		a = 1.0/sqrt(a) ;
		priv->vec.x *= a ;
		priv->vec.y *= a ;
	      }
	    }
	    break ;
	}


	/* TM2 = pointer matrix */

	mat2d[0][0] = mat2d[1][1] = priv->vec.y ;
	mat2d[0][1] = -priv->vec.x ;
	mat2d[1][0] = priv->vec.x ;
	Gfx_set_matrix_2d(gfx, TM2, mat2d) ;

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;


	/* draw circle */

	Gfx_dbuf_color(gfx,BLACK) ;
	Gfx_polygon_2df(gfx, 1, circle_bnds, circle, GFX_SHADE_CONSTANT) ;

	Gfx_flush(Main_gfx,1) ;

	/* draw frequency */

	ix = (ins->ix0*3 + ins->ix1)/4 ;
	iy = (ins->iy0 + ins->iy1*3)/4 ;
	rop = Gfx_dbuf_get_color(gfx,WHITE) ;
	rop = PIX_SRC | PIX_COLOR(rop) ; /* Pell */
	pw_ttext(Main_pw, ix,iy, rop, Main_font, priv->freq_c) ; /* Pell */


	/* draw tic marks */

	Gfx_dbuf_color(gfx,WHITE) ;
	i = N_TICS1 ;
	ptr = tics1 ;
	while(i > 0)
	{
	  Gfx_vec_init_2df(gfx, i*2, fit) ;
	  fit /= 2 ;
	  i-= fit ;
	  while(--fit >= 0)
	  {
	    Gfx_vec_move_2df(gfx, ptr) ;
	    ++ptr ;
	    Gfx_vec_draw_2df(gfx, ptr) ;
	    ++ptr ;
	  }
	  Gfx_vec_term_2df(gfx) ;
	}

	Gfx_dbuf_color(gfx,LGREY) ;
	i = N_TICS2 ;
	ptr = tics2 ;
	while(i > 0)
	{
	  Gfx_vec_init_2df(gfx, i*2, fit) ;
	  fit /= 2 ;
	  i-= fit ;
	  while(--fit >= 0)
	  {
	    Gfx_vec_move_2df(gfx, ptr) ;
	    ++ptr ;
	    Gfx_vec_draw_2df(gfx, ptr) ;
	    ++ptr ;
	  }
	  Gfx_vec_term_2df(gfx) ;
	}

	/* draw pointer */

	Gfx_select_matrix(gfx, TM2) ;
	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_polygon_2df(gfx, 1, pointer_bnds, pointer, GFX_SHADE_CONSTANT) ;
}





static
draw_adf(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Adf_private	*priv = (Adf_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;



	update_adf(gfx,ins) ;

#ifdef	COMMENT
	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;
#endif	COMMENT

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	/* draw bolts   JPD 10.28.88 */

	Gfx_dbuf_color(gfx,LGREY) ;
        Gfx_polygon_2df(gfx, 1, uplcir_bnds, uplcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, uprcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowlcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowrcir, GFX_SHADE_CONSTANT) ;

	/* draw trim */

	Gfx_dbuf_color(gfx,LGREY) ;
	Gfx_line_2df(gfx, trim_size, trim) ;
}



static
set_frequency(ins)
register Instrument	*ins ;
{
register Adf_private	*priv = (Adf_private *) ins->private ;
	int		adf_freq ;
	int		lad,lam,las, lod,lom,los ;
	float		cvd() ;
	int		n ;

	if( ins->fstate == 7 )
	  return ;

	ins->fstate = 6 ;		/* no reception */
	if( adf_file != NULL )
	{
	  for(;;)
	  {
	    n = fscanf(adf_file, "%d %d.%d.%d %d.%d.%d %*s %*s",
		&adf_freq, &lad,&lam,&las, &lod,&lom,&los) ;
	    if( n != 7 )
	      break ;
	    if( adf_freq == priv->freq )
	    {
	      ins->fstate = 0 ;
	      priv->station.x = (cvd(lod,lom,los) - LONB)*(RAD*RADIUS*COSLAT) ;
	      priv->station.y = (cvd(lad,lam,las) - LATB) * (RAD*RADIUS) ;
	      break ;
	    }
	  }
	  rewind( adf_file ) ;
	}
}




float
cvd(d,m,s)
	int	d,m,s ;
{
	int	sign ;

	if( d < 0 )
	{
	  sign = -1 ;
	  d = -d ;
	}
	else
	  sign = 1 ;

	return sign * (d + (m/60.) + (s/3600.)) ;
}



adf_map(gfx, adf)
register Gfx_context	gfx ;
register Instrument	*adf ;
{
register Adf_private	*priv = (Adf_private *) adf->private ;
	Point3df	p1,p2 ;

	if( adf->fstate == 0 )
	{
	  p1.x = priv->station.x ;
	  p1.y = priv->station.y ;
	  p1.z = 0. ;

	  p2.x = Me_Posn.x ;
	  p2.y = Me_Posn.y ;
	  p2.z = 0. ;

	  Gfx_dbuf_color(gfx,RED) ;
	  Gfx_move_3df(gfx, &p1) ;
	  Gfx_draw_3df(gfx, &p2) ;
	}
}
