#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	SCALE	(1.943844)		/* convert m/s to knots */

#define	MAX_ERROR	(3/SCALE)	/* max error = 3 kts */
#define	MAX_KTS		(200)		/* top of scale */
#define DISP_KTS	(150)		/* as far as scale is labeled */

#define	AS2THETA(speed)	((speed) * (SCALE / MAX_KTS * M_PI * 2) )

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


static	int	update_as(), draw_as(), input_as(), fail_as() ;


/*
 * this file draws the airspeed indicator
 *
 * failure states:
 *   1 - pitot blocked, altitude increase == "speed" increase
 *   2 - static blocked, altitude increase == sensitivity decrease
 *   3 - frozen in position.
 */


static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df	disp = {
	{.5,0.},
	{0.,.5},
	{.5,.5}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Point2df zero = {0.,0.} ;

static	Point2df as_tics[DISP_KTS/5+1][2] ;



Instrument	*
create_airspeed(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for airspeed */
{
	Instrument	*ins ;
	float		theta ;
	int		i ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_as ;
	ins->update = update_as ;
	ins->fail = fail_as ;
	ins->input = input_as ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_STATIC | INS_PITOT ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	ins->dvalue = ins->value = Me_Speed ;
	theta = AS2THETA(Me_Speed) ;
	ins->vec.x = sin(theta) * .7 ;
	ins->vec.y = cos(theta) * .7 ;

	/* compute tic marks, if there's more than one airspeed indicator
	   (unlikely), this will be done more than once; no big deal */

	for(i=0; i<=DISP_KTS/5; ++i)
	{
	  theta = AS2THETA(i*(5/SCALE)) ;
	  as_tics[i][0].x = sin(theta) * .9 ;
	  as_tics[i][0].y = cos(theta) * .9 ;
	  if( i%2 == 0 )
	    as_tics[i][1].x = sin(theta) * .8,
	    as_tics[i][1].y = cos(theta) * .8 ;
	  else
	    as_tics[i][1].x = sin(theta) * .85,
	    as_tics[i][1].y = cos(theta) * .85 ;
	}

	return ins ;
}



static
input_as(ins,event)
register Instrument	*ins ;
register Event		*event ;
{
	switch( event_id(event) )
	{
	  case MS_RIGHT:
	    fail_as(ins) ;
	    break ;
	}
}



static	int
fail_as(ins)
register Instrument	*ins ;
{
	ins->fstate = 3 ;
}





static
update_as(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;
	float	theta ;


	/* first, handle error accumulation */

	switch( ins->fstate )
	{
	  case 0:
	    ins->value = Me_Speed ;
	    break ;

	  case 1:
	    break ;

	  case 2:
	    break ;

	  case 3:
	    break ;
	}

	if( ins->value - ins->dvalue > MAX_ERROR  ||
	    ins->value - ins->dvalue < -MAX_ERROR )
	{
	  register float theta ;
	  ins->dvalue = ins->value ;
	  theta = AS2THETA(ins->dvalue) ;
	  ins->vec.x = sin(theta) * .7 ;
	  ins->vec.y = cos(theta) * .7 ;
	}

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &ins->vec) ;
}





static
draw_as(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;

	update_as(gfx,ins) ;
	bolts_and_trim(gfx,1,DISP_KTS/5+1,as_tics) ;
}
