#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	MOUSE_SCALE	(.001)		/* adjustment scale */
#define	MAX_ERROR	5		/* when we need to adjust display */
#define	LAPSE		1000.0		/* feet/inch Hg */
#define	M_FT		3.28		/* convert meters to feet */

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


static	int	last_x ;		/* last x-value in input_alt() */

static	int	update_alt(), draw_alt(), input_alt(), fail_alt() ;

typedef	struct {
	  Point2df	littlehand ;
	  char		window[6] ;
	} Alt_private ;

static	Point2df zero = {0.,0.} ;

/*
 * this file draws the altimeter
 *
 * failure states:
 *   1 - static port blocked
 *
 * For now, assume that pressure is always 29.92
 * altimeter setting is stored in "error".
 */




extern	int	debug_level ;

static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;



static	Point2df alt_tics[10][2] ;


Instrument	*
create_alt(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for Alt */
{
	Instrument	*ins ;
	Alt_private	*priv ;
	int		i ;
	float		theta ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_alt ;
	ins->update = update_alt ;
	ins->fail = fail_alt ;
	ins->input = input_alt ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_STATIC ;
	ins->dvalue = -999.0 ;
	ins->fstate = 0 ;
	ins->error = 29.92 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	priv = (Alt_private *) malloc( sizeof(Alt_private) ) ;
	ins->private = (caddr_t) priv ;
	sprintf(priv->window, "%5.2f", ins->error) ;

	/* compute tic marks, if there's more than one altimeter
	   (unlikely), this will be done more than once; no big deal */

	for(i=0; i<10; ++i)
	{
	  theta = i*(2*PI/10.) ;
	  alt_tics[i][0].x = sin(theta) * .95 ;
	  alt_tics[i][0].y = cos(theta) * .95 ;
	  alt_tics[i][1].x = sin(theta) * .9 ;
	  alt_tics[i][1].y = cos(theta) * .9 ;
	}

	return ins ;
}



static
input_alt(ins,event)
register Instrument	*ins ;
register Event		*event ;
{
register Alt_private	*priv = (Alt_private *) ins->private ;

	switch( event_id(event) )
	{
	  case MS_LEFT:
	    instrument_grabio(ins, event_is_down(event) ) ;
	    last_x = event_x(event) ;
	    break ;

	  case MS_RIGHT:
	    fail_alt(ins) ;
	    break ;

	  case LOC_MOVE:
	  case LOC_DRAG:
	    ins->error += (event_x(event) - last_x) * MOUSE_SCALE ;
	    last_x = event_x(event) ;
	    sprintf(priv->window, "%5.2f", ins->error) ;
	    break ;
	}
}



static	int
fail_alt(ins)
register Instrument	*ins ;
{
	ins->fstate = 1 ;
}





static
update_alt(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Alt_private	*priv = (Alt_private *) ins->private ;
	int	i,j ;
	int	ix,iy ;
	int	rop ;


	/* first, handle error accumulation */

	switch( ins->fstate )
	{
	  case 0:
	    ins->value = Me_Posn.z * M_FT + (ins->error-29.92)*LAPSE ;
	    break ;

	  case 1:		/* static tube blocked, altimeter freezes */
	    break ;
	}

	if( ins->dvalue - ins->value > MAX_ERROR ||
	    ins->dvalue - ins->value < -MAX_ERROR)
	{
	  register int feet, hund, thous ;
	  register float theta ;
	  ins->dvalue = ins->value ;
	  feet = ins->value ;
	  hund = feet % 1000 ;
	  thous = feet % 10000 ;
	  theta = M_PI*2*.001 * hund ;
	  ins->vec.x = .7 * sin(theta) ;
	  ins->vec.y = .7 * cos(theta) ;
	  theta = M_PI*2*.0001 * thous ;
	  priv->littlehand.x = .5 * sin(theta) ;
	  priv->littlehand.y = .5 * cos(theta) ;
	}

	/* draw Kollsman window */

	ix = ins->ix1 - 50 ;
	iy = (ins->iy0 + ins->iy1)/2 ;
	rop = Gfx_dbuf_get_color(gfx,WHITE) ;
	rop = PIX_SRC | PIX_COLOR(rop) ; /* Pell */
	pw_ttext(Main_pw, ix,iy, rop, Main_font, priv->window) ; /* Pell */

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &ins->vec) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &priv->littlehand) ;
}





static
draw_alt(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Alt_private	*priv = (Alt_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	update_alt(gfx,ins) ;
	bolts_and_trim(gfx,1,10,alt_tics) ;
}
