#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	MOUSE_SCALE	.01		/* adjustment scale */
#define	MAX_ERROR	.01		/* when we need to adjust error */
#define	PITCH_CARD	1		/* map pitch to deviation of card */

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


typedef	struct {
	  float	last_error ;
	  float	ex,ey ;			/* current error vector */
	} Ai_private ;

static	int	last_y ;		/* last y-value in input_ai() */

static	int	update_ai(), draw_ai(), input_ai(), fail_ai() ;


/*
 * this file draws the attitide indicator
 *
 * failure states:
 *   1 - for 150 frames, slowly increase error
 *   2 - for another 150 frames, slowly decrease error
 *   3 - calculate composite matrix of aircraft position and AI
 *	error and freeze AI at that point.
 */


extern	int	debug_level ;

static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df matP = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df matR = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;


/* thing that goes up/down */

static	Point2df pitch_card1[] = {{-.8,0.}, {.8,0.}, {.8,1.6}, {-.8,1.6}} ;
static	Point2df pitch_card2[] = {{-.8,0.}, {.8,0.}, {.8,-1.6}, {-.8,-1.6}} ;
static	int	pitch_card_bnds[] = {PSIZE(pitch_card1)} ;
static	Point2df lines[] = {
	  {-.25,.20}, {.25,.20},
	  {-.25,.10}, {.25,.10},
	  {-.25,-.10}, {.25,-.10},
	  {-.25,-.20}, {.25,-.20},
	  {0.,0.}, {-2.,-1.},
	  {0.,0.}, {-1.,-1.},
	  {0.,0.}, { 1.,-1.},
	  {0.,0.}, { 2.,-1.}} ;


/* thing that goes side to side */

static	Point2df frame[] = {
	  { -1.01,-1.01}, { -1.01, 1.01}, {  1.01, 1.01}, {  1.01,-1.01},

	  { 0.000, 0.800}, { 0.139, 0.788}, { 0.274, 0.752},
	  { 0.400, 0.693}, { 0.514, 0.613}, { 0.613, 0.514},
	  { 0.693, 0.400}, { 0.752, 0.274}, { 0.788, 0.139},
	  { 0.800, 0.000}, { 0.788,-0.139}, { 0.752,-0.274},
	  { 0.693,-0.400}, { 0.613,-0.514}, { 0.514,-0.613},
	  {-0.514,-0.613}, {-0.613,-0.514}, {-0.693,-0.400},
	  {-0.752,-0.274}, {-0.788,-0.139}, {-0.800, 0.000},
	  {-0.788, 0.139}, {-0.752, 0.274}, {-0.693, 0.400},
	  {-0.613, 0.514}, {-0.514, 0.613}, {-0.400, 0.693},
	  {-0.274, 0.752}, {-0.139, 0.788}} ;
static	int	frame_bnds[] = {4,29} ;

static	Point2df tics[] = {
	  {-.90, 0.}, {-.80, 0.},
	  {-.78,.45}, {-.69,.40},
	  {-.45,.78}, {-.40,.69},
	  {-.29,.80}, {-.27,.75},
	  {-.15,.84}, {-.14,.79},
	  {  0.,.90}, {  0.,.80},
	  { .15,.84}, { .14,.79},
	  { .29,.80}, { .27,.75},
	  { .45,.78}, { .40,.69},
	  { .78,.45}, { .69,.40},
	  { .90, 0.}, { .80, 0.}} ;

static	Point2df index[] = {{0.,.9}, {-.1,1.}, {.1,1.}} ;
static	int	 index_bnds[] = { PSIZE(index) } ;

#ifdef	COMMENT
static	Point2df mask1[] = {{-.5,.6}, {.5,.6}, {.5,1.4}, {-.5,1.4}} ;
static	Point2df mask2[] = {{-.5,-.6}, {.5,-.6}, {.5,-1.4}, {-.5,-1.4}} ;
static	int	mask_bnds[] = { PSIZE(mask1) } ;
#endif	COMMENT



/* little picture of an airplane
 *
 *  ___   ___
 *     \ /
 *	|
 *	|
 */

static	Point2df plane1[] = {{-.35,0.}, {-.1,0.}, {0.,-.1}, {0.,-1.6}} ;
static	Point2df plane2[] = {{.35,0.}, {.1,0.}, {0.,-.1}} ;
static	Point2df plane3[] = {{-.02,0.},{0.,-.02},{.02,0.},{0.,.02},{-.02,0.}};



static	Point2df	ccir[] = {
	  { 0.100,-0.800}, { 0.087,-0.750}, { 0.050,-0.713},
	  { 0.000,-0.700}, {-0.050,-0.713}, {-0.087,-0.750},
	  {-0.100,-0.800}, {-0.087,-0.850}, {-0.050,-0.887},
	  { 0.000,-0.900}, { 0.050,-0.887}, { 0.087,-0.850}} ;
static	int		ccir_bnds[] = {PSIZE(ccir)} ;



Instrument	*
create_ai(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for Ai */
{
	Instrument	*ins ;
	Ai_private	*priv ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_ai ;
	ins->update = update_ai ;
	ins->fail = fail_ai ;
	ins->input = input_ai ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_VACUUM ;
	ins->dvalue = ins->value = 0.0 ;	/* roll vector */
	ins->vec.x = 1.0 ;  ins->vec.y = 0.0 ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
	ins->error_rate = (rand()%10) * .1 * RAD ;
	ins->next = NULL ;

	priv = (Ai_private *) malloc( sizeof(Ai_private) ) ;
	priv->last_error = 0.0 ;
	priv->ex = 1.0; priv->ey = 0.0 ;
	ins->private = (caddr_t) priv ;

	return ins ;
}



static
input_ai(ins,event)
register Instrument	*ins ;
register Event		*event ;
{
register Ai_private	*priv = (Ai_private *) ins->private ;

	switch( event_id(event) )
	{
	  case MS_LEFT:
	    instrument_grabio(ins, event_is_down(event) ) ;
	    last_y = event_y(event) ;
	    break ;

	  case MS_RIGHT:
	    fail_ai(ins) ;
	    break ;

	  case LOC_MOVE:
	  case LOC_DRAG:
	    ins->error -= (event_y(event) - last_y) * MOUSE_SCALE ;
	    last_y = event_y(event) ;
	    break ;
	}
}



static	int
fail_ai(ins)
register Instrument	*ins ;
{
register Ai_private	*priv = (Ai_private *) ins->private ;

	ins->fcount = 150 ;
	ins->fstate = 1 ;
}





static
update_ai(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Ai_private	*priv = (Ai_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	/* first, handle error accumulation */

	ins->value = Roll_sin ;

	if( ins->value - ins->dvalue > MAX_ERROR ||
	    ins->value - ins->dvalue < -MAX_ERROR)
	{
	  ins->dvalue = -Roll_sin ;
	  ins->vec.y = Roll_sin ;
	  ins->vec.x = Roll_cos ;
	}

	/* DRAW:
	 *
	 * Calculate rotation matrix equal to roll of plane, [R]
	 * Calculate translation matrix proportional to pitch of plane [P]
	 * xform = [P][R]
	 * draw thing that goes up/down
	 * xform = [R]
	 * draw thing that rolls side/side
	 * xform = ident
	 * draw frame and little wings
	 */

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0x3c) ;

	/* matrices:  TM1 = composite, TM2 = roll */

	matP[2][1] = -PITCH_CARD * Me_Forward.z ;
	Gfx_set_matrix_2d(gfx, TM1, matP) ;
	Gfx_select_matrix(gfx, TM1) ;

	matR[0][0] = matR[1][1] = ins->vec.x ;
	matR[0][1] = ins->vec.y ;
	matR[1][0] = -ins->vec.y ;
	Gfx_set_matrix_2d(gfx, TM2, matR) ;

	Gfx_matrix_mul_2d(gfx, TM1, TM1, TM2) ;

	/* draw pitch card */

	Gfx_dbuf_color(gfx,BLUE) ;
	Gfx_polygon_2df(gfx, 1, pitch_card_bnds, pitch_card1,
				GFX_SHADE_CONSTANT) ;
	Gfx_dbuf_color(gfx,LGREY) ;
	Gfx_polygon_2df(gfx, 1, pitch_card_bnds, pitch_card2,
				GFX_SHADE_CONSTANT) ;

	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_vec_init_2df(gfx, 16, fit) ;
	Gfx_vec_move_2df(gfx, &lines[0]) ; Gfx_vec_draw_2df(gfx, &lines[1]) ;
	Gfx_vec_move_2df(gfx, &lines[2]) ; Gfx_vec_draw_2df(gfx, &lines[3]) ;
	Gfx_vec_move_2df(gfx, &lines[4]) ; Gfx_vec_draw_2df(gfx, &lines[5]) ;
	Gfx_vec_move_2df(gfx, &lines[6]) ; Gfx_vec_draw_2df(gfx, &lines[7]) ;
	Gfx_vec_move_2df(gfx, &lines[8]) ; Gfx_vec_draw_2df(gfx, &lines[9]) ;
	Gfx_vec_move_2df(gfx, &lines[10]) ; Gfx_vec_draw_2df(gfx, &lines[11]) ;
	Gfx_vec_move_2df(gfx, &lines[12]) ; Gfx_vec_draw_2df(gfx, &lines[13]) ;
	Gfx_vec_move_2df(gfx, &lines[14]) ; Gfx_vec_draw_2df(gfx, &lines[15]) ;
	Gfx_vec_term_2df(gfx) ;


	/* draw little airplane shape , matrices:  TM1 = offset, TM2 = roll*/

	matP[2][1] = ins->error ;
	Gfx_set_matrix_2d(gfx, TM1, matP) ;
	Gfx_select_matrix(gfx, TM1) ;
	Gfx_dbuf_color(gfx, RED) ;
	Gfx_line_2df(gfx, PSIZE(plane1), plane1) ;
	Gfx_line_2df(gfx, PSIZE(plane2), plane2) ;
	Gfx_line_2df(gfx, PSIZE(plane3), plane3) ;

	/* matrices:  TM1 = ident, TM2 = roll */

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;


	/* draw trim, cutting off top & bottom of pitch card */

	Gfx_dbuf_color(gfx, BLACK) ;
	Gfx_polygon_2df(gfx, 2, frame_bnds, frame, GFX_SHADE_CONSTANT) ;
	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_vec_init_2df(gfx, 22, fit) ;
	Gfx_vec_move_2df(gfx, &tics[0]) ; Gfx_vec_draw_2df(gfx, &tics[1]) ;
	Gfx_vec_move_2df(gfx, &tics[2]) ; Gfx_vec_draw_2df(gfx, &tics[3]) ;
	Gfx_vec_move_2df(gfx, &tics[4]) ; Gfx_vec_draw_2df(gfx, &tics[5]) ;
	Gfx_vec_move_2df(gfx, &tics[6]) ; Gfx_vec_draw_2df(gfx, &tics[7]) ;
	Gfx_vec_move_2df(gfx, &tics[8]) ; Gfx_vec_draw_2df(gfx, &tics[9]) ;
	Gfx_vec_move_2df(gfx, &tics[10]) ; Gfx_vec_draw_2df(gfx, &tics[11]) ;
	Gfx_vec_move_2df(gfx, &tics[12]) ; Gfx_vec_draw_2df(gfx, &tics[13]) ;
	Gfx_vec_move_2df(gfx, &tics[14]) ; Gfx_vec_draw_2df(gfx, &tics[15]) ;
	Gfx_vec_move_2df(gfx, &tics[16]) ; Gfx_vec_draw_2df(gfx, &tics[17]) ;
	Gfx_vec_move_2df(gfx, &tics[18]) ; Gfx_vec_draw_2df(gfx, &tics[19]) ;
	Gfx_vec_move_2df(gfx, &tics[20]) ; Gfx_vec_draw_2df(gfx, &tics[21]) ;
	Gfx_vec_term_2df(gfx) ;
	Gfx_dbuf_color(gfx,LGREY) ;
	Gfx_polygon_2df(gfx, 1, ccir_bnds, ccir, GFX_SHADE_CONSTANT) ;


	/* draw index marker */

	Gfx_select_matrix(gfx, TM2) ;
	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_polygon_2df(gfx, 1, index_bnds, index, GFX_SHADE_CONSTANT) ;


}





static
draw_ai(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Ai_private	*priv = (Ai_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	update_ai(gfx,ins) ;

#ifdef	COMMENT
	/* draw bolts   JPD 10.28.88 */

	Gfx_set_matrix_2d(gfx, TM2, ident2d) ;
	Gfx_dbuf_color(gfx,LGREY) ;
        Gfx_polygon_2df(gfx, 1, uplcir_bnds, uplcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, uprcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowlcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowrcir, GFX_SHADE_CONSTANT) ;

	/* draw trim */

	Gfx_dbuf_color(gfx,LGREY) ;
	Gfx_line_2df(gfx, trim_size, trim) ;
#endif	COMMENT
}
