#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	MOUSE_SCALE	2		/* pixels/minute of adjustment */
#define	MINUTES		60		/* convert minutes to seconds */
#define	MAX_MINS	(1*MINUTES)
#define	MAX_SUN		(30*MINUTES)

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


static	int	update_clock(), draw_clock(), input_clock(), fail_clock() ;

typedef	struct {
	  long		time ;		/* current time, seconds */
	  long		last_min ;	/* last time minute hand moved */
	  long		last_sec ;	/* last time second hand moved */
	  long		last_sun ;	/* last time sun moved */
	  Point2df	minutehand ;	/* vectors for drawing hands */
	  Point2df	secondhand ;
	} Clock_private ;

static	Point2df zero = {0.,0.} ;

/*
 * this file draws the altimeter
 *
 * failure states:
 *   1 - stopped
 */




extern	int	debug_level ;

static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Point2df clock_tics[12][2] ;



Instrument	*
create_clock(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored */
{
	Instrument	*ins ;
	Clock_private	*priv ;
	struct tm	*tm ;
	int		i ;
	float		theta ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_clock ;
	ins->update = update_clock ;
	ins->fail = fail_clock ;
	ins->input = input_clock ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_ELECTRIC ;
	ins->value = 0.0 ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	priv = (Clock_private *) malloc( sizeof(Clock_private) ) ;
	ins->private = (caddr_t) priv ;

	priv->time = Now.tv_sec ;
	tm = localtime(&Now.tv_sec) ;
	priv->last_min = 0 ;
	priv->last_sec = 0 ;
	priv->last_sun = Now.tv_sec ;

	/* compute tic marks, if there's more than one clock (unlikely),
	   this will be done more than once; no big deal */

	for(i=0; i<12; ++i)
	{
	  theta = i*(2*PI/12) ;
	  clock_tics[i][0].x = sin(theta) * .95 ;
	  clock_tics[i][0].y = cos(theta) * .95 ;
	  clock_tics[i][1].x = sin(theta) * .9 ;
	  clock_tics[i][1].y = cos(theta) * .9 ;
	}

	return ins ;
}



static
input_clock(ins,event,dx,dy)
register Instrument	*ins ;
register Event		*event ;
	int		dx,dy ;
{
register Clock_private	*priv = (Clock_private *) ins->private ;

	switch( event_id(event) )
	{
	  case MS_LEFT:
	    instrument_grabio(ins, event_is_down(event) ) ;
	    if( event_is_up(event)  &&  Day_state >= DARK )
	      Shade_stars(priv->time) ;
	    break ;

	  case MS_RIGHT:
	    fail_clock(ins) ;
	    break ;

	  case LOC_MOVE:
	  case LOC_DRAG:
	    if( dx > MOUSE_SCALE  ||
		dx < -MOUSE_SCALE)
	    {
	      priv->time += 60*(dx/MOUSE_SCALE) ;
	    }
	    break ;
	}
}



static	int
fail_clock(ins)
register Instrument	*ins ;
{
	ins->fstate = 1 ;
}





static
update_clock(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Clock_private	*priv = (Clock_private *) ins->private ;
	int	i,j ;
	int	ix,iy ;
	int	rop ;


	/* first, handle error accumulation */

	switch( ins->fstate )
	{
	  case 0:
	    ins->value += Dtime ;
	    if( ins->value > 1.0 )
	    {
	      ins->value -= 1.0 ;
	      priv->time += 1 ;
	    }
	    break ;

	  case 1:
	    break ;
	}

	if( priv->time != priv->last_sec )
	{
	  register float theta ;
	  register struct tm *tm ;

	  priv->last_sec = priv->time ;
	  tm = localtime(&priv->time) ;
	  theta = M_PI * 2.0 * tm->tm_sec * (1./60.) ;
	  priv->secondhand.x = sin(theta) * .9 ;
	  priv->secondhand.y = cos(theta) * .9 ;

	  if( priv->time - priv->last_min > (1*60)  ||
	      priv->time - priv->last_min < -(1*60))
	  {
	    priv->last_min = priv->time ;
	    theta = M_PI*2.0*(tm->tm_min + tm->tm_sec*(1./60.))* (1./60.) ;
	    priv->minutehand.x = sin(theta) * .8 ;
	    priv->minutehand.y = cos(theta) * .8 ;
	    theta = M_PI*2.0*(tm->tm_hour + tm->tm_min*(1./60.))* (1./12.) ;
	    ins->vec.x = sin(theta) * .5 ;
	    ins->vec.y = cos(theta) * .5 ;
	    if( priv->time - priv->last_sun > MAX_SUN  ||
		priv->time - priv->last_sun < -MAX_SUN)
	    {
	      priv->last_sun = priv->time ;
	      Color_compute(priv->time) ;
	      GC_Terrain() ;
	      Reshade_Terrain() ;
#ifdef	COMMENT
	      if( Day_state >= DARK)
		Shade_stars(priv->time) ;
#endif	COMMENT
	    }
	  }
	}

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &ins->vec) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &priv->minutehand) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &priv->secondhand) ;
}





static
draw_clock(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Clock_private	*priv = (Clock_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	update_clock(gfx,ins) ;

	bolts_and_trim(gfx,1,12,clock_tics) ;
}
