#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	MOUSE_SCALE	(.1*RAD)	/* adjustment scale */
#define	MAX_ERROR	(.5*RAD)	/* when we need to adjust error */

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "parameters.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


typedef	struct {
	  float	last_error ;
	  float	ex,ey ;			/* current error vector */
	} Dg_private ;

static	int	update_dg(), draw_dg(), input_dg(), fail_dg() ;


/*
 * this file draws the gyro-compass
 *
 * failure states:
 *   1 - for 150 frames, slowly increase error_rate
 *   2 - for another 150 frames, slowly decrease error_rate
 *   3 - calculate composite matrix of aircraft position and compass
 *	error and freeze compass at that point.
 */


#define	PSIZE(pts)	(sizeof(pts)/sizeof(Point2df))




static  Point2df	plane[] = {
	{ 0.000, 0.300}, { 0.000, 0.250}, { 0.045, 0.200}, { 0.065, 0.110},
	{ 0.250, 0.000}, { 0.250,-0.065}, { 0.065,-0.035}, { 0.050,-0.135},
	{ 0.035,-0.185}, { 0.135,-0.235}, { 0.135,-0.285}, { 0.000,-0.250},
	{-0.135,-0.285}, {-0.135,-0.235}, {-0.035,-0.185}, {-0.050,-0.135},
	{-0.065,-0.035}, {-0.250,-0.065}, {-0.250, 0.000}, {-0.065, 0.110},
	{-0.045, 0.200}, {-0.000, 0.250}} ;

static	Point2df	marks1[] = {{-.6,0.},{-.4,0.}} ;
static	Point2df	marks2[] = {{0.,.6},{0.,.4}} ;
static	Point2df	marks3[] = {{.6,0.},{.4,0.}} ;
static	Point2df	marks4[] = {{-.38,.38},{-.35,.35}} ;
static	Point2df	marks5[] = {{.38,.38},{.35,.35}} ;
static	Point2df	plane1[] = {{-.15,0.},{.15,0.}} ;
static	Point2df	plane2[] = {{0.,0.},{0.,.05}} ;


extern	int	debug_level ;

static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;




Instrument	*
create_dg(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for DG */
{
	Instrument	*ins ;
	Dg_private	*priv ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_dg ;
	ins->update = update_dg ;
	ins->fail = fail_dg ;
	ins->input = input_dg ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_VACUUM ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
#ifdef	COMMENT
	ins->error_rate = (random()%10) * .01 * RAD ;
#endif	COMMENT
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	priv = (Dg_private *) malloc( sizeof(Dg_private) ) ;
	priv->last_error = 0.0 ;
	priv->ex = 1.0; priv->ey = 0.0 ;
	ins->private = (caddr_t) priv ;

	return ins ;
}



static
input_dg(ins,event,dx,dy)
register Instrument	*ins ;
register Event		*event ;
	int		dx,dy ;
{
register Dg_private	*priv = (Dg_private *) ins->private ;

	switch( event_id(event) )
	{
	  case MS_LEFT:
	    instrument_grabio(ins, event_is_down(event) ) ;
	    break ;

	  case MS_RIGHT:
	    fail_dg(ins) ;
	    break ;

	  case LOC_MOVE:
	  case LOC_DRAG:
	    ins->error -= dx * MOUSE_SCALE ;
	    break ;
	}
}



static	int
fail_dg(ins)
register Instrument	*ins ;
{
register Dg_private	*priv = (Dg_private *) ins->private ;

	ins->fcount = 150 ;
	ins->fstate = 1 ;
}





static
update_dg(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Dg_private	*priv = (Dg_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	/* first, handle error accumulation */

	if( ins->fstate )
	{
	  switch( ins->fstate )
	  {
	    case 1:
	      ins->error_rate += .4 * RAD * Dtime ;
	      if( --ins->fcount < 0 )
	      {
		++ins->fstate ;
		ins->fcount = 150 ;
	      }
	      break ;

	    case 2:
	      ins->error_rate -= .4 * RAD * Dtime ;
	      if( --ins->fcount < 0 )
	      {
		register float x=priv->ex, y=priv->ey ;
		priv->ex = Head_cos * x - Head_sin * y ;
		priv->ey = Head_cos * y + Head_sin * x ;
		++ins->fstate ;
	      }
	      break ;

	    case 3:
	      break ;
	  }
	}

	if( ins->fstate < 3 )
	{
	  ins->error += ins->error_rate * Dtime ;
	  if( ins->error - priv->last_error > MAX_ERROR ||
	      ins->error - priv->last_error < -MAX_ERROR)
	  {
	    priv->ex = cos(ins->error) ;
	    priv->ey = sin(ins->error) ;
	    priv->last_error = ins->error ;
	  }
	}

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	/* draw circle */

	Gfx_dbuf_color(gfx,BLUE) ;
	Gfx_polygon_2df(gfx, 1, circle_bnds, circle, GFX_SHADE_CONSTANT) ;


	/* add in error */

	mat2d[0][0] = mat2d[1][1] = priv->ex ;
	mat2d[0][1] = priv->ey ;
	mat2d[1][0] = -priv->ey ;
	Gfx_set_matrix_2d(gfx, TM1, mat2d) ;

	/* build inverse transform of plane's position */

	if( ins->fstate < 3 )
	{
	  mat2d[0][0] = mat2d[1][1] = Head_cos ;
	  mat2d[0][1] = Head_sin ;
	  mat2d[1][0] = -Head_sin ;

	  Gfx_set_matrix_2d(gfx, TM2, mat2d) ;
	  Gfx_matrix_mul_2d(gfx, TM1, TM2, TM1) ;
	}


	/* draw tic marks */

	Gfx_dbuf_color(gfx,WHITE) ;
	i = N_TICS1 ;
	ptr = tics1 ;
	while(i > 0)
	{
	  Gfx_vec_init_2df(gfx, i*2, fit) ;
	  fit /= 2 ;
	  i-= fit ;
	  while(--fit >= 0)
	  {
	    Gfx_vec_move_2df(gfx, ptr) ;
	    ++ptr ;
	    Gfx_vec_draw_2df(gfx, ptr) ;
	    ++ptr ;
	  }
	  Gfx_vec_term_2df(gfx) ;
	}

	Gfx_dbuf_color(gfx,LGREY) ;
	i = N_TICS2 ;
	ptr = tics2 ;
	while(i > 0)
	{
	  Gfx_vec_init_2df(gfx, i*2, fit) ;
	  fit /= 2 ;
	  i-= fit ;
	  while(--fit >= 0)
	  {
	    Gfx_vec_move_2df(gfx, ptr) ;
	    ++ptr ;
	    Gfx_vec_draw_2df(gfx, ptr) ;
	    ++ptr ;
	  }
	  Gfx_vec_term_2df(gfx) ;
	}


	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_line_2df(gfx, N_let_size, N_let) ;
	Gfx_line_2df(gfx, E1_let_size, E1_let) ;
	Gfx_line_2df(gfx, E2_let_size, E2_let) ;
	Gfx_line_2df(gfx, S_let_size, S_let) ;
	Gfx_line_2df(gfx, W_let_size, W_let) ;


	/* draw plane for orientation   JPD 10.28.88  */

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_line_2df(gfx, PSIZE(plane), plane) ;


	/* draw marks */

	Gfx_line_2df(gfx, PSIZE(marks1), marks1) ;
	Gfx_line_2df(gfx, PSIZE(marks2), marks2) ;
	Gfx_line_2df(gfx, PSIZE(marks3), marks3) ;
	Gfx_line_2df(gfx, PSIZE(marks4), marks4) ;
	Gfx_line_2df(gfx, PSIZE(marks5), marks5) ;
}





static
draw_dg(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Dg_private	*priv = (Dg_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	update_dg(gfx,ins) ;

	/* draw bolts   JPD 10.28.88 */

	Gfx_dbuf_color(gfx,LGREY) ;
        Gfx_polygon_2df(gfx, 1, uplcir_bnds, uplcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, uprcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowlcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowrcir, GFX_SHADE_CONSTANT) ;

	/* draw trim */

	Gfx_dbuf_color(gfx,LGREY) ;
	Gfx_line_2df(gfx, trim_size, trim) ;
}
