#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

/*
 * this file draws the image seen on the screen.  It is the top-level
 * graphics routine
 */


#define	STAT_LEFT	300	/* x-posn of status lines */

#ifdef	COMMENT
#define	CULL_DEBUG		/* display aerial view */
#endif	COMMENT


#include <stdio.h>
#include <sunwindow/window_hs.h>
#include <sunwindow/pw_dblbuf.h>
#include <math.h>
#include "gfx.h"
#include "flight.h"
#include "object_types.h"
#include "graphics.h"

extern	Rect	*Gfx_rect() ;

extern	int	debug_level ;

extern	struct pixrect	flight_pr ;

static	Matrix3df	debug_mat, composite ;



draw_image()
{
	int	i,j,k,iret ;
	Matrix3df	matrix, R_matrix ;
	Point3df	p, q ;
	Point3df	p1,p2,p3,p4,p5 ;
	float	dx ;
	Rect	*rect ;
static	Rect	onepix = {0,0,1,1} ;
static	int	count ;
	char	line[80] ;
	int	color ;
	int	mask = 2 ;
	int	blind ;		/* flag, true if can't see out of window */
	int	ontop ;		/* true if on top of highest cloud layer */
	int	onbot ;		/* true if below lowest cloud layer */

	rect = Gfx_rect(Main_gfx) ;

	/* set db control and clear back buffer */


	Gfx_wait(Main_gfx) ;

	if( hwdb )
	{
	  pw_dbl_flip( Main_pw ) ;
	  pw_dbl_set( Main_pw, PW_DBL_WRITE, PW_DBL_BACK, 0 ) ;
	}
	else
	{
	  Gfx_dbuf_swap(Main_gfx) ;
	  Gfx_dbuf_control(Main_gfx, mask) ;
	}


	ontop = !((cloud_1 && Me_Posn.z < cloud_tops_1 ) ||
		  (cloud_2 && Me_Posn.z < cloud_tops_2 )) ;

	onbot = !((cloud_1 && Me_Posn.z > cloud_base_1 ) ||
		  (cloud_2 && Me_Posn.z > cloud_base_2 )) ;

	blind = hood ||
	      (cloud_1 && Me_Posn.z>cloud_base_1 && Me_Posn.z<cloud_tops_1) ||
	      (cloud_2 && Me_Posn.z>cloud_base_2 && Me_Posn.z<cloud_tops_2) ;

	/* paint the sky in the upper part of the screen */

	if( blind  ||  !ontop )
	  Gfx_clear_vp(Main_gfx, Color_Mapping[CLOUD][0],
		0.0, 1.0, 1.0-VIEWSIZE, 1.0) ;
	else
	  Gfx_clear_vp(Main_gfx, Color_Mapping[SKY][0],
		0.0, 1.0, 1.0-VIEWSIZE, 1.0) ;

	/* paint the control panel grey */

	Gfx_clear_vp(Main_gfx, GREY, 0.0, 1.0, 0., 1.0-VIEWSIZE) ;

	pw_lock(Main_pw, &onepix ) ;
	  
	Gfx_select_matrix(Main_gfx, VIEW_MAT) ;
	Gfx_viewport_2d(Main_gfx, 0.0, 1.0, 1.0-VIEWSIZE, 1.0) ;


	/* draw stars if night */

	if( Day_state >= DARK  &&  !blind  &&  ontop )
	{
	  Gfx_set_clip_planes(Main_gfx,0x3f) ;
	  Draw_Stars() ;
	}

	if( cloud_1  &&
	    Me_Posn.z > cloud_base_1-CLOUD_TRANS  &&
	    Me_Posn.z < cloud_base_1 )
	  Cloud_shade() ;


	if( !blind  &&  onbot )
	{
	  if( debug_level )
	  {
	    debug_mat[0][0] = 1.0; debug_mat[0][1] = 0.0;
	    debug_mat[0][2] = 0.0; debug_mat[0][3] = 0.0;

	    debug_mat[1][0] = 0.0; debug_mat[1][1] = .45;
	    debug_mat[1][2] = 0.9; debug_mat[1][3] = 0.0;

	    debug_mat[2][0] = 0.0; debug_mat[2][1] = 0.9;
	    debug_mat[2][2] = -.45; debug_mat[2][3] = 0.0;

	    debug_mat[3][0] = 0.0 ;
	    debug_mat[3][1] = 10000.0 * .45 -5000.0 * .9 ;
	    debug_mat[3][2] = 10000.0 * .9 + 5000.0 * .45 ;
	  }



	  /* draw scenery */

	  Gfx_set_clip_planes(Main_gfx,0x3f) ;

	  Gfx_mul_matrix( composite, Me_imatrix, P_matrix) ;
	  Gfx_set_matrix_3d(Main_gfx, VIEW_MAT, composite) ;

	  Draw_Terrain(&Me_Posn, composite) ;
	  notify_dispatch();	/* Pell */
	  Draw_Runways(&Me_Posn, composite) ;
	  notify_dispatch();	/* Pell */

	  if( debug_level )
	  {
	    Set_color(Main_gfx,RED) ;
	    Gfx_move_3df(Main_gfx, POINT3(p1,-40.,0.,0.)) ;
	    Gfx_move_3df(Main_gfx, POINT3(p1, 40.,0.,0.)) ;
	    Gfx_move_3df(Main_gfx, POINT3(p1, 0.,40.,0.)) ;
	    Gfx_move_3df(Main_gfx, POINT3(p1, 0.,-80.,0.)) ;
	  }
	}








	/* set 1:1 view transformation */

	Gfx_window_mat(0.0, 1.0, 0.0, 1.0, 0.0, 1.0, matrix) ;
	Gfx_set_matrix_3d(Main_gfx, VIEW_MAT, matrix) ;


	/* mark screen center */

	Set_color(Main_gfx,BLUE) ;
	Gfx_move_3df(Main_gfx,  POINT3(p1, 0.45, 0.45, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.55, 0.45, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.55, 0.55, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.45, 0.55, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.45, 0.45, 0.0)) ;
	Gfx_move_3df(Main_gfx,  POINT3(p1, 0.45, 0.5, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.47, 0.5, 0.0)) ;
	Gfx_move_3df(Main_gfx,  POINT3(p1, 0.53, 0.5, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.55, 0.5, 0.0)) ;
	Gfx_move_3df(Main_gfx,  POINT3(p1, 0.5, 0.45, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.5, 0.47, 0.0)) ;
	Gfx_move_3df(Main_gfx,  POINT3(p1, 0.5, 0.53, 0.0)) ;
	Gfx_draw_3df(Main_gfx,  POINT3(p1, 0.5, 0.55, 0.0)) ;

	if( map )
	  draw_map(Main_gfx,.5,.625,1.,1.) ;


#ifdef	COMMENT
	/* draw aerial view */

	Gfx_viewport_2d(Main_gfx, 0.5,1., 0.,1.) ;
	Gfx_window_mat(-5000.,5000., -5000.,5000., -1000.,1000., matrix) ;
	Gfx_set_matrix_3d(Main_gfx, WIN_MAT, matrix) ;
	Gfx_select_matrix(Main_gfx, WIN_MAT) ;
	(*world_view.func) (&world_view, &center, P_matrix ) ;

	Set_color(Main_gfx,WHITE) ;
	Gfx_move_3df(Main_gfx, &Me_Posn) ;
	p2 = Me_Forward ;
	p2.x *= 200. ;
	p2.y *= 200. ;
	p2.z *= 200. ;
	p2.x += Me_Posn.x ;
	p2.y += Me_Posn.y ;
	p2.z += Me_Posn.z ;
	Gfx_draw_3df(Main_gfx, &p2) ;

#endif	COMMENT

	draw_instruments() ;
	notify_dispatch();	/* Pell */

	Gfx_flush(Main_gfx,1) ;

#ifdef	COMMENT
	{
	  char	line[40] ;
	  sprintf(line, "power: %8.1f",Me_Power*100.0) ;
	  pw_text(Main_pw, 10,30, PIX_SRC, Main_font, line) ;
	  sprintf(line, "speed: %8.1f",Me_Speed*1.94) ;
	  pw_text(Main_pw, 10,50, PIX_SRC, Main_font, line) ;
	  sprintf(line, "vsi:   %8.1f",Me_Speed*Me_Forward.z*196) ;
	  pw_text(Main_pw, 10,70, PIX_SRC, Main_font, line) ;
	}
#endif	COMMENT

	pw_unlock(Main_pw) ;
}
