#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

/*
 * this file goes through the object list and draws all of the objects
 * on the screen
 */



#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "object_types.h"
#include "graphics.h"

extern	int	debug_level ;



	int	current_color ;

#define	set_color(color) if(color != current_color)	\
				Set_color(Main_gfx, current_color = color) ;



#ifdef	DEBUG
#define	register
#endif	DEBUG


void
polygon_trim(gfx, nloop,loops,vertices)
register Gfx_context	gfx ;
	int		nloop ;
register int		*loops ;
register Point3df	*vertices ;
{
register Point3df	*p0 ;
register int	nvert ;

	while(nloop--)
	{
	  nvert = *loops++ ;
	  p0 = vertices ;
	  Gfx_move_3df(gfx, vertices) ;
	  ++vertices ;
	  while(--nvert)
	  {
	    Gfx_draw_3df(gfx, vertices) ;
	    ++vertices ;
	  }
	  Gfx_draw_3df(gfx, p0) ;
	}
}




draw_polyline_node(node, viewpt, matrix)
register Polyline_Node	*node ;
	Point3df	*viewpt ;
	Matrix3df	matrix ;
{
	if( Day_state < DARKEST )
	  while( node != NULL )
	  {
	    set_color(node->color) ;
	    Gfx_line_3df(Main_gfx, node->npts, node->points) ;

	    node = node->next ;
	  }
}



draw_polypoint_node(node, viewpt, matrix)
register Polyline_Node	*node ;
	Point3df	*viewpt ;
	Matrix3df	matrix ;
{
	if( Day_state < DARKEST )
	  while( node != NULL )
	  {
	    set_color(node->color) ;
	    Gfx_points_3df(Main_gfx, 0, node->npts, node->points) ;

	    node = node->next ;
	  }
}



draw_street_node(node, viewpt, matrix)
register Street_Node	*node ;
	Point3df	*viewpt ;
	Matrix3df	matrix ;
{
	if( Day_state > DUSK )
	  while( node != NULL )
	  {
	    if( node->dotscolor != -1 )
	    {
	      set_color(node->dotscolor) ;
	      Gfx_points_3df(Main_gfx, 0, node->npts, node->points) ;
	    }
	    node = node->next ;
	  }
	else
	  while( node != NULL )
	  {
	    if( node->linecolor != -1 )
	    {
	      set_color(node->linecolor) ;
	      Gfx_line_3df(Main_gfx, node->npts, node->points) ;
	    }
	    node = node->next ;
	  }
}



draw_polygon_node(node, viewpt, matrix)
register Polygon_Node	*node ;
	Point3df	*viewpt ;
	Matrix3df	matrix ;
{
	if( Day_state < DARKEST )
	  while( node != NULL )
	  {
	    set_color(node->color) ;
	    Gfx_polygon_3df(Main_gfx, node->nloop,
				     node->loops,
				     node->vertices,
				     0);
	    if( node->trimcolor >= 0 )
	    {
	      set_color(node->trimcolor) ;
	      polygon_trim(Main_gfx, node->nloop, node->loops, node->vertices) ;
	    }

	    node = node->next ;
	  }
}



#ifdef	COMMENT
draw_g_polygon_node(node, viewpt, matrix)
register Polygon_Node	*node ;
	Point3df	*viewpt ;
	Matrix3df	matrix ;
{
	if( Day_state < DARKEST )
	  while( node != NULL )
	  {
	    Gfx_polygon_3df_gouraud(Main_gfx, node->nloop, node->loops,
			    node->vertices);
	    node = node->next ;
	  }
}
#endif	COMMENT



draw_transform_node(node, viewpt, matrix)
register Transform_Node	*node ;
register Point3df	*viewpt ;
register Matrix3df	matrix ;
{
	Point3df	newview ;
	Matrix3df	newmatrix ;
	Matrix3df	showoff ;

	Gfx_mul_matrix(newmatrix, node->matrix, matrix) ;
	Gfx_set_matrix_3d(Main_gfx, VIEW_MAT, newmatrix) ;

	newview = *viewpt ;
	Gfx_point_transform3df(node->imatrix, &newview) ;

	/* draw the child */

	(*node->child->func)(node->child, &newview, newmatrix) ;

	/* restore old matrix */
	Gfx_set_matrix_3d(Main_gfx, VIEW_MAT, matrix) ;
}




draw_list_node(node, viewpt, matrix)
register List_Node	*node ;
register Point3df	*viewpt ;
	Matrix3df	matrix ;
{
register Desc_Node	**next = node->children ;
register Desc_Node	*ptr ;

	while( ptr = *next++ )
	  (*ptr->func)( ptr, viewpt, matrix ) ;
}





draw_bsp_node(node, viewpt, matrix)
register Bsp_Node	*node ;
register Point3df	*viewpt ;
	Matrix3df	matrix ;
{
	Point3df	vector ;
register Desc_Node	*first, *last ;
	int		color, trimcolor ;

	vector = *viewpt ;

	vector.x -= node->center.x ;
	vector.y -= node->center.y ;
	vector.z -= node->center.z ;

	if((vector.x*node->normal.x +
	    vector.y*node->normal.y +
	    vector.z*node->normal.z ) < 0.0)	/* back face */
	{
	  first = node->front_child ;
	  last = node->back_child ;
	  color = node->bcolor ;
	  trimcolor = node->btrim ;
	}
	else
	{
	  first = node->back_child ;
	  last = node->front_child ;
	  color = node->fcolor ;
	  trimcolor = node->ftrim ;
	}

	if( first != NULL )
	  (*first->func)( first, viewpt, matrix ) ;


	if( node->nloop > 0  &&  Day_state < DARKEST )
	{
	  if( color >= 0 )
	  {
	    set_color( color ) ;
	    Gfx_polygon_3df(Main_gfx, node->nloop,
				     node->loops,
				     node->vertices,
				     0 ) ;
	  }
	  if( trimcolor >= 0 )
	  {
	    set_color( trimcolor ) ;
	    polygon_trim(Main_gfx, node->nloop, node->loops, node->vertices) ;
	  }
	}


	if( last != NULL )
	  (*last->func)( last, viewpt, matrix ) ;
}



static	int
clipcheck(m,pt,cnt)
register Matrix3df	m ;
register Point3df	*pt ;
register int		cnt ;
{
	float	x,y,z,w ;
	int	flag = 0x3f, f1 ;

	while(cnt-- > 0  &&  flag != 0)
	{
	  f1 = 0 ;
	  w = m[0][3] * pt->x + m[1][3] * pt->y + m[2][3] * pt->z + m[3][3] ;
	  x = m[0][0] * pt->x + m[1][0] * pt->y + m[2][0] * pt->z + m[3][0] ;
	  if(x > w) f1 |= 0x10 ;
	  else if(x < -w) f1 |= 0x20 ;
	  y = m[0][1] * pt->x + m[1][1] * pt->y + m[2][1] * pt->z + m[3][1] ;
	  if(y > w) f1 |= 4 ;
	  else if(y < -w) f1 |= 8 ;
	  z = m[0][2] * pt->x + m[1][2] * pt->y + m[2][2] * pt->z + m[3][2] ;
	  if(z > w) f1 |= 1 ;
	  else if(z < 0.0) f1 |= 2 ;
#ifdef	COMMENT
	  else if(z < -w) f1 |= 2 ;
#endif	COMMENT
	  flag &= f1 ;
	  ++pt ;
	}

	return(flag) ;
}
	



draw_named_object(node, viewpt, matrix)
register Named_Object	*node ;
register Point3df	*viewpt ;
register Matrix3df	matrix ;
{
	Point3df	pt[8] ;
	int		flag ;
	float		dist ;

	/* load if neccessary */
	/* build transformation matrix (model space to world) */
	/* and at the same time build  (world to model space) */

	/* try to do trivial reject */
	pt[0].x = node->b0.x ; pt[0].y = node->b0.y ; pt[0].z = node->b0.z ;
	pt[1].x = node->b1.x ; pt[1].y = node->b0.y ; pt[1].z = node->b0.z ;
	pt[2].x = node->b0.x ; pt[2].y = node->b1.y ; pt[2].z = node->b0.z ;
	pt[3].x = node->b1.x ; pt[3].y = node->b1.y ; pt[3].z = node->b0.z ;
	pt[4].x = node->b0.x ; pt[4].y = node->b0.y ; pt[4].z = node->b1.z ;
	pt[5].x = node->b1.x ; pt[5].y = node->b0.y ; pt[5].z = node->b1.z ;
	pt[6].x = node->b0.x ; pt[6].y = node->b1.y ; pt[6].z = node->b1.z ;
	pt[7].x = node->b1.x ; pt[7].y = node->b1.y ; pt[7].z = node->b1.z ;
	if( clipcheck(matrix, pt, 8) )
	  return ;

	/* see if object is further away than visibility */
	{
	  register float xd,yd ;

	  if( (xd = node->b0.x - viewpt->x) < 0.0  &&
	      (xd = viewpt->x - node->b1.x) < 0.0 )
	    xd = 0.0 ;
	  else
	    xd *= xd ;

	  if( (yd = node->b0.y - viewpt->y) < 0.0  &&
	      (yd = viewpt->y - node->b1.y) < 0.0 )
	    yd = 0.0 ;
	  else
	    yd *= yd ;

	  dist = xd+yd ;
	}
	if( dist > vis_sq )
	  return ;

	current_color = -1 ;

	(*node->child->func)(node->child, viewpt, matrix) ;
}





draw_view_node(node, viewpt, matrix)
register View_Node	*node ;
register Point3df	*viewpt ;
	Matrix3df	matrix ;
{
register float		Dist ;

	{
	  register float	dx,dy,dz ;

	  dx = viewpt->x - node->center.x ;
	  dy = viewpt->y - node->center.y ;
	  dz = viewpt->z - node->center.z ;
	  Dist = dx*dx + dy*dy + dz*dz ;
	}

	while(Dist < node->thresh && node->next)
	  node = node->next ;

	if( node->child != NULL )
	  (*node->child->func)( node->child, viewpt, matrix ) ;
}





draw_terrain_node(node, viewpt, matrix)
register Terrain_Node	*node ;
register Point3df	*viewpt ;
register Matrix3df	matrix ;
{
	float		x0,y0,x1,y1 ;	/* corners of terrain		*/
	int		mx,my ;		/* indices of terrain center	*/
	float		xm,ym ;		/* center of terrain		*/
register Terrain_data	*dpx, *dpy ;	/* data pointers		*/
	Point3df	points[4] ;
static	int		loops[1] = { 4 } ;
static	int		loops3[1] = { 3 } ;
	int		cull ;		/* true if object out of sight	*/
	float		distance ;	/* distance**2 to nearest point	*/


	/* Try to cull the object, this is important */


	/* first, get distance to nearest point
	   KLUDGE ALERT: this assumes sx,sy > 0 which may not be true */

	{
	  register float xd,yd ;

	  if( (xd = node->x0 - viewpt->x) < 0.0  &&
	      (xd = viewpt->x - (node->x0+node->sx)) < 0.0 )
	    xd = 0.0 ;
	  else
	    xd *= xd ;

	  if( (yd = node->y0 - viewpt->y) < 0.0  &&
	      (yd = viewpt->y - (node->y0+node->sy)) < 0.0 )
	    yd = 0.0 ;
	  else
	    yd *= yd ;

	  distance = xd+yd ;
	}


	/* first try, see if further than visibility */

	if( distance > vis_sq )
	  return ;


	/* next try, transform all 4 points and see if they're all
	   clipped on any plane. */

	points[0] = node->p0.p ;
	points[1] = node->p1.p ;
	points[2] = node->p2.p ;
	points[3] = node->p3.p ;
	if( clipcheck(matrix, points, 4) )
	  return ;


	/* finally, see if it's greater than max_disp */

	if( distance > node->max_disp )
	{
	  set_color(node->p0.color) ;
	  Gfx_polygon_3df(Main_gfx, 1, loops, points, 0) ;

	  return ;
	}



	if( Day_state < DARKEST )
	{
	  /* ok, let's draw it */

	  node->last_disp = Now.tv_sec ;

	  if( node->data == NULL )
	    Load_terrain(node) ;




	  /* split terrain into 4 quadrants, left and right of viewpoint
	   * and front/back of viewpoint.  We have to paint all four
	   * quadrants because we don't know the view direction.
	   * first-stage clipping should eliminate unwanted patches.
	   * Note that up to 3 quadrants could be empty if viewpoint is
	   * not over the terrain
	   *
	   *	0  1  2  3  4  5  6  7  8  9	edges
	   *	+--+--+--+--+--+--+--+--+--+
	   *	| 0| 1| 2| 3| 4| 5| 6| 7| 8|	quads.
	   *	+--+--+--+--+--+--+--+--+--+
	   *	|  |  |  |  |  |  |  |  |  |
	   *	+--+--+--+--+--+--+--+--+--+
	   * ^		 ^	^		^
	   * A		 B	C		D
	   *
	   * Case A: viewpoint left of left edge of patch, mx=0,
	   *		don't paint left quads
	   *		paint right quads from mx to nx-2.
	   *
	   * Case B: viewpoint right on edge, mx=3
	   *		paint left quads from 0 to mx-1
	   *		paint right quads from mx to nx-2.
	   *
	   * Case C: viewpoint between edges, mx=5
	   *		paint left quads from 0 to mx-1
	   *		paint right quads from mx to nx-2.
	   *
	   * Case D: viewpoint right of right edge of patch, mx=nx-1
	   *		paint left quads from 0 to mx-1
	   *		don't paint right quads.
	   *
	   */

	  x0 = node->x0 ;
	  y0 = node->y0 ;
	  x1 = node->x0 + (node->nx-1) * node->dx ;
	  y1 = node->y0 + (node->ny-1) * node->dy ;

	  if(viewpt->x < x0)
	    mx = 0 ;			/* no left quadrants */
	  else if(viewpt->x > x1)
	    mx = node->nx - 1 ;		/* no right quadrants */
	  else
	    mx = ((node->nx - 1) * (viewpt->x - x0))/(x1 - x0) ;

	  if(viewpt->y < y0)
	    my = 0 ;			/* no back quadrants */
	  else if(viewpt->y > y1)
	    my = node->ny - 1 ;		/* no front quadrants */
	  else
	    my = ((node->ny - 1) * (viewpt->y - y0))/(y1 - y0) ;

	  xm = node->x0 + mx * node->dx ;
	  ym = node->y0 + my * node->dy ;


	  if( mx > 0  &&  my > 0 )	/* back-left quad */
	  {
	    points[0].x = x0 ; points[0].y = y0 ; points[0].z = 0.0 ;
	    points[1].x = xm ; points[1].y = y0 ; points[1].z = 0.0 ;
	    points[2].x = x0 ; points[2].y = ym ; points[2].z = 0.0 ;
	    points[3].x = xm ; points[3].y = ym ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data ;
	      x = x0 ;
	      for(i=0; i<mx; ++i)
	      {
		dpy = dpx ;
		y = y0 ;
		for(j=0; j<my; ++j)
		{
		  points[0].x = points[1].x = x ;
		  points[2].x = points[3].x = x + node->dx ;
		  points[0].y = points[3].y = y ;
		  points[1].y = points[2].y = y + node->dy ;
		  points[0].z = dpy->elev ;
		  points[1].z = (dpy+1)->elev ;
		  points[2].z = (dpy+node->ny+1)->elev ;
		  points[3].z = (dpy+node->ny)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops, points, 0) ;
#ifdef	COMMENT
		  points[0].x = x ;
		  points[0].y = y ;
		  points[0].z = dpy->elev ;
		  points[1].x = x + node->dx ;
		  points[1].y = y ;
		  points[1].z = (dpy+node->ny)->elev ;
		  points[2].x = x ;
		  points[2].y = y + node->dy ;
		  points[2].z = (dpy+1)->elev ;
		  points[3].x = x + node->dx ;
		  points[3].y = y + node->dy ;
		  points[3].z = (dpy+node->ny+1)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points, 0) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points+1, 0) ;
#endif	COMMENT
		  y += node->dy ;
		  ++dpy ;
		}
		x += node->dx ;
		dpx += node->ny ;
	      }
	    }
	  }



	  if( mx > 0  &&  my < node->ny-1 )	/* front-left quad */
	  {
	    points[0].x = x0 ; points[0].y = ym ; points[0].z = 0.0 ;
	    points[1].x = xm ; points[1].y = ym ; points[1].z = 0.0 ;
	    points[2].x = x0 ; points[2].y = y1 ; points[2].z = 0.0 ;
	    points[3].x = xm ; points[3].y = y1 ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data ;
	      x = x0 ;
	      for(i=0; i<mx; ++i)
	      {
		dpy = dpx + (node->ny-2) ;
		y = y1 - node->dy ;
		for(j = node->ny-2; j >= my; --j)
		{
		  points[0].x = points[1].x = x ;
		  points[2].x = points[3].x = x + node->dx ;
		  points[0].y = points[3].y = y ;
		  points[1].y = points[2].y = y + node->dy ;
		  points[0].z = dpy->elev ;
		  points[1].z = (dpy+1)->elev ;
		  points[2].z = (dpy+node->ny+1)->elev ;
		  points[3].z = (dpy+node->ny)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops, points, 0) ;
#ifdef	COMMENT
		  points[0].x = x ;
		  points[0].y = y ;
		  points[0].z = dpy->elev ;
		  points[1].x = x + node->dx ;
		  points[1].y = y ;
		  points[1].z = (dpy+node->ny)->elev ;
		  points[2].x = x ;
		  points[2].y = y + node->dy ;
		  points[2].z = (dpy+1)->elev ;
		  points[3].x = x + node->dx ;
		  points[3].y = y + node->dy ;
		  points[3].z = (dpy+node->ny+1)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points, 0) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points+1, 0) ;
#endif	COMMENT
		  y -= node->dy ;
		  --dpy ;
		}
		x += node->dx ;
		dpx += node->ny ;
	      }
	    }
	  }



	  if( mx < node->nx-1  &&  my > 0 )	/* back-right quad */
	  {
	    points[0].x = xm ; points[0].y = y0 ; points[0].z = 0.0 ;
	    points[1].x = x1 ; points[1].y = y0 ; points[1].z = 0.0 ;
	    points[2].x = xm ; points[2].y = ym ; points[2].z = 0.0 ;
	    points[3].x = x1 ; points[3].y = ym ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data + (node->nx-2)*node->ny ;
	      x = x1 - node->dx ;
	      for(i = node->nx-2; i >= mx; --i)
	      {
		dpy = dpx ;
		y = y0 ;
		for(j=0; j<my; ++j)
		{
		  points[0].x = points[1].x = x ;
		  points[2].x = points[3].x = x + node->dx ;
		  points[0].y = points[3].y = y ;
		  points[1].y = points[2].y = y + node->dy ;
		  points[0].z = dpy->elev ;
		  points[1].z = (dpy+1)->elev ;
		  points[2].z = (dpy+node->ny+1)->elev ;
		  points[3].z = (dpy+node->ny)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops, points, 0) ;
#ifdef	COMMENT
		  points[0].x = x ;
		  points[0].y = y ;
		  points[0].z = dpy->elev ;
		  points[1].x = x + node->dx ;
		  points[1].y = y ;
		  points[1].z = (dpy+node->ny)->elev ;
		  points[2].x = x ;
		  points[2].y = y + node->dy ;
		  points[2].z = (dpy+1)->elev ;
		  points[3].x = x + node->dx ;
		  points[3].y = y + node->dy ;
		  points[3].z = (dpy+node->ny+1)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points, 0) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points+1, 0) ;
#endif	COMMENT
		  y += node->dy ;
		  ++dpy ;
		}
		x -= node->dx ;
		dpx -= node->ny ;
	      }
	    }
	  }



	  if( mx < node->nx-1  &&  my < node->ny-1 )	/* front-right quad */
	  {
	    points[0].x = xm ; points[0].y = ym ; points[0].z = 0.0 ;
	    points[1].x = x1 ; points[1].y = ym ; points[1].z = 0.0 ;
	    points[2].x = xm ; points[2].y = y1 ; points[2].z = 0.0 ;
	    points[3].x = x1 ; points[3].y = y1 ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data + (node->nx-2)*node->ny ;
	      x = x1 - node->dx ;
	      for(i = node->nx-2; i >= mx; --i)
	      {
		dpy = dpx + (node->ny-2) ;
		y = y1 - node->dy ;
		for(j = node->ny-2; j >= my; --j)
		{
		  points[0].x = points[1].x = x ;
		  points[2].x = points[3].x = x + node->dx ;
		  points[0].y = points[3].y = y ;
		  points[1].y = points[2].y = y + node->dy ;
		  points[0].z = dpy->elev ;
		  points[1].z = (dpy+1)->elev ;
		  points[2].z = (dpy+node->ny+1)->elev ;
		  points[3].z = (dpy+node->ny)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops, points, 0) ;
#ifdef	COMMENT
		  points[0].x = x ;
		  points[0].y = y ;
		  points[0].z = dpy->elev ;
		  points[1].x = x + node->dx ;
		  points[1].y = y ;
		  points[1].z = (dpy+node->ny)->elev ;
		  points[2].x = x ;
		  points[2].y = y + node->dy ;
		  points[2].z = (dpy+1)->elev ;
		  points[3].x = x + node->dx ;
		  points[3].y = y + node->dy ;
		  points[3].z = (dpy+node->ny+1)->elev ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points, 0) ;
		  Gfx_polygon_3df(Main_gfx, 1, loops3, points+1, 0) ;
#endif	COMMENT
		  y -= node->dy ;
		  --dpy ;
		}
		x -= node->dx ;
		dpx -= node->ny ;
	      }
	    }
	  }
	}


	/* finally, draw objects on this terrain */

	if( node->child != NULL )
	  (*node->child->func)(node->child, viewpt, matrix) ;
}





draw_g_terrain_node(node, viewpt, matrix)
register Terrain_Node	*node ;
register Point3df	*viewpt ;
register Matrix3df	matrix ;
{
	float		x0,y0,x1,y1 ;	/* corners of terrain		*/
	int		mx,my ;		/* indices of terrain center	*/
	float		xm,ym ;		/* center of terrain		*/
register Terrain_data	*dpx, *dpy ;	/* data pointers		*/
	Point3dfc	cpoints[4] ;
	Point3df	points[4] ;
static	int		loops[1] = { 4 } ;
	int		cull ;		/* true if object out of sight	*/
	float		distance ;	/* distance**2 to nearest point	*/


	/* Try to cull the object, this is important */


	/* first, get distance to nearest point
	   KLUDGE ALERT: this assumes sx,sy > 0 which may not be true */

	{
	  register float xd,yd ;

	  if( (xd = node->x0 - viewpt->x) < 0.0  &&
	      (xd = viewpt->x - (node->x0+node->sx)) < 0.0 )
	    xd = 0.0 ;
	  else
	    xd *= xd ;

	  if( (yd = node->y0 - viewpt->y) < 0.0  &&
	      (yd = viewpt->y - (node->y0+node->sy)) < 0.0 )
	    yd = 0.0 ;
	  else
	    yd *= yd ;

	  distance = xd+yd ;
	}


	/* first try, see if further than visibility */

	if( distance > vis_sq )
	  return ;


	/* next try, transform all 4 points and see if they're all
	   clipped on any plane. */

	points[0] = node->p0.p ;
	points[1] = node->p1.p ;
	points[2] = node->p2.p ;
	points[3] = node->p3.p ;
	if( clipcheck(matrix, points, 4) )
	  return ;


	/* finally, see if it's greater than max_disp */

	if( distance > node->max_disp )
	{
	  set_color(node->p0.color) ;
	  Gfx_polygon_3df(Main_gfx, 1, loops, points, 0) ;

	  return ;
	}



	if( Day_state < DARKEST )
	{
	  /* ok, let's draw it */

	  node->last_disp = Now.tv_sec ;

	  if( node->data == NULL )
	    Load_terrain(node) ;




	  /* split terrain into 4 quadrants, left and right of viewpoint
	   * and front/back of viewpoint.  We have to paint all four
	   * quadrants because we don't know the view direction.
	   * first-stage clipping should eliminate unwanted patches.
	   * Note that up to 3 quadrants could be empty if viewpoint is
	   * not over the terrain
	   */

	  x0 = node->x0 ;
	  y0 = node->y0 ;
	  x1 = node->x0 + (node->nx-1) * node->dx ;
	  y1 = node->y0 + (node->ny-1) * node->dy ;

	  if(viewpt->x < x0)
	    mx = 0 ;			/* no left quadrants */
	  else if(viewpt->x > x1)
	    mx = node->nx ;		/* no right quadrants */
	  else
	    mx = ((node->nx - 1) * (viewpt->x - x0))/(x1 - x0) ;

	  if(viewpt->y < y0)
	    my = 0 ;			/* no back quadrants */
	  else if(viewpt->y > y1)
	    my = node->ny ;		/* no front quadrants */
	  else
	    my = ((node->ny - 1) * (viewpt->y - y0))/(y1 - y0) ;

	  xm = node->x0 + mx * node->dx ;
	  ym = node->y0 + my * node->dy ;


	  if( mx > 0  &&  my > 0 )	/* back-left quad */
	  {
	    points[0].x = x0 ; points[0].y = y0 ; points[0].z = 0.0 ;
	    points[1].x = xm ; points[1].y = y0 ; points[1].z = 0.0 ;
	    points[2].x = x0 ; points[2].y = ym ; points[2].z = 0.0 ;
	    points[3].x = xm ; points[3].y = ym ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data ;
	      x = x0 ;
	      for(i=0; i<mx; ++i)
	      {
		dpy = dpx ;
		y = y0 ;
		for(j=0; j<my; ++j)
		{
		  cpoints[0].p.x = cpoints[1].p.x = x ;
		  cpoints[2].p.x = cpoints[3].p.x = x + node->dx ;
		  cpoints[0].p.y = cpoints[3].p.y = y ;
		  cpoints[1].p.y = cpoints[2].p.y = y + node->dy ;
		  cpoints[0].p.z = dpy->elev ;
		  cpoints[0].color = dpy->color << 16 ;
		  cpoints[1].p.z = (dpy+1)->elev ;
		  cpoints[1].color = (dpy+1)->color << 16 ;
		  cpoints[2].p.z = (dpy+node->ny+1)->elev ;
		  cpoints[2].color = (dpy+node->ny+1)->color << 16 ;
		  cpoints[3].p.z = (dpy+node->ny)->elev ;
		  cpoints[3].color = (dpy+node->ny)->color << 16 ;
		  Gfx_polygon_3df_gouraud(Main_gfx, 1, loops, cpoints, 0) ;
		  y += node->dy ;
		  ++dpy ;
		}
		x += node->dx ;
		dpx += node->ny ;
	      }
	    }
	  }



	  if( mx > 0  &&  my < node->ny )	/* front-left quad */
	  {
	    points[0].x = x0 ; points[0].y = ym ; points[0].z = 0.0 ;
	    points[1].x = xm ; points[1].y = ym ; points[1].z = 0.0 ;
	    points[2].x = x0 ; points[2].y = y1 ; points[2].z = 0.0 ;
	    points[3].x = xm ; points[3].y = y1 ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data ;
	      x = x0 ;
	      for(i=0; i<mx; ++i)
	      {
		dpy = dpx + (node->ny-2) ;
		y = y1 - node->dy ;
		for(j = node->ny-2; j >= my; --j)
		{
		  cpoints[0].p.x = cpoints[1].p.x = x ;
		  cpoints[2].p.x = cpoints[3].p.x = x + node->dx ;
		  cpoints[0].p.y = cpoints[3].p.y = y ;
		  cpoints[1].p.y = cpoints[2].p.y = y + node->dy ;
		  cpoints[0].p.z = dpy->elev ;
		  cpoints[0].color = dpy->color << 16 ;
		  cpoints[1].p.z = (dpy+1)->elev ;
		  cpoints[1].color = (dpy+1)->color << 16 ;
		  cpoints[2].p.z = (dpy+node->ny+1)->elev ;
		  cpoints[2].color = (dpy+node->ny+1)->color << 16 ;
		  cpoints[3].p.z = (dpy+node->ny)->elev ;
		  cpoints[3].color = (dpy+node->ny)->color << 16 ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df_gouraud(Main_gfx, 1, loops, cpoints, 0) ;
		  y -= node->dy ;
		  --dpy ;
		}
		x += node->dx ;
		dpx += node->ny ;
	      }
	    }
	  }



	  if( mx < node->nx  &&  my > 0 )	/* back-right quad */
	  {
	    points[0].x = xm ; points[0].y = y0 ; points[0].z = 0.0 ;
	    points[1].x = x1 ; points[1].y = y0 ; points[1].z = 0.0 ;
	    points[2].x = xm ; points[2].y = ym ; points[2].z = 0.0 ;
	    points[3].x = x1 ; points[3].y = ym ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data + (node->nx-2)*node->ny ;
	      x = x1 - node->dx ;
	      for(i = node->nx-2; i >= mx; --i)
	      {
		dpy = dpx ;
		y = y0 ;
		for(j=0; j<my; ++j)
		{
		  cpoints[0].p.x = cpoints[1].p.x = x ;
		  cpoints[2].p.x = cpoints[3].p.x = x + node->dx ;
		  cpoints[0].p.y = cpoints[3].p.y = y ;
		  cpoints[1].p.y = cpoints[2].p.y = y + node->dy ;
		  cpoints[0].p.z = dpy->elev ;
		  cpoints[0].color = dpy->color << 16 ;
		  cpoints[1].p.z = (dpy+1)->elev ;
		  cpoints[1].color = (dpy+1)->color << 16 ;
		  cpoints[2].p.z = (dpy+node->ny+1)->elev ;
		  cpoints[2].color = (dpy+node->ny+1)->color << 16 ;
		  cpoints[3].p.z = (dpy+node->ny)->elev ;
		  cpoints[3].color = (dpy+node->ny)->color << 16 ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df_gouraud(Main_gfx, 1, loops, cpoints, 0) ;
		  y += node->dy ;
		  ++dpy ;
		}
		x -= node->dx ;
		dpx -= node->ny ;
	      }
	    }
	  }



	  if( mx < node->nx  &&  my < node->ny )	/* front-right quad */
	  {
	    points[0].x = xm ; points[0].y = ym ; points[0].z = 0.0 ;
	    points[1].x = x1 ; points[1].y = ym ; points[1].z = 0.0 ;
	    points[2].x = xm ; points[2].y = y1 ; points[2].z = 0.0 ;
	    points[3].x = x1 ; points[3].y = y1 ; points[3].z = 0.0 ;
	    if( !clipcheck(matrix, points, 4) )
	    {
	      register int i,j ;
	      register float x,y ;
	      dpx = node->data + (node->nx-2)*node->ny ;
	      x = x1 - node->dx ;
	      for(i = node->nx-2; i >= mx; --i)
	      {
		dpy = dpx + (node->ny-2) ;
		y = y1 - node->dy ;
		for(j = node->ny-2; j >= my; --j)
		{
		  cpoints[0].p.x = cpoints[1].p.x = x ;
		  cpoints[2].p.x = cpoints[3].p.x = x + node->dx ;
		  cpoints[0].p.y = cpoints[3].p.y = y ;
		  cpoints[1].p.y = cpoints[2].p.y = y + node->dy ;
		  cpoints[0].p.z = dpy->elev ;
		  cpoints[0].color = dpy->color << 16 ;
		  cpoints[1].p.z = (dpy+1)->elev ;
		  cpoints[1].color = (dpy+1)->color << 16 ;
		  cpoints[2].p.z = (dpy+node->ny+1)->elev ;
		  cpoints[2].color = (dpy+node->ny+1)->color << 16 ;
		  cpoints[3].p.z = (dpy+node->ny)->elev ;
		  cpoints[3].color = (dpy+node->ny)->color << 16 ;
		  set_color(dpy->color) ;
		  Gfx_polygon_3df_gouraud(Main_gfx, 1, loops, cpoints, 0) ;
		  y -= node->dy ;
		  --dpy ;
		}
		x -= node->dx ;
		dpx -= node->ny ;
	      }
	    }
	  }
	}


	/* finally, draw objects on this terrain */

	if( node->child != NULL )
	  (*node->child->func)(node->child, viewpt, matrix) ;
}






print_mat(mat)
register Matrix3df	mat ;
{
	int	i,j ;

	for(i=0; i<4; ++i)
	{
	  for(j=0; j<4; ++j)
	    printf("%10.3f", mat[i][j]) ;
	  putchar('\n') ;
	}
}
