/*	%Z%%M% %I% %E% SMI	*/


#ifndef FLIGHT_H
#define FLIGHT_H


#include <sys/time.h>
#include "parameters.h"



#ifndef	GFX_DEFINED

typedef struct {
	  float	x,y,z ;
	} Point3df ;

typedef float	Matrix3df[4][4] ;

typedef	struct {
	  Point3df	p ;
	  int		color ;
	} Point3dfc ;

#endif	GFX_DEFINED




/*	*** GAME PARAMETERS ***		*/

/****
 *
 * description of ship's position & orientation in 3-space.
 *
 * right-hand rule used throughout.
 *
 * ship's position is given
 * as a 3-space location in m.  Ship's forward orientation
 * is given as two 3-space unit vectors.  Velocity is a scalar (this
 * may change)
 *
 * Me_Forward2 is the normalized projection of the
 * plane's orientation on the xy plane.
 *
 * The ship's z-coordinate is looking straight along the nose, +x is to
 * the right & +y is up
 *
 ****/

	struct itimerval Main_Timer, Sleep_Timer ;

	struct timeval	Last_Time, Now, Daytime ;
	float	Dtime ;

	int	debug_level ;

	Point3df	Me_Delta ;
	Point3df	Me_Posn ;		/* meters */
	float		Me_Speed ;		/* meters/sec */
	float		Me_Speed_z ;		/* Vert. speed */
	float		Me_Power ;		/* percent */
	float		Me_Throttle ;		/* throttle setting */
	float		Me_Heading ;		/* heading, 0=N, 90=E */
	float		Me_Pitch ;		/* pitch above horizontal */
	float		Me_Roll ;		/* roll to right */
	float		Me_ROT ;		/* rate of turn */
	float		Head_cos, Head_sin ;	/* cos,sin of heading */
	float		Pitch_cos, Pitch_sin ;	/* cos,sin of pitch */
	float		Roll_cos, Roll_sin ;	/* cos,sin of roll */
	Point3df	Me_Forward, Me_Up, Me_Right ;
	Point3df	Me_Motion ;		/*direction of motion */
	Point3df	Me_Forward2 ;
	Matrix3df	Me_Matrix ;		/* map ship to WC */
	Matrix3df	Me_imatrix ;		/* map WC to ship */

	float	Kp1 ;		/* rate of power change */
	float	PowerMax ;	/* max power, kg*m^2/sec^3 */
	float	Me_Mass ;	/* mass of plane */
	float	Dind, Dpar ;	/* induced and parasitic drag coeffs. */
	float	Ks1 ;		/* rate of speed change */
	float	Vs0 ;		/* stall speed for standard configuration */
	float	Zs ;		/* minimum stalling altitude */
	float	Ksf ;		/* dependency of Vs on flaps */



	/* Weather info */

	float	visibility ;			/* visibility in meters */
	float	vis_sq, vis_2, vis_2_sq ;	/* vis**2, vis/2, (vis/2)**2 */
	int	cloud_1, cloud_2 ;		/* flags */
	int	cloud_base_1, cloud_tops_1 ;	/* first cloud layer */
	int	cloud_base_2, cloud_tops_2 ;	/* second cloud layer */
	Point3df	Sun_vec ;		/* vector to sun for shading */
	int	Day_state ;
	float	wind_dir, wind_speed ;		/* wind info (radians true) */
	float	wind_sin, wind_cos ;


	/* various user modes */

	int	hood ;			/* "under the hood" */
	int	map ;			/* show moving map */
	float	maprange ;		/* how many km to view */
	int	trail ;			/* show trail on map */
	Point2df trail_dat[MAX_TRAIL] ;	/* where we've been */
	int	trail_ptr, trail_num ;	/* pointers into trail_dat */
	float	trail_dtime ;		/* how long since last entry */

#define	DAY	0
#define	DUSK	1
#define	DARK	2
#define	DARKEST	3




#endif	FLIGHT_H
