#ifndef lint
static  char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include <suntool/sunview.h>
#include <suntool/canvas.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"

static	Instrument	*first, *create_dg(), *create_vg(),
			*create_airspeed(), *create_vsi(),
			*create_throttle(), *create_alt(),
			*create_ai(), *create_clock(), *create_adf(),
			*create_vor() ;

static	Instrument	*grabio ;
static	Cursor		old_cursor, instr_cursor ;
static	int		lastx,lasty ;
static	int		firstx,firsty ;


extern	Canvas		Main_Canvas ;

Instrument	*vor1, *vor2, *adf ;


#define	Add_ins(ptr)	((ptr)->next = first, first = (ptr))


init_instruments()
{
	Instrument	*ptr ;

	first = NULL ;
	grabio = NULL ;

	instr_cursor = cursor_create(CURSOR_SHOW_CURSOR,FALSE,0) ;

	ptr = create_clock(.01,.17, .15,.15, "" ) ;
	Add_ins( ptr ) ;

	ptr = create_airspeed(.17,.17, .15,.15, "" ) ;
	Add_ins( ptr ) ;

	ptr = create_ai(.33,.17, .15,.15, "" ) ;
	Add_ins( ptr ) ;

	ptr = create_dg(.33,.01, .15,.15, "" ) ;
	Add_ins( ptr ) ;

	ptr = create_alt(.49,.17, .15,.15, "" ) ;
	Add_ins( ptr ) ;

	ptr = create_vsi(.49,.01, .15,.15, "" ) ;
	Add_ins( ptr ) ;

	ptr = create_throttle(.66,.01, .05,.23, "" ) ;
	Add_ins( ptr ) ;

	ptr = create_vg(.94,.2, .05,.05, "") ;
	Add_ins( ptr ) ;

	adf = ptr = create_adf(.75,.17, .15,.15, "" ) ;
	Add_ins( ptr ) ;

	vor1 = ptr = create_vor(.01,.01, .15,.15, "y" ) ;
	Add_ins( ptr ) ;

	vor2 = ptr = create_vor(.17,.01, .15,.15, "" ) ;
	Add_ins( ptr ) ;
}



draw_instruments()
{
	Instrument	*ptr ;

	for( ptr = first; ptr != NULL; ptr = ptr->next )
	  if( ptr->redraw != NULL )
	    (*ptr->redraw)(Main_gfx, ptr) ;
}



update_instruments()
{
	Instrument	*ptr ;

	for( ptr = first; ptr != NULL; ptr = ptr->next )
	  if( ptr->update != NULL )
	    (*ptr->update)(Main_gfx, ptr) ;
}



resize_instruments(width,height)
	int	width,height ;
{
	Instrument	*ptr ;

	for( ptr = first; ptr != NULL; ptr = ptr->next )
	{
	  ptr->ix0 = ptr->x0 * width ;
	  ptr->iy0 = height - ptr->y1 * height ;
	  ptr->ix1 = ptr->x1 * width ;
	  ptr->iy1 = height - ptr->y0 * height ;
	}
}




int
instrument_input(event)
	Event	*event ;
{
register Instrument	*ptr ;
register int		ix = event_x(event), iy = event_y(event) ;
	int		found = 0 ;
	int		dx,dy ;
	Rect		*rect ;

	dx = event_x(event) - lastx ;
	dy = event_y(event) - lasty ;
	lastx = event_x(event) ;
	lasty = event_y(event) ;

	if( grabio != NULL )
	{
	  switch( event_id(event) )
	  {
	    case LOC_WINEXIT:
	    case LOC_RGNEXIT:
	      rect = (Rect *) window_get(Main_Canvas, WIN_RECT) ;
	      lastx = rect->r_width/2 ;
	      lasty = rect->r_height/2 ;
	      window_set(Main_Canvas, WIN_MOUSE_XY, lastx,lasty, 0) ;
	      break ;

	    case LOC_WINENTER:
	    case LOC_RGNENTER:
	      break ;

	    default:
	      if( grabio->input != NULL )
		    (*grabio->input)(grabio,event,dx,dy) ;
	      break ;
	  }
	  found = 1 ;
	}
	else if( event_id(event) != LOC_DRAG  &&  event_id(event) != LOC_MOVE )
	  for( ptr = first; ptr != NULL; )
	  {
	    if( ix >= ptr->ix0  &&  ix <= ptr->ix1  &&
		iy >= ptr->iy0  &&  iy <= ptr->iy1)
	    {
	      if( ptr->input != NULL )
		(*ptr->input)(ptr,event,dx,dy) ;
	      ptr = NULL ;
	      found = 1 ;
	    }
	    else
	      ptr = ptr->next ;
	  }
	return found ;
}




instrument_grabio(instrument, flag)
	Instrument	*instrument ;
	int		flag ;
{
	Cursor		new_cursor ;

	if( flag )
	{
	  grabio = instrument ;
	  old_cursor = (Cursor) window_get( Main_Canvas, WIN_CURSOR) ;
	  new_cursor = instr_cursor ;
	  firstx = lastx ;
	  firsty = lasty ;
	}
	else
	{
	  grabio = NULL ;
	  new_cursor = old_cursor ;
	  window_set(Main_Canvas, WIN_MOUSE_XY, firstx,firsty, 0) ;
	}
	window_set( Main_Canvas, WIN_CURSOR, new_cursor, 0 ) ;
}
