#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

/*
 * this file draws moving map image.
 */


#include <stdio.h>
#include <sunwindow/window_hs.h>
#include <math.h>
#include "gfx.h"
#include "flight.h"
#include "object_types.h"
#include "graphics.h"
#include "instrument.h"

extern	int	debug_level ;

extern	Instrument	*vor1, *vor2, *adf ;

#define	MAX_ITEMS	100

static	short	airport_data[] = {
	0x0000,0x0100,0x0380,0x1FF0,0x1C70,0x1830,0x3018,0x701C,
	0x3018,0x1830,0x1C70,0x1FF0,0x0380,0x0100,0x0000,0x000} ;

static	short	adf_data[]={
	0x0000,0x0540,0x0000,0x1290,0x0820,0x4384,0x1450,0x4544,
	0x1450,0x4384,0x0820,0x1290,0x0000,0x0540,0x0000,0x0000};

static	short	vor_data[]={
	0x0000,0x0000,0x0000,0x0380,0x0380,0x0280,0x0440,0x0920,
	0x0C60,0x0EE0,0x07C0,0x0000,0x0000,0x0000,0x0000,0x0000};

static	short	plane_data[] = {
	0x0000,0x0000,0x0180,0x0180,0x1FF8,0x1FF8,0x0180,0x0180,
	0x0180,0x0180,0x03C0,0x07E0,0x0000,0x0000,0x0000,0x0000} ;

static	mpr_static(airport_pr, 16,16,1,airport_data) ;
static	mpr_static(adf_pr, 16,16,1,adf_data) ;
static	mpr_static(vor_pr, 16,16,1,vor_data) ;
static	mpr_static(plane_pr, 16,16,1,plane_data) ;

static	int	num_items ;
static	struct	{
	  float	x,y ;
	  enum {AIRPORT, VOR, ADF, PLANE} type ;
	} items[MAX_ITEMS] ;

#define	MAP_LON(lod,lom,los)	(cvd(lod,lom,los) - LONB)*(RAD*RADIUS*COSLAT);
#define	MAP_LAT(lad,lam,las)	(cvd(lad,lam,las) - LATB) * (RAD*RADIUS) ;

#define	FLUSH()		while( (i=getc(ifile)) != EOF  &&  i != '\n' )

load_map()
{
	int	i,n ;
	FILE	*ifile ;
	int	lad,lam,las, lod,lom,los ;
	int	vor_type, course ;
	float	cvd() ;
	int	line ;

	num_items = 0 ;
	items[0].x = 0. ;
	items[0].y = 0. ;
	items[0].type = PLANE ;
	++num_items ;

	items[1].x = 0. ;
	items[1].y = 0. ;
	items[1].type = AIRPORT ;
	++num_items ;

	ifile = fopen("runways.dat","r") ;
	if( ifile != NULL )
	{
	  line = 1 ;
	  while(num_items < MAX_ITEMS)
	  {
	    n = fscanf(ifile, "%d.%d.%d %d.%d.%d %*d %*d %*s",
		&lad,&lam,&las, &lod,&lom,&los) ;
	    if( n == EOF )
	      break ;
	    else if( n != 6 )
	    {
	      fprintf(stderr, "runways.dat, syntax error in line %d\n", line) ;
	      FLUSH() ;
	    }
	    else
	    {
	      items[num_items].x = MAP_LON(lod,lom,los) ;
	      items[num_items].y = MAP_LAT(lad,lam,las) ;
	      items[num_items].type = AIRPORT ;
	      ++num_items ;
	    }
	    ++line ;
	  }
	  fclose(ifile) ;
	}

	ifile = fopen("adfs.dat","r") ;
	if( ifile != NULL )
	{
	  line = 1 ;
	  while(num_items < MAX_ITEMS)
	  {
	    n = fscanf(ifile, "%*d %d.%d.%d %d.%d.%d %*s %*s",
		&lad,&lam,&las, &lod,&lom,&los) ;
	    if( n == EOF )
	      break ;
	    else if( n != 6 )
	    {
	      fprintf(stderr, "adfs.dat, syntax error in line %d\n", line) ;
	      FLUSH() ;
	    }
	    else
	    {
	      items[num_items].x = MAP_LON(lod,lom,los) ;
	      items[num_items].y = MAP_LAT(lad,lam,las) ;
	      items[num_items].type = ADF ;
	      ++num_items ;
	    }
	    ++line ;
	  }
	  fclose(ifile) ;
	}

	ifile = fopen("vors.dat","r") ;
	if( ifile != NULL )
	{
	  line = 1 ;
	  while(num_items < MAX_ITEMS)
	  {
	    n = fscanf(ifile, "%*d %d.%d.%d %d.%d.%d %*d %d %d %*s %*s",
		&lad,&lam,&las, &lod,&lom,&los, &vor_type, &course) ;
	    if( n == EOF )
	      break ;
	    else if( n != 8 )
	    {
	      fprintf(stderr, "vors.dat, syntax error in line %d\n", line) ;
	      FLUSH() ;
	    }
	    else
	    {
	      items[num_items].x = MAP_LON(lod,lom,los) ;
	      items[num_items].y = MAP_LAT(lad,lam,las) ;
	      items[num_items].type = VOR ;
	      if( vor_type != 0 )
	      {
		/* offset localizers and ILSs 2000' to put them at end of
		   runway */
		items[num_items].x -= sin(course*RAD) * 2000*FEET ;
		items[num_items].y -= cos(course*RAD) * 2000*FEET ;
	      }
	      ++num_items ;
	    }
	    ++line ;
	  }
	  fclose(ifile) ;
	}
}

draw_map(gfx,x0,y0,x1,y1)
register Gfx_context	*gfx ;
	float		x0,y0,x1,y1 ;
{
	int	i,j,k,iret ;
	int	ix,iy ;
	int	ix0,ix1,iy0,iy1,ixc,iyc ;
	Matrix3df	matrix, R_matrix ;
	Point3df	p, q ;
	Point3df	p1,p2,p3,p4,p5 ;
register Point3df	*ptr ;
	Rect	*rect, *Gfx_rect() ;
static	Rect	onepix = {0,0,1,1} ;
static	int	count ;
	int	color ;
	float	xscale, yscale ;
	Pixrect	*pr ;

	rect = Gfx_rect(gfx) ;
	ix0 = rect->r_width*x0 ;
	ix1 = rect->r_width*x1 ;
	iy0 = rect->r_height*(1.-y1) ;
	iy1 = rect->r_height*(1.-y0) ;
	ixc = (ix0+ix1)/2 ;
	iyc = (iy0+iy1)/2 ;


	xscale = (ix1-ix0)*.5/maprange ;
	yscale = (iy1-iy0)*.5/maprange ;


	Gfx_viewport_2d(gfx, x0, x1, y0, y1) ;
	Gfx_clear_vp(gfx, Color_Mapping[GREY][0], x0, x1, y0, y1) ;
	Gfx_set_clip_planes(Main_gfx,0x3c) ;

	/* lay out glyphs */

	items[0].x = Me_Posn.x ;
	items[0].y = Me_Posn.y ;

	for(i=0; i<num_items; ++i)
	{
	  ix = (items[i].x - Me_Posn.x) * xscale + ixc ;
	  iy = -(items[i].y - Me_Posn.y) * yscale + iyc ;
	  if( ix >= ix0  &&  ix <= ix1  &&
	      iy >= iy0  &&  iy <= iy1 )
	  {
	    switch( items[i].type )
	    {
	      case ADF:
		color = Color_Mapping[RED][0] ;
		pr = &adf_pr ;
		break ;
	      case VOR:
		color = Color_Mapping[BLUE][0] ;
		pr = &vor_pr ;
		break ;
	      case AIRPORT:
		color = Color_Mapping[GREEN][0] ;
		pr = &airport_pr ;
		break ;
	      case PLANE:
		color = Color_Mapping[WHITE][0] ;
		pr = &plane_pr ;
		break ;
	    }
	    color = PIX_SRC | PIX_COLOR(Gfx_dbuf_get_color(gfx,color)) ;
	    pw_stencil(Main_pw, ix-8,iy-8,16,16, color, pr,0,0, pr,0,0) ;
	  }
	}

	Gfx_window_mat(Me_Posn.x-maprange,Me_Posn.x+maprange,
			Me_Posn.y-maprange,Me_Posn.y+maprange,0.,1.,matrix) ;
	Gfx_set_matrix_3d(gfx, VIEW_MAT, matrix) ;
	Gfx_select_matrix(gfx, VIEW_MAT) ;

	adf_map(gfx, adf) ;
	vor_map(gfx, vor1) ;
	vor_map(gfx, vor2) ;
}
