#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <suntool/sunview.h>
#include <suntool/panel.h>
#include "math.h"
#include "gfx.h"
#include "flight.h"
#include "graphics.h"
#include "object_types.h"


extern	Frame		Main_Frame ;
extern	Frame		Props_frame ;
extern	Panel		Props_panel ;

static	Panel_item	vis_item, cloud_1_item, base_1_item, tops_1_item,
			apply_1_item, zoom1_item, zoom10_item,
			wind1_item, wind2_item ;

static	void	visibility_proc(), cloud_1_proc(), apply_proc_1(),
		zoom_proc(), home_proc(), hold_proc(), recall_proc(),
		apply_proc2() ;

static	Point3df	H_Posn, H_Forward, H_Up, H_Right ;
static	float		H_Roll, H_Pitch, H_Heading ;


Init_panel()
{
	char	line[40] ;
	int	row=0 ;

	Props_frame = window_create(Main_Frame, FRAME,
		FRAME_LABEL, "Flight Properties",
		FRAME_SHOW_LABEL, TRUE,
		0 ) ;
	if(Props_frame == NULL)
	  exit(1) ;

	Props_panel = window_create(Props_frame, PANEL, 0) ;
	if(Props_panel == NULL)
	  exit(1) ;

	vis_item = panel_create_item(Props_panel, PANEL_CYCLE,
	    PANEL_LABEL_STRING, "Visibility:",
	    PANEL_CHOICE_STRINGS, "100 sm", "20 sm", "10 sm", "7 sm",
		"5 sm", "3 sm", "2 sm", "1 sm", "1/2 sm", "1/4 sm", 0,
	    PANEL_VALUE, 0,
	    PANEL_NOTIFY_PROC, visibility_proc,
	    PANEL_ITEM_X, ATTR_COL(0),
	    PANEL_ITEM_Y, ATTR_ROW(row++),
	    0 ) ;

	sprintf(line,"%d", wind_dir ) ;
	wind1_item = panel_create_item(Props_panel, PANEL_TEXT,
	    PANEL_LABEL_STRING, "wind direction, degrees:",
	    PANEL_VALUE, line,
	    PANEL_VALUE_DISPLAY_LENGTH, 5,
	    PANEL_ITEM_X, ATTR_COL(0),
	    PANEL_ITEM_Y, ATTR_ROW(row++),
	    0 ) ;

	sprintf(line,"%d", (int) (wind_speed*(MILES/HOURS)) ) ;
	wind2_item = panel_create_item(Props_panel, PANEL_TEXT,
	    PANEL_LABEL_STRING, "wind speed, knots:",
	    PANEL_VALUE, line,
	    PANEL_VALUE_DISPLAY_LENGTH, 5,
	    0 ) ;

	(void) panel_create_item(Props_panel, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, apply_proc2,
	    PANEL_LABEL_IMAGE, panel_button_image(Props_panel,"apply",0,0),
	    0 ) ;

	cloud_1_item = panel_create_item(Props_panel, PANEL_CYCLE,
	    PANEL_LABEL_STRING, "Clouds:",
	    PANEL_CHOICE_STRINGS, "off", "on", 0,
	    PANEL_VALUE, cloud_1 ? 1 : 0,
	    PANEL_NOTIFY_PROC, cloud_1_proc,
	    PANEL_ITEM_X, ATTR_COL(0),
	    PANEL_ITEM_Y, ATTR_ROW(row++),
	    0 ) ;

	sprintf(line,"%d", (int) (cloud_base_1*(1./FEET)+.5) ) ;
	base_1_item = panel_create_item(Props_panel, PANEL_TEXT,
	    PANEL_LABEL_STRING, "cloud base, ft:",
	    PANEL_VALUE, line,
	    PANEL_SHOW_ITEM, cloud_1,
	    PANEL_VALUE_DISPLAY_LENGTH, 10,
	    0 ) ;

	sprintf(line,"%d", (int) (cloud_tops_1*(1./FEET)+.5) ) ;
	tops_1_item = panel_create_item(Props_panel, PANEL_TEXT,
	    PANEL_LABEL_STRING, "cloud tops, ft:",
	    PANEL_VALUE, line,
	    PANEL_SHOW_ITEM, cloud_1,
	    PANEL_VALUE_DISPLAY_LENGTH, 10,
	    0 ) ;

	apply_1_item = panel_create_item(Props_panel, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, apply_proc_1,
	    PANEL_LABEL_IMAGE, panel_button_image(Props_panel,"apply",0,0),
	    PANEL_SHOW_ITEM, cloud_1,
	    0 ) ;

	zoom1_item = panel_create_item(Props_panel, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, zoom_proc,
	    PANEL_LABEL_IMAGE, panel_button_image(Props_panel,"zoom 1",0,0),
	    PANEL_ITEM_X, ATTR_COL(0),
	    PANEL_ITEM_Y, ATTR_ROW(row++),
	    0 ) ;

	zoom10_item = panel_create_item(Props_panel, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, zoom_proc,
	    PANEL_LABEL_IMAGE, panel_button_image(Props_panel,"zoom 10",0,0),
	    0 ) ;

	(void) panel_create_item(Props_panel, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, home_proc,
	    PANEL_LABEL_IMAGE, panel_button_image(Props_panel,"home",0,0),
	    0 ) ;

	(void) panel_create_item(Props_panel, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, hold_proc,
	    PANEL_LABEL_IMAGE, panel_button_image(Props_panel,"hold",0,0),
	    0 ) ;

	(void) panel_create_item(Props_panel, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, recall_proc,
	    PANEL_LABEL_IMAGE, panel_button_image(Props_panel,"recall",0,0),
	    0 ) ;
}



panel_start()
{
	window_set(Props_frame, WIN_SHOW, TRUE, 0) ;
}



static	void
visibility_proc(item, value, event)
	Panel_item	item ;
	int		value ;
	Event		*event ;
{
static	float	values[] = {100., 20., 10., 7., 5., 3., 2., 1., .5, .25} ;

	visibility = MILES * values[value] ;
	vis_sq = visibility*visibility ;
	vis_2 = visibility/2.0 ;
	vis_2_sq = vis_2*vis_2 ;
	Gfx_persp_window_mat(0.,5.,-5.,5.,-2.5,2.5,5.,visibility,P_matrix) ;
	Color_compute(Now.tv_sec) ;
}



static	void
cloud_1_proc(item, value, event)
	Panel_item	item ;
	int		value ;
	Event		*event ;
{
	cloud_1 = value == 1 ;
	panel_set(base_1_item, PANEL_SHOW_ITEM, cloud_1, 0 ) ;
	panel_set(tops_1_item, PANEL_SHOW_ITEM, cloud_1, 0 ) ;
	panel_set(apply_1_item, PANEL_SHOW_ITEM, cloud_1, 0 ) ;
}



static	float
get_value(item, value, scale, format)
	Panel_item	item ;
	float		value ;
	float		scale ;
	char		*format ;
{
	char	*string ;
	char	line[40] ;
	float	rval, atof() ;

	string = (char *) panel_get_value(item) ;
	if( string != NULL  &&  *string != '\0' )
	    rval = scale * atof(string) ;
	else
	{
	    sprintf(line,format, (int) (value/scale+.005) ) ;
	    panel_set(item, PANEL_VALUE, line, 0 ) ;
	    rval = value ;
	}
	return rval ;
}



static	void
apply_proc_1(item, event)
	Panel_item	item ;
	Event		*event ;
{
	cloud_base_1 = get_value(base_1_item, (float) cloud_base_1,
		FEET, "%.0f")+.5 ;
	cloud_tops_1 = get_value(tops_1_item, (float) cloud_tops_1,
		FEET, "%.0f")+.5 ;
}



static	void
apply_proc2(item, event)
	Panel_item	item ;
	Event		*event ;
{
	wind_dir = get_value(wind1_item, wind_dir, RAD, "%.0f") ;
	wind_speed = get_value(wind2_item, wind_speed, NM/HOURS, "%.0f") ;
	wind_sin = -sin(wind_dir) * wind_speed ;
	wind_cos = -cos(wind_dir) * wind_speed ;
}



static	void
zoom_proc(item, event)
	Panel_item	item ;
	Event		*event ;
{
	float	distance = item==zoom1_item ? (1*MILES) : (10*MILES) ;

	Me_Posn.x += distance * Me_Forward.x ;
	Me_Posn.y += distance * Me_Forward.y ;
}



static	void
home_proc(item, event)
	Panel_item	item ;
	Event		*event ;
{
	Me_Posn.x = 0.0 ;
	Me_Posn.y = -3800.0 ;
	Me_Posn.z =  244.0 ;
	Me_Heading = 0.0 ; Head_sin = 0.0 ; Head_cos = 1.0 ;
	Me_Pitch = 0.0 ; Pitch_sin = 0.0 ; Pitch_cos = 1.0 ;
	Me_Roll = 0.0 ; Roll_sin = 0.0 ; Roll_cos = 1.0 ;
	Me_Forward.x = 0.0 ;
	Me_Forward.y = 1.0 ;
	Me_Forward.z = 0.0 ;
	Me_Up.x = 0.0 ;
	Me_Up.y = 0.0 ;
	Me_Up.z = 1.0 ;
	Me_Right.x = 1.0 ;
	Me_Right.y = 0.0 ;
	Me_Right.z = 0.0 ;
	Me_Delta.x = 0.0 ;
	Me_Delta.y = 0.0 ;
	Me_Delta.z = 0.0 ;
	Me_Speed = DEFAULT_SPEED ;
	Me_Speed_z = 0.0 ;
}



static	void
hold_proc(item, event)
	Panel_item	item ;
	Event		*event ;
{
	H_Posn = Me_Posn ;
	H_Forward = Me_Forward ;
	H_Up = Me_Up ;
	H_Right = Me_Right ;
	H_Roll = Me_Roll ;
	H_Pitch = Me_Pitch ;
	H_Heading = Me_Heading ;
}



static	void
recall_proc(item, event)
	Panel_item	item ;
	Event		*event ;
{
	Me_Posn = H_Posn ;
	Me_Forward = H_Forward ;
	Me_Up = H_Up ;
	Me_Right = H_Right ;
	Me_Roll = H_Roll ;
	Me_Pitch = H_Pitch ;
	Me_Heading = H_Heading ;
}
