/*	%Z%%M% %I% %E% SMI	*/


/*
 * This file contains compilation parameters for the game
 *
 * note that all values are MKS units
 *
 * +x is east, +y is north, +z is up.
 */


#ifndef FLIGHT_PARAMETERS
#define	FLIGHT_PARAMETERS


#define	MILES	1609.34		/* convert miles to meters */
#define	NM	1852.		/* convert nautical miles to meters */
#define	FEET	.3048		/* convert feet to meters */
#define	HOURS	3600		/* convert hours to seconds */
#define	PI	3.1415926535
#define	RAD	(3.1415926535/180.)	/* convert degrees to radians */
#define	DEG	(180./3.1415926535)	/* convert radians to degrees */

#define	LATB	37.45		/* lattitude of 0,0 */
#define	LONB	-122.1333	/* longitude of 0,0 */
#define	COSLAT	0.7939		/* cos(LATB) */
#define	RADIUS	(4000*MILES)	/* radius of earth */



	/* note: given that the view angle is +/- 45 degrees, we want
	   the delta vector to accurately reflect the place the player
	   is pointing.  Therefore, we scale by 2*tan(45) = 2.  Change
	   this if the view angle changes */

#define	VIEWSCALE	2.0
#define	VIEWSIZE	0.66		/* fraction of screen used by outside
					   view */

/* control inputs
 */


#define	PITCH_SCALE	(45*RAD)	/* pitch change/sec for full stick */
#define	ROLL_SCALE	(45*RAD)	/* roll change/sec for full stick */




/* autopilot limits
 */

#define	AUTO_RANGE	3000.0		/* distance you can lock up within */
#define	MAX_TURN	0.3		/* max Rate Of Turn		*/
#define	MAX_TURN_ERROR	0.001		/* max R.O.T. error before kicking in */
#define	MAX_SLEW	0.3		/* maximum R.O.T. slew rate	*/
#define	SCAN_TURN	0.1		/* R.O.T. while searching	*/
#define	FEEDBACK_GAIN	2.0		/* controls how tightly we track */
#define	ROLL_RATE	0.3		/* how fast ship rolls		*/


/*
 * time interval for display updates
 */

#define	DISPLAY_INTERVAL	600000		/* microseconds /Pell (for pw & 3/60) */
#define	SLEEP_INTERVAL		30		/* time between updates */


/*
 * default speed
 */

#define	DEFAULT_SPEED	31.29	/* 70 mph = 31.29 m/s */




#define	CLOUD_TRANS	10	/* 10 meters from clear to solid */

#define	VASI_VIS	(3000*3000)	/* visible for 3000 meters */


#define	MAX_TRAIL	360		/* how many old locations to store */


#endif	FLIGHT_PARAMETERS
