#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif



/* This file holds a list of "generic" runways.  The database
 * consists of records in this format:
 *
 *	dd.mm.ss dd.mm.ss elev direction name
 *
 * It is assumed that all "generic" runways are 4000' feet long.
 *
 */


#define	MAX_RUNWAYS	100

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/dir.h>
#include "gfx_mac.h"
#include "flight.h"
#include "object_types.h"
#include "graphics.h"



typedef	struct	{
	  float	x,y ;		/* base position */
	  float	elev ;
	  float	dir ;		/* direction runway faces */
	} Runway ;

static	Runway	runways[MAX_RUNWAYS] ;
static	int	num_runways ;

extern	int	current_color ;


#ifdef	DEBUG
#define	register
#endif	DEBUG


Init_Runways()
{
	int		i,j,n ;
	FILE		*ifile ;
	Point3df	v1,v2 ;		/* vectors for shading */
	int		lad,lam,las, lod,lom,los ;
	int		elev, dir ;
	float		cvd() ;

	num_runways = 0 ;

	ifile = fopen("runways.dat","r") ;
	if( ifile != NULL )
	{
	  while(num_runways < MAX_RUNWAYS)
	  {
	    n = fscanf(ifile,"%d.%d.%d %d.%d.%d %d %d %*s",
		&lad,&lam,&las, &lod,&lom,&los, &elev, &dir) ;
	    if( n != 8 )
	      break ;
	    runways[num_runways].x =
		(cvd(lod,lom,los) - LONB)*(RAD*RADIUS*COSLAT);
	    runways[num_runways].y =
		(cvd(lad,lam,las) - LATB) * (RAD*RADIUS) ;
	    runways[num_runways].elev = elev * FEET ;
	    runways[num_runways].dir = dir * RAD ;
	    ++num_runways ;
	  }
	  fclose(ifile) ;
	}
}



/*
 * top level runway handling, scan through all runways
 * and pass to draw_terrain_node()
 */

Draw_Runways(viewpt, matrix)
	Point3df	*viewpt ;		/* inverse xform of Me_Posn */
	Matrix3df	matrix ;		/* view matrix */
{
	int	i,j ;
static	Point3df Rnwy[4] = {{-10.,-609.6,0.}, {-10., 609.6,0.},
			   { 10., 609.6,0.}, { 10.,-609.6,0.}} ;
static	int	bnds[] = {4} ;
register float		dist,tmp ;
register Runway	*ptr ;
	int		color ;
	Matrix3df	mat ;
	float		sine, cose ;
	float		visib ;

	mat[0][2] = 0.0 ;	mat[0][3] = 0.0 ;
	mat[1][2] = 0.0 ;	mat[1][3] = 0.0 ;
	mat[2][0] = 0.0 ;	mat[2][1] = 0.0 ;
	mat[2][2] = 1.0 ;	mat[2][3] = 0.0 ;
	mat[3][3] = 1.0 ;
	current_color = (Color_Mapping[CITY][0] + Color_Mapping[CITY][1]) / 2 ;
	Set_color(Main_gfx, current_color) ;
	Gfx_select_matrix(Main_gfx, TM1) ;
	visib = vis_sq < (10*10*MILES*MILES) ? vis_sq : (10*10*MILES*MILES) ;
	for(i=0; i<num_runways; ++i)
	{
	  ptr = &runways[i] ;
	  dist = Me_Posn.x - ptr->x ;
	  dist *= dist ;
	  tmp = Me_Posn.y - ptr->y ;
	  dist += tmp*tmp ;
	  if( dist < visib )
	  {
	    sine = sin(ptr->dir) ;
	    cose = cos(ptr->dir) ;
	    mat[0][0] = cose ;	mat[0][1] = -sine ;
	    mat[1][0] = sine ;	mat[1][1] = cose ;
	    mat[3][0] = ptr->x ; mat[3][1] = ptr->y ;
	    mat[3][2] = ptr->elev ;
	    Gfx_set_matrix_3d(Main_gfx, TM1, mat) ;
	    Gfx_matrix_mul_3d(Main_gfx, TM1, TM1, VIEW_MAT) ;
	    if( dist < (3*3*MILES*MILES) )
	    {
	      Gfx_polygon_3df(Main_gfx, 1, bnds, Rnwy, 0 ) ;
	      polygon_trim(Main_gfx,1,bnds,Rnwy) ;
	    }
	    else
	      Gfx_line_3df(Main_gfx, 2, Rnwy) ;
	  }
	}
	Gfx_select_matrix(Main_gfx, VIEW_MAT) ;
}
