#ifndef lint
static	char sccsid[] = "@(#)stars.c 1.2 89/03/15 SMI";
#endif


#define	NUMSTARS	1000		/* up to 2000 stars in memory */
#define	NMAG		3		/* up to magnitude 5 */
#define	EQUINOX		80		/* 80th day of the year */


#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/time.h>
#include "gfx_mac.h"
#include "flight.h"
#include "object_types.h"
#include "graphics.h"


typedef	struct {
	  int ra ;
	  short decl ;
	  short mag ;
	} mstar ;

#ifdef	DEBUG
#define	register
#endif	DEBUG


static	mstar	*Stars ;
static	int	Nstars ;
static	Point3df *Star3d[NMAG] ;
static	int	Nstar3d[NMAG] ;
static	int	MAGLIM[NMAG] = {300,500,9900} ;



/*
 * Initialize star list, search for "stars.dat" files in the current
 * directory.
 */

Init_Stars()
{
	int		i ;
	FILE		*ifile ;
	struct timeval	tv;

	for(i=0; i<NMAG; ++i)
	{
	  Star3d[i] = NULL ;
	  Nstar3d[i] = 0 ;
	}

	ifile = fopen("stars.dat", "r") ;
	if( ifile == NULL )
	  Stars = NULL ;
	else
	{
	  char *str ;
	  char sign ;
	  int hh,mm,ss, dd,m2 ;
	  int ra, decl, mag, n ;
	  register mstar *ptr ;

	  Stars = ptr = (mstar *) malloc( NUMSTARS * sizeof(mstar) ) ;

	  n = fscanf(ifile, "%2d%2d%2d%c%2d%2d%3d%*[^\n]",
		&hh,&mm,&ss,&sign,&dd,&m2,&mag) ;

	  for(Nstars=0;
	      Nstars<NUMSTARS  &&  n == 7;
	      ++Nstars)
	  {
	    ptr->ra = (hh*60+mm)*60+ss ;
	    ptr->decl = dd*60+m2 ; if( sign == '-' ) ptr->decl = -ptr->decl ;
	    ptr->mag = mag ;
	    ++ptr ;
	    n = fscanf(ifile, "%2d%2d%2d%c%2d%2d%3d%*[^\n]",
		  &hh,&mm,&ss,&sign,&dd,&m2,&mag) ;
	  }

	  fclose(ifile) ;
	  gettimeofday(&tv,NULL) ;
	  Shade_stars(tv.tv_sec) ;
	}
}



/*
 * find all the visible stars, put them in Star3d
 * for now, assume lattitude = 37.5 degrees
 * this is all crude approximation anyway
 */

Shade_stars(time)
	long	time ;
{
register int		i,n ;
	struct tm	*tm ;
	int		zenith_ra ;	/* 0 - 86400 */
	int		zenith_decl ;	/* -5400 - 5400 */
	int		yr ;
	int		seconds ;

	for(i=0; i<NMAG; ++i)
	{
	  if( Star3d[i] != NULL )
	  {
	    free(Star3d[i]) ;
	    Star3d[i] = NULL ;
	  }
	  Nstar3d[i] = 0 ;
	}

	tm = localtime(&time) ;

	/* get time of year, relative to vernal equinox (call it Mar 21) */

	if( (yr = tm->tm_yday - EQUINOX) < 0 ) yr += 365 ;
	zenith_ra = (yr*86400)/365 ;

	/* add time of day, relative to noon */

	zenith_ra += (tm->tm_hour*60 + tm->tm_min)*60 + tm->tm_sec - 43200 ;
	if( zenith_ra < 0 ) zenith_ra += 86400 ;
	else if( zenith_ra > 86400 ) zenith_ra -= 86400 ;

	zenith_decl = 2250 ;

	{
	  register mstar *ptr ;
	  register Point3df *ptr3d ;
	  float xfc, xfs ;
	  int max_decl = zenith_decl + 5400 ;
	  int min_decl = zenith_decl - 5400 ;

	  n = 0 ;
	  Star3d[0] = (Point3df *) malloc( Nstars* sizeof(Point3df) ) ;

	  xfc = cos(zenith_decl*.0002909) ;
	  xfs = sin(zenith_decl*.0002909) ;

	  /* convert them to 3-d */

	  ptr = Stars ;
	  ptr3d = Star3d[0] ;
	  for(i=Nstars; --i >= 0;)
	  {
	    if( ptr->decl > min_decl  &&  ptr->decl < max_decl )
	    {
	      register float tmp1, tmp2 ;

	      tmp1 = (float)(ptr->ra - zenith_ra) * 0.000072722 ;
	      tmp2 = (float)ptr->decl * 2.909e-04 ;
	      ptr3d->y = sin(tmp2) * 100. ;
	      tmp2 = cos(tmp2) ;
	      ptr3d->x = tmp2*sin(tmp1) * 100. ;
	      ptr3d->z = tmp2*cos(tmp1) * 100. ;
	      tmp1 = ptr3d->y*xfc - ptr3d->z*xfs ;
	      tmp2 = ptr3d->y*xfs + ptr3d->z*xfc ;
	      ptr3d->y = tmp1 ;
	      ptr3d->z = tmp2 ;
	      if( ptr3d->z > 0.0 )
	      {
		if( ptr->mag > MAGLIM[n] )
		{
		  ++n ;
		  Star3d[n] = ptr3d ;
		}
		++ptr3d ;
		++Nstar3d[n] ;
	      }
	    }
	    ++ptr ;
	  }
	}
}









Draw_Stars()
{
	Matrix3df	matrix, composite ;


	matrix[0][0] = Me_Right.x ;
	matrix[1][0] = Me_Right.y ;
	matrix[2][0] = Me_Right.z ;
	matrix[3][0] = 0.0 ;
	matrix[0][1] = Me_Up.x ;
	matrix[1][1] = Me_Up.y ;
	matrix[2][1] = Me_Up.z ;
	matrix[3][1] = 0.0 ;
	matrix[0][2] = Me_Forward.x ;
	matrix[1][2] = Me_Forward.y ;
	matrix[2][2] = Me_Forward.z ;
	matrix[3][2] = 0.0 ;
	matrix[0][3] = 0.0 ;
	matrix[1][3] = 0.0 ;
	matrix[2][3] = 0.0 ;
	matrix[3][3] = 1.0 ;

	Gfx_mul_matrix( composite, matrix, P_matrix) ;
	Gfx_set_matrix_3d(Main_gfx, VIEW_MAT, composite) ;

	if( Nstar3d[0] > 0 )
	{
	  Set_color(Main_gfx,WHITE) ;
	  Gfx_points_3df(Main_gfx, 0, Nstar3d[0], Star3d[0]) ;
	}

	if( Nstar3d[1] > 0 )
	{
	  Set_color(Main_gfx,LGREY) ;
	  Gfx_points_3df(Main_gfx, 0, Nstar3d[1], Star3d[1]) ;
	}

	if( Nstar3d[2] > 0 )
	{
	  Set_color(Main_gfx,GREY) ;
	  Gfx_points_3df(Main_gfx, 0, Nstar3d[2], Star3d[2]) ;
	}
}
