#ifndef lint
static	char sccsid[] = "@(#)throttle.c 1.1 89/03/10 SMI";
#endif

#define	RAD	(3.1415926535/180.)	/* convert degrees->radians */

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


static	int	update_throttle(), draw_throttle(), input_throttle(),
		fail_throttle() ;


/*
 * this file handles the throttle.
 *
 * failure states:
 *   1 - Throttle has no effect.
 *   2 - Throttle goes to zero.
 */


#define	PSIZE(pts)	(sizeof(pts)/sizeof(Point2df))


static	Matrix2df ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df disp = {
	{1.8,0.},
	{0.,1.8},
	{-.9,-.9}} ;

static	Point2df point = {0.,0.} ;

static	int	last_y ;




Instrument	*
create_throttle(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for airspeed */
{
	Instrument	*ins ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_throttle ;
	ins->update = NULL ;
	ins->fail = fail_throttle ;
	ins->input = input_throttle ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = 0 ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	ins->value = Me_Throttle ;

	return ins ;
}



static
input_throttle(ins,event)
register Instrument	*ins ;
register Event		*event ;
{
	float	newvalue ;

	switch( event_id(event) )
	{
	  case MS_RIGHT:
	    fail_throttle(ins) ;
	    break ;

	  case MS_LEFT:
	    instrument_grabio(ins, event_is_down(event) ) ;
	    if( event_is_up(event) )
	      break ;

	  case LOC_DRAG:
	  case LOC_MOVE:
	    ins->value = 
		(float) (event_y(event) - ins->iy1)/(ins->iy0 - ins->iy1) ;
	    if(ins->value > 1.0 )
		ins->value = 1.0 ;
	    if(ins->value < 0.0 )
		ins->value = 0.0 ;
	    if( ins->fstate == 0 )
		Me_Throttle = ins->value ;
	    break ;
	}
}



static	int
fail_throttle(ins)
register Instrument	*ins ;
{
	ins->fstate = 1 ;
}





static
update_throttle(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;
	float	theta ;



#ifdef	COMMENT
	switch( ins->fstate )
	{
	  case 0:
	    ins->value = Me_Speed ;
	    break ;

	  case 1:
	    break ;

	  case 2:
	    break ;

	  case 3:
	    break ;
	}
#endif	COMMENT

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, disp) ;
	Gfx_select_matrix(gfx, TM1) ;

	Gfx_dbuf_color(gfx,WHITE) ;
	point.x = .25 ; point.y = ins->value ;
	Gfx_move_2df(gfx, &point) ;
	point.x = .75 ;
	Gfx_draw_2df(gfx, &point) ;
}





static
draw_throttle(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;

	update_throttle(gfx,ins) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	Gfx_dbuf_color(gfx,LGREY) ;
	Gfx_line_2df(gfx, trim_size, trim) ;
}
