#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


static	int	update_vguage(), draw_vguage(), input_vguage(), fail_vguage() ;


/*
 * this file draws the vacuum guage
 *
 * failure states:
 *   1 - for 150 frames, slowly decrease value
 *   2 - read 0
 */


#define	PSIZE(pts)	(sizeof(pts)/sizeof(Point2df))


static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df	vgdisp = {
	{.5,0.},
	{0.,.5},
	{.5,.5}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Point2df zero = {0.,0.} ;




Instrument	*
create_vg(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for VG */
{
	Instrument	*ins ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_vguage ;
	ins->update = update_vguage ;
	ins->fail = fail_vguage ;
	ins->input = input_vguage ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_VACUUM ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	ins->value = 5.0 ;
	ins->vec.x = 0.0 ;
	ins->vec.y = .7 ;

	return ins ;
}



static
input_vguage(ins,event)
register Instrument	*ins ;
register Event		*event ;
{
	switch( event_id(event) )
	{
	  case MS_RIGHT:
	    fail_vguage(ins) ;
	    break ;
	}
}



static	int
fail_vguage(ins)
register Instrument	*ins ;
{
	ins->fstate = 1 ;
	ins->fcount = 150 ;
	ins->error_rate = ins->value / 150.0 ;
}





static
update_vguage(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;
	float	theta ;


	/* first, handle error accumulation */

	if( ins->fstate )
	{
	  switch( ins->fstate )
	  {
	    case 1:
	      if( --ins->fcount < 0 )
	      {
		ins->value = 0.0 ;
		++ins->fstate ;
	      }
	      else
	      {
		ins->value -= ins->error_rate ;
	      }
	      theta = (ins->value - 5.0) * 30 * RAD ;
	      ins->vec.x = sin( theta ) *.7 ;
	      ins->vec.y = cos( theta ) *.7 ;
	      break ;

	    case 2:
	      break ;
	  }
	}

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &ins->vec) ;
}





static
draw_vguage(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	update_vguage(gfx,ins) ;
	Gfx_line_2df(gfx, circle_size, circle) ;
}
