#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	MOUSE_SCALE	(.1*RAD)	/* adjustment scale */
#define	MAX_ERROR	(.5*RAD)	/* when we need to adjust error */

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


typedef	struct {
	  int	freq ;
	  char	freq_c[8] ;
	  int	obs ;
	  char	obs_c[4] ;
	  char	obs2_c[4] ;
	  int	gs ;		/* gs needle? */
	  int	vor_type ;	/* type: 0=regular, 1=loc, 2=ILS */
	  Point3df station ;	/* location of station */
	  Point2df vec ;	/* vector to ship from station in W.C. */
	  int	dev ;		/* deviation / localizer course */
	  int	cdi ;
	} Vor_private ;

static	enum	{
	  AD_FREQ,
	  AD_OBS,
	} adjusting ;

static	int	update_vor(), draw_vor(), input_vor(), fail_vor() ;

static	FILE	*vor_file = NULL ;


/*
 * this file draws the VOR
 *
 * failure states:
 *   0 - normal, tracking
 *   1 - no reception
 *   2 - broken
 *
 */


#define	PSIZE(pts)	(sizeof(pts)/sizeof(Point2df))

static	Point2df	vortics[] = {
	  {-.5,.03},{-.5,-.03}, {-.4,.03},{-.4,-.03}, {-.3,.03},{-.3,-.03},
	  {-.2,.03},{-.2,-.03}, {-.1,.03},{-.1,-.03}, { .0,.1},{ .0,-.1},
	  { .1,.03},{ .1,-.03}, { .2,.03},{ .2,-.03}, { .3,.03},{ .3,-.03},
	  { .4,.03},{ .4,-.03}, { .5,.03},{ .5,-.03}} ;

static	Point2df	vertics[] = {
	  {.03,-.5},{-.03,-.5}, {.03,-.4},{-.03,-.4}, {.03,-.3},{-.03,-.3},
	  {.03,-.2},{-.03,-.2}, {.03,-.1},{-.03,-.1}, {.1, .0},{-.1, .0},
	  {.03, .1},{-.03, .1}, {.03, .2},{-.03, .2}, {.03, .3},{-.03, .3},
	  {.03, .4},{-.03, .4}, {.03, .5},{-.03, .5}} ;

static	Point2df	pointer[2] = {{0.,-.4}, {0.,.4}} ;
static	Point2df	gs_pointer[2] = {{-.4,0.}, {.4,0.}} ;

extern	int	debug_level ;

static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;




Instrument	*
create_vor(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored for Vor */
{
	Instrument	*ins ;
	Vor_private	*priv ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_vor ;
	ins->update = update_vor ;
	ins->fail = fail_vor ;
	ins->input = input_vor ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_ELECTRIC ;
	ins->dvalue = 0.0 ;
	ins->fstate = 1 ;
	ins->error = 0.0 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	priv = (Vor_private *) malloc( sizeof(Vor_private) ) ;
	priv->freq = 1090 ;
	strcpy(priv->freq_c, "109.0") ;
	priv->obs = 0 ;
	strcpy(priv->obs_c, "  0") ;
	strcpy(priv->obs2_c, "180") ;
	priv->gs = *line == 'y' ;
	priv->vor_type = 0 ;
	ins->private = (caddr_t) priv ;

	if( vor_file == NULL )
	  vor_file = fopen("vors.dat", "r") ;

	return ins ;
}


static	int	ddy ;

static
input_vor(ins,event,dx,dy)
register Instrument	*ins ;
	Event		*event ;
	int		dx,dy ;
{
register Vor_private	*priv = (Vor_private *) ins->private ;
	int	frac_x, frac_y ;

	switch( event_id(event) )
	{
	  case MS_LEFT:
	    instrument_grabio(ins, event_is_down(event) ) ;
	    if( event_is_down(event))
	    {
	      frac_x = ((event_x(event) - ins->ix0)*256)/(ins->ix1 - ins->ix0) ;
	      frac_y = ((event_y(event) - ins->iy0)*256)/(ins->iy1 - ins->iy0) ;
	      if( frac_x < 100  &&  frac_y > 160 )
		adjusting = AD_FREQ ;
	      else
		adjusting = AD_OBS ;
	    }
	    else
	    {
	      if( adjusting == AD_FREQ )
		set_frequency(ins) ;
	    }
	    break ;

	  case MS_RIGHT:
	    fail_vor(ins) ;
	    break ;

	  case LOC_MOVE:
	  case LOC_DRAG:
	    ddy += dy ; frac_y = ddy/4 ; ddy -= frac_y*4 ;
	    switch( adjusting )
	    {
	      case AD_FREQ:
		priv->freq -= frac_y ;
		if( priv->freq > 1179 )
		  priv->freq = 1179 ;
		else if( priv->freq < 1081 )
		  priv->freq = 1081 ;
		sprintf(priv->freq_c, "%5.1f", priv->freq*.1) ;
		break ;
	      case AD_OBS:
		priv->obs -= frac_y ;
		if( priv->obs > 360 )
		  priv->obs -= 360 ;
		else if( priv->obs < 0 )
		  priv->obs += 360 ;
		sprintf(priv->obs_c, "%3d", priv->obs) ;
		sprintf(priv->obs2_c, "%3d", (priv->obs+180)%360) ;
		break ;
	    }
	    break ;
	}
}



static	int
fail_vor(ins)
register Instrument	*ins ;
{
register Vor_private	*priv = (Vor_private *) ins->private ;

	ins->fstate = 2 ;
}





static
update_vor(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Vor_private	*priv = (Vor_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;
	int	ix,iy,rop ;
	char	*crs_flag ;
	int	gs_dev ;


	switch( ins->fstate )
	{
	  case 0:
	    /* xform station to ships coords to get vector */
	    priv->vec.x = Me_Posn.x - priv->station.x ;
	    priv->vec.y = Me_Posn.y - priv->station.y ;

	    /* shit, do some trig here */
	    ins->value = atan2(priv->vec.x,priv->vec.y) * DEG ;
	    if( priv->vor_type == 0 )
	      priv->cdi = (priv->dev + priv->obs - ins->value) * 10 ;
	    else
	      priv->cdi = (priv->dev - ins->value) * 10 ;

	    if( priv->cdi < -1800 )
	      priv->cdi += 3600 ;
	    else if( priv->cdi > 1800 )
	      priv->cdi -= 3600 ;

	    if( priv->cdi < -950 )
	      crs_flag = "TO", priv->cdi = -1800 - priv->cdi ;
	    else if( priv->cdi < -850 )
	      crs_flag = "OFF" ;
	    else if( priv->cdi < 850 )
	      crs_flag = "FROM" ;
	    else if( priv->cdi < 950 )
	      crs_flag = "OFF" ;
	    else
	      crs_flag = "TO", priv->cdi = 1800 - priv->cdi ;

	    if( priv->vor_type != 0 )
	      priv->cdi *= 4 ;

	    if( priv->cdi < -110 )
	      priv->cdi = -110 ;
	    else if( priv->cdi > 110 )
	      priv->cdi = 110 ;

	    if( priv->gs )
	      if( priv->vor_type == 2 )
	      {
		register float dist, d2 ;

		dist = Me_Posn.x - priv->station.x ;
		d2 = Me_Posn.y - priv->station.y ;
		dist = sqrt(dist*dist+d2*d2) ;
		d2 = Me_Posn.z - priv->station.z ;

		d2 = atan2(d2,dist) * DEG ;
		gs_dev = (3 - d2) * 40 ;		/* three degree GS */

		if( gs_dev < -110 )
		  gs_dev = -110 ;
		else if( gs_dev > 110 )
		  gs_dev = 110 ;
	      }
	      else
		gs_dev = 0 ;
	    break ;
	  
	  case 1:
	  case 2:
	    priv->cdi = 0 ;
	    gs_dev = 0 ;
	    crs_flag = "OFF" ;
	    break ;
	}


	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;


	/* draw circle */

	Gfx_dbuf_color(gfx,BLACK) ;
	Gfx_polygon_2df(gfx, 1, circle_bnds, circle, GFX_SHADE_CONSTANT) ;

	Gfx_flush(Main_gfx,1) ;

	/* draw frequency */

	ix = (ins->ix0*3 + ins->ix1)/4 ;
	iy = (ins->iy0 + ins->iy1*3)/4 ;
	rop = Gfx_dbuf_get_color(gfx,WHITE) ;
	rop = PIX_SRC | PIX_COLOR(rop) ; /* Pell */
	pw_ttext(Main_pw, ix,ins->iy1, rop, Main_font, priv->freq_c) ; /* Pell */


	/* draw obs setting */

	ix = (ins->ix0 + ins->ix1)/2 ;
	iy = (ins->iy0*3 + ins->iy1)/4 ;
	pw_ttext(Main_pw, ix-8,iy, rop, Main_font, priv->obs_c) ; /* Pell */

	ix = (ins->ix0 + ins->ix1)/2 ;
	iy = (ins->iy0 + ins->iy1*7)/8 ;
	pw_ttext(Main_pw, ix-8,iy, rop, Small_font, priv->obs2_c) ; /* Pell */


	/* draw to/from */

	ix = (ins->ix0 + ins->ix1*3)/4 ;
	iy = (ins->iy0 + ins->iy1*3)/4 ;
	pw_ttext(Main_pw, ix,iy, rop, Main_font, crs_flag) ; /* Pell */


	/* draw tic marks */

	Gfx_dbuf_color(gfx,WHITE) ;
	ptr = vortics ;
	for(i=11; i > 0;)
	{
	  Gfx_vec_init_2df(gfx, i*2, fit) ;
	  fit /= 2 ;
	  i -= fit ;
	  while(--fit >= 0)
	  {
	    Gfx_vec_move_2df(gfx, ptr) ; ++ptr ;
	    Gfx_vec_draw_2df(gfx, ptr) ; ++ptr ;
	  }
	  Gfx_vec_term_2df(gfx) ;
	}

	/* draw pointer */

	pointer[0].x = pointer[1].x = priv->cdi * .005 ;
	Gfx_vec_init_2df(gfx, 2, fit) ;
	Gfx_vec_move_2df(gfx, pointer) ;
	Gfx_vec_draw_2df(gfx, pointer+1) ;
	Gfx_vec_term_2df(gfx) ;


	if( priv->gs )
	{
	  /* draw vertical tic marks */

	  Gfx_dbuf_color(gfx,WHITE) ;
	  ptr = vertics ;
	  for(i=11; i > 0;)
	  {
	    Gfx_vec_init_2df(gfx, i*2, fit) ;
	    fit /= 2 ;
	    i -= fit ;
	    while(--fit >= 0)
	    {
	      Gfx_vec_move_2df(gfx, ptr) ; ++ptr ;
	      Gfx_vec_draw_2df(gfx, ptr) ; ++ptr ;
	    }
	    Gfx_vec_term_2df(gfx) ;
	  }

	  /* draw pointer */

	  gs_pointer[0].y = gs_pointer[1].y = gs_dev * .005 ;
	  Gfx_vec_init_2df(gfx, 2, fit) ;
	  Gfx_vec_move_2df(gfx, gs_pointer) ;
	  Gfx_vec_draw_2df(gfx, gs_pointer+1) ;
	  Gfx_vec_term_2df(gfx) ;
	}
}





static
draw_vor(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register Vor_private	*priv = (Vor_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;



	update_vor(gfx,ins) ;

#ifdef	COMMENT
	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;
#endif	COMMENT

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	/* draw bolts   JPD 10.28.88 */

	Gfx_dbuf_color(gfx,LGREY) ;
        Gfx_polygon_2df(gfx, 1, uplcir_bnds, uplcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, uprcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowlcir, GFX_SHADE_CONSTANT) ;
	Gfx_polygon_2df(gfx, 1, uprcir_bnds, lowrcir, GFX_SHADE_CONSTANT) ;

	/* draw trim */

	Gfx_dbuf_color(gfx,LGREY) ;
	Gfx_line_2df(gfx, trim_size, trim) ;
}



static
set_frequency(ins)
register Instrument	*ins ;
{
register Vor_private	*priv = (Vor_private *) ins->private ;
	int		freq ;
	int		vor_type, dev ;
	int		lad,lam,las, lod,lom,los, elev ;
	float		cvd() ;
	int		n ;

	if( ins->fstate == 2 )
	  return ;

	ins->fstate = 1 ;		/* no reception */
	if( vor_file != NULL )
	{
	  for(;;)
	  {
	    n = fscanf(vor_file, "%d %d.%d.%d %d.%d.%d %d %d %d %*s %*s",
		&freq, &lad,&lam,&las, &lod,&lom,&los, &elev, &vor_type, &dev) ;
	    if( n != 10 )
	      break ;
	    if( freq == priv->freq )
	    {
	      ins->fstate = 0 ;
	      priv->vor_type = vor_type ;
	      priv->station.x = (cvd(lod,lom,los) - LONB)*(RAD*RADIUS*COSLAT) ;
	      priv->station.y = (cvd(lad,lam,las) - LATB) * (RAD*RADIUS) ;
	      priv->station.z = elev*FEET ;
	      priv->dev = dev ;
	      if( vor_type != 0 )
	      {
		/* offset localizers and ILSs 2000' to put them at end of
		   runway */
		priv->station.x -= sin(dev*RAD) * 2000*FEET ;
		priv->station.y -= cos(dev*RAD) * 2000*FEET ;
	      }
	      break ;
	    }
	  }
	  rewind( vor_file ) ;
	}
}



vor_map(gfx, vor)
register Gfx_context	gfx ;
register Instrument	*vor ;
{
register Vor_private	*priv = (Vor_private *) vor->private ;
	Point3df	p1,p2 ;
	float		theta,dx,dy ;

	if( vor->fstate == 0 )
	{
	  if( priv->vor_type == 0 )
	    theta = priv->dev + priv->obs ;
	  else
	    theta = priv->dev ;
	  theta *= RAD ;
	  dx = 100000. * sin(theta) ;
	  dy = 100000. * cos(theta) ;

	  p1.x = priv->station.x + dx ;
	  p1.y = priv->station.y + dy ;
	  p1.z = 0. ;

	  p2.x = priv->station.x - dx ;
	  p2.y = priv->station.y - dy ;
	  p2.z = 0. ;

	  Gfx_dbuf_color(gfx,BLUE) ;
	  Gfx_move_3df(gfx, &p1) ;
	  Gfx_draw_3df(gfx, &p2) ;
	}
}
