#ifndef lint
static	char sccsid[] = "%Z%%M% %I% %E% SMI";
#endif

#define	SCALE	197.			/* convert m/s to ft/min */
#define	LEAK	7			/* ~7 second lag */
#define	V2THETA(v)	((v) * (PI * SCALE / MAX_FPM))

#define	MAX_ERROR	(10./SCALE)	/* max error = 10 ft/min */
#define	MAX_FPM		2500		/* top of scale (180 deg to right) */

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <math.h>
#include "gfx_mac.h"
#include "flight.h"
#include "graphics.h"
#include "instrument.h"
#include "trim.h"


typedef	struct {
	  float old_alt ;		/* old altitude */
	} vsi_private ;

static	int	update_vsi(), draw_vsi(), input_vsi(), fail_vsi() ;


/*
 * this file draws the vacuum guage
 *
 * failure states:
 *   1 - static blocked, heading for zero
 *   2 - frozen at zero.
 */


#define	PSIZE(pts)	(sizeof(pts)/sizeof(Point2df))

#ifdef	DEBUG
#define	register
#endif	DEBUG

static	Matrix2df	ident2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Matrix2df	disp = {
	{.5,0.},
	{0.,.5},
	{.5,.5}} ;

static	Matrix2df mat2d = {
	{1.,0.},
	{0.,1.},
	{0.,0.}} ;

static	Point2df zero = {0.,0.} ;

static	Point2df vsi_tics[21][2] ;


Instrument	*
create_vsi(x,y,w,h, line)
	float	x,y,w,h ;
	char	*line ;			/* ignored */
{
	Instrument	*ins ;
	vsi_private	*priv ;
	float		theta ;
	int		i ;
	Point2df	*ptr ;

	ins = (Instrument *) malloc( sizeof(Instrument) ) ;

	ins->redraw = draw_vsi ;
	ins->update = update_vsi ;
	ins->fail = fail_vsi ;
	ins->input = input_vsi ;
	ins->x0 = x ;
	ins->y0 = y ;
	ins->x1 = x+w ;
	ins->y1 = y+h ;
	ins->flags = INS_STATIC ;
	ins->fstate = 0 ;
	ins->error = 0.0 ;
	ins->error_rate = 0.0 ;
	ins->next = NULL ;

	priv = (vsi_private *) malloc( sizeof(vsi_private) ) ;
	ins->private = (caddr_t) priv ;
	ins->dvalue = ins->value = Me_Speed*Me_Forward.z ;
	theta = V2THETA(ins->dvalue) ;
	ins->vec.x = -cos(theta) * .7 ;
	ins->vec.y = sin(theta) * .7 ;
	priv->old_alt = Me_Posn.z ;

	/* compute tic marks, if there's more than one VSI
	   (unlikely), this will be done more than once; no big deal */

	ptr = &vsi_tics[0][0] ;
	for(i = -1000; i<=1000; i+=100 )
	{
	  theta = V2THETA(i/SCALE) ;
	  ptr->x = -cos(theta) * .9 ;
	  ptr->y = sin(theta) * .9 ;
	  ++ptr ;
	  if( i%500 == 0 )
	    ptr->x = -cos(theta) * .8,
	    ptr->y = sin(theta) * .8 ;
	  else
	    ptr->x = -cos(theta) * .85,
	    ptr->y = sin(theta) * .85 ;
	  ++ptr ;
	}

	return ins ;
}



static
input_vsi(ins,event)
register Instrument	*ins ;
register Event		*event ;
{
	switch( event_id(event) )
	{
	  case MS_RIGHT:
	    fail_vsi(ins) ;
	    break ;
	}
}



static	int
fail_vsi(ins)
register Instrument	*ins ;
{
	ins->fstate = 1 ;
	ins->fcount = 150 ;
	ins->error_rate = ins->value/150.0 ;
}





static
update_vsi(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register vsi_private	*priv = (vsi_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;


	/* first, handle error accumulation */

	switch( ins->fstate )
	{
	  case 0:
	    ins->value = Me_Posn.z - priv->old_alt ;
	    priv->old_alt += ins->value*(1.0/LEAK)*Dtime ;
	    break ;

	  case 1:
	    if( --ins->fcount < 0 )
	    {
	      ins->value = 0.0 ;
	      ++ins->fstate ;
	    }
	    else
	      ins->value -= ins->error_rate ;
	    break ;

	  case 2:
	    break ;
	}

	if( ins->value - ins->dvalue > MAX_ERROR  ||
	    ins->value - ins->dvalue < -MAX_ERROR )
	{
	  register float theta ;
	  ins->dvalue = ins->value ;
	  theta = ins->dvalue * ((1.0/LEAK) * SCALE * (1.0/MAX_FPM) * M_PI) ;
	  ins->vec.x = -cos(theta) * .7 ;
	  ins->vec.y = sin(theta) * .7 ;
	}

	Gfx_viewport_2d(gfx, ins->x0,ins->x1, ins->y0,ins->y1 ) ;
	Gfx_set_clip_planes(gfx,0) ;

	Gfx_set_matrix_2d(gfx, TM1, ident2d) ;
	Gfx_select_matrix(gfx, TM1) ;

	Gfx_dbuf_color(gfx,WHITE) ;
	Gfx_move_2df(gfx, &zero) ;
	Gfx_draw_2df(gfx, &ins->vec) ;
}





static
draw_vsi(gfx,ins)
register Gfx_context	gfx ;
register Instrument	*ins ;
{
register vsi_private	*priv = (vsi_private *) ins->private ;
	int	i,j ;
register Point2df	*ptr ;
	int	fit ;

	update_vsi(gfx,ins) ;
	bolts_and_trim(gfx,1,21,vsi_tics) ;
}
